

package org.eso.oca.data;


/**
 * Class DidParametersDAO
 *
 *  Handles database calls for the did_parameters table.
 * 
 * @version $Revision: 1.6 $
 * @author pnunes, Apr 22, 2005
 */

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;

import org.apache.log4j.Logger;

/**
 * Handles database calls for the did_parameters table.
 */
public class DidParametersDAO {
    
    private static Logger logger = Logger.getLogger(DidParametersDAO.class);

    /**
     * Column code of type Types.NUMERIC mapped to Long.
     */
    public static final int ID_CODE = 0;
    public static final int TYPE_CODE = Types.NUMERIC;
    public static final String NAME_CODE = "code";

    /**
     * Column parameter of type Types.VARCHAR mapped to String.
     */
    public static final int ID_PARAMETER = 1;
    public static final int TYPE_PARAMETER = Types.VARCHAR;
    public static final String NAME_PARAMETER = "parameter";

    /**
     * Column zclass of type Types.VARCHAR mapped to String.
     */
    public static final int ID_ZCLASS = 2;
    public static final int TYPE_ZCLASS = Types.VARCHAR;
    public static final String NAME_ZCLASS = "zclass";

    /**
     * Column context of type Types.VARCHAR mapped to String.
     */
    public static final int ID_CONTEXT = 3;
    public static final int TYPE_CONTEXT = Types.VARCHAR;
    public static final String NAME_CONTEXT = "context";

    /**
     * Column type of type Types.VARCHAR mapped to String.
     */
    public static final int ID_TYPE = 4;
    public static final int TYPE_TYPE = Types.VARCHAR;
    public static final String NAME_TYPE = "type";

    /**
     * Column format of type Types.VARCHAR mapped to String.
     */
    public static final int ID_FORMAT = 5;
    public static final int TYPE_FORMAT = Types.VARCHAR;
    public static final String NAME_FORMAT = "format";

    /**
     * Column unit of type Types.VARCHAR mapped to String.
     */
    public static final int ID_UNIT = 6;
    public static final int TYPE_UNIT = Types.VARCHAR;
    public static final String NAME_UNIT = "unit";

    /**
     * Column comment of type Types.VARCHAR mapped to String.
     */
    public static final int ID_COMMENT = 7;
    public static final int TYPE_COMMENT = Types.VARCHAR;
    public static final String NAME_COMMENT = "comment";

    /**
     * Column description of type Types.LONGVARCHAR mapped to String.
     */
    public static final int ID_DESCRIPTION = 8;
    public static final int TYPE_DESCRIPTION = Types.LONGVARCHAR;
    public static final String NAME_DESCRIPTION = "description";

    /**
     * Column range of type Types.VARCHAR mapped to String.
     */
    public static final int ID_RANGE = 9;
    public static final int TYPE_RANGE = Types.VARCHAR;
    public static final String NAME_RANGE = "range";

    /**
     * Column default_value of type Types.VARCHAR mapped to String.
     */
    public static final int ID_DEFAULT_VALUE = 10;
    public static final int TYPE_DEFAULT_VALUE = Types.VARCHAR;
    public static final String NAME_DEFAULT_VALUE = "default_value";

    /**
     * Column mandatory of type Types.TINYINT mapped to Integer.
     */
    public static final int ID_MANDATORY = 11;
    public static final int TYPE_MANDATORY = Types.TINYINT;
    public static final String NAME_MANDATORY = "mandatory";

    /**
     * Column zprotected of type Types.TINYINT mapped to Integer.
     */
    public static final int ID_ZPROTECTED = 12;
    public static final int TYPE_ZPROTECTED = Types.TINYINT;
    public static final String NAME_ZPROTECTED = "zprotected";


    private static final String TABLE_NAME = "did_parameters";

    /**
     * Create an array of type string containing all the fields of the did_parameters table.
     */
    private static final String[] FIELD_NAMES = 
    {
        "did_parameters.code"
        ,"did_parameters.parameter"
        ,"did_parameters.zclass"
        ,"did_parameters.context"
        ,"did_parameters.type"
        ,"did_parameters.format"
        ,"did_parameters.unit"
        ,"did_parameters.comment"
        ,"did_parameters.description"
        ,"did_parameters.range"
        ,"did_parameters.default_value"
        ,"did_parameters.mandatory"
        ,"did_parameters.zprotected"
    };

    /**
     * Field that contains the comma separated fields of the did_parameters table.
     */
    private static final String ALL_FIELDS = "did_parameters.code"
                            + ",did_parameters.parameter"
                            + ",did_parameters.zclass"
                            + ",did_parameters.context"
                            + ",did_parameters.type"
                            + ",did_parameters.format"
                            + ",did_parameters.unit"
                            + ",did_parameters.comment"
                            + ",did_parameters.description"
                            + ",did_parameters.range"
                            + ",did_parameters.default_value"
                            + ",did_parameters.mandatory"
                            + ",did_parameters.zprotected";

    private static DidParametersDAO singleton = new DidParametersDAO();

    /**
     * Get the DidParametersManager singleton.
     *
     * @return DidParametersManager 
     */
    synchronized public static DidParametersDAO getInstance()
    {
        return singleton;
    }

    /**
     * Sets your own DidParametersManager instance.
     <br>
     * This is optional, by default we provide it for you.
     */
    synchronized public static void setInstance(DidParametersDAO instance)
    {
        singleton = instance;
    }


    /**
     * Creates a new DidParametersBean instance.
     *
     * @return the new DidParametersBean 
     */
    public DidParameter createDidParametersBean()
    {
        return new DidParameter();
    }



    //////////////////////////////////////
    // LOAD ALL
    //////////////////////////////////////

    /**
     * Loads all the rows from did_parameters.
     *
     * @return an array of DidParametersManager pObject
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //38
    public DidParameter[] loadAll() throws SQLException, JdbcConnectionException 
    {
        Connection c = null;
        PreparedStatement ps = null;
        try 
        {
            c = getConnection();
            ps = c.prepareStatement("SELECT " + ALL_FIELDS + " FROM did_parameters",ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            return loadByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }

    //////////////////////////////////////
    // SQL 'WHERE' METHOD
    //////////////////////////////////////
    /**
     * Retrieves an array of DidParametersBean given a sql 'where' clause.
     *
     * @param where the sql 'where' clause
     * @return the resulting DidParametersBean table 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //49
    public DidParameter[] loadByWhere(String where) throws SQLException, JdbcConnectionException
    {
        return loadByWhere(where, null);
    }

    /**
     * Retrieves an array of DidParametersBean given a sql where clause, and a list of fields.
     * It is up to you to pass the 'WHERE' in your where clausis.
     *
     * @param where the sql 'where' clause
     * @param fieldList table of the field's associated constants
     * @return the resulting DidParametersBean table 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //51
    public DidParameter[] loadByWhere(String where, int[] fieldList) throws SQLException, JdbcConnectionException
    {
        String sql = null;
        if(fieldList == null)
            sql = "select * from did_parameters " + where;
        else
        {
            StringBuffer buff = new StringBuffer(128);
            buff.append("select ");
            for(int i = 0; i < fieldList.length; i++)
            {
                if(i != 0)
                    buff.append(",");
                buff.append(FIELD_NAMES[fieldList[i]]);
            }
            buff.append(" from did_parameters ");
            buff.append(where);
            sql = buff.toString();
            buff = null;
        }
        Connection c = null;
        Statement pStatement = null;
        ResultSet rs =  null;
        java.util.ArrayList v = null;
        logger.debug(sql);
        try 
        {
            c = getConnection();
            pStatement = c.createStatement();
            rs =  pStatement.executeQuery(sql);
            v = new java.util.ArrayList();
            while(rs.next())
            {
                if(fieldList == null)
                    v.add(decodeRow(rs));
                else
                    v.add(decodeRow(rs, fieldList));
            }

            return (DidParameter[])v.toArray(new DidParameter[0]);
        }
        finally
        {
            if (v != null) { v.clear(); }
            getManager().close(pStatement, rs);
            freeConnection(c);
        }
    }


    /**
     * Deletes all rows from did_parameters table.
     * @return the number of deleted rows.
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public int deleteAll() throws SQLException, JdbcConnectionException
    {
        return deleteByWhere("");
    }


    /**
     * Deletes rows from the did_parameters table using a 'where' clause.
     * It is up to you to pass the 'WHERE' in your where clausis.
     * <br>Attention, if 'WHERE' is omitted it will delete all records. 
     *
     * @param where the sql 'where' clause
     * @return the number of deleted rows
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public int deleteByWhere(String where) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;

        try
        {
            c = getConnection();
            String delByWhereSQL = "DELETE FROM did_parameters " + where;
            ps = c.prepareStatement(delByWhereSQL);
            return ps.executeUpdate();
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // SAVE 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Saves the DidParametersBean pObject into the database.
     *
     * @param pObject the DidParametersBean pObject to be saved
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //100
    public DidParameter save(DidParameter pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer _sql = null;

        try
        {
            c = getConnection();
            if (pObject.isNew())
            { // SAVE 
                beforeInsert(pObject); // listener callback
                int _dirtyCount = 0;
                _sql = new StringBuffer("INSERT into did_parameters (");
    
                if (pObject.isCodeModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("code");
                    _dirtyCount++;
                }

                if (pObject.isParameterModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("parameter");
                    _dirtyCount++;
                }

                if (pObject.isZclassModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("zclass");
                    _dirtyCount++;
                }

                if (pObject.isContextModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("context");
                    _dirtyCount++;
                }

                if (pObject.isTypeModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("type");
                    _dirtyCount++;
                }

                if (pObject.isFormatModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("format");
                    _dirtyCount++;
                }

                if (pObject.isUnitModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("unit");
                    _dirtyCount++;
                }

                if (pObject.isCommentModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("comment");
                    _dirtyCount++;
                }

                if (pObject.isDescriptionModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("description");
                    _dirtyCount++;
                }

                if (pObject.isRangeModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("range");
                    _dirtyCount++;
                }

                if (pObject.isDefaultValueModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("default_value");
                    _dirtyCount++;
                }

                if (pObject.isMandatoryModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("mandatory");
                    _dirtyCount++;
                }

                if (pObject.isZprotectedModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("zprotected");
                    _dirtyCount++;
                }

                _sql.append(") values (");
                if(_dirtyCount > 0) {
                    _sql.append("?");
                    for(int i = 1; i < _dirtyCount; i++) {
                        _sql.append(",?");
                    }
                }
                _sql.append(")");

                ps = c.prepareStatement(_sql.toString(), Statement.RETURN_GENERATED_KEYS);
                _dirtyCount = 0;

                if (pObject.isCodeModified()) {
                    DbConnManager.setLong(ps, ++_dirtyCount, pObject.getCode());
                }
    
                if (pObject.isParameterModified()) {
                    ps.setString(++_dirtyCount, pObject.getParameter());
                }
    
                if (pObject.isZclassModified()) {
                    ps.setString(++_dirtyCount, pObject.getZclass());
                }
    
                if (pObject.isContextModified()) {
                    ps.setString(++_dirtyCount, pObject.getContext());
                }
    
                if (pObject.isTypeModified()) {
                    ps.setString(++_dirtyCount, pObject.getType());
                }
    
                if (pObject.isFormatModified()) {
                    ps.setString(++_dirtyCount, pObject.getFormat());
                }
    
                if (pObject.isUnitModified()) {
                    ps.setString(++_dirtyCount, pObject.getUnit());
                }
    
                if (pObject.isCommentModified()) {
                    ps.setString(++_dirtyCount, pObject.getComment());
                }
    
                if (pObject.isDescriptionModified()) {
                    ps.setString(++_dirtyCount, pObject.getDescription());
                }
    
                if (pObject.isRangeModified()) {
                    ps.setString(++_dirtyCount, pObject.getRange());
                }
    
                if (pObject.isDefaultValueModified()) {
                    ps.setString(++_dirtyCount, pObject.getDefaultValue());
                }
    
                if (pObject.isMandatoryModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getMandatory());
                }
    
                if (pObject.isZprotectedModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getZprotected());
                }
    
                ps.executeUpdate();
    
                pObject.isNew(false);
                pObject.resetIsModified();
                afterInsert(pObject); // listener callback
            }
            else 
            { // UPDATE 
                beforeUpdate(pObject); // listener callback
                _sql = new StringBuffer("UPDATE did_parameters SET ");
                boolean useComma=false;

                if (pObject.isCodeModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("code").append("=?");
                }

                if (pObject.isParameterModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("parameter").append("=?");
                }

                if (pObject.isZclassModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("zclass").append("=?");
                }

                if (pObject.isContextModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("context").append("=?");
                }

                if (pObject.isTypeModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("type").append("=?");
                }

                if (pObject.isFormatModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("format").append("=?");
                }

                if (pObject.isUnitModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("unit").append("=?");
                }

                if (pObject.isCommentModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("comment").append("=?");
                }

                if (pObject.isDescriptionModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("description").append("=?");
                }

                if (pObject.isRangeModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("range").append("=?");
                }

                if (pObject.isDefaultValueModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("default_value").append("=?");
                }

                if (pObject.isMandatoryModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("mandatory").append("=?");
                }

                if (pObject.isZprotectedModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("zprotected").append("=?");
                }
                _sql.append("");
                ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                int _dirtyCount = 0;

                if (pObject.isCodeModified()) {
                    DbConnManager.setLong(ps, ++_dirtyCount, pObject.getCode());
                }

                if (pObject.isParameterModified()) {
                      ps.setString(++_dirtyCount, pObject.getParameter());
                }

                if (pObject.isZclassModified()) {
                      ps.setString(++_dirtyCount, pObject.getZclass());
                }

                if (pObject.isContextModified()) {
                      ps.setString(++_dirtyCount, pObject.getContext());
                }

                if (pObject.isTypeModified()) {
                      ps.setString(++_dirtyCount, pObject.getType());
                }

                if (pObject.isFormatModified()) {
                      ps.setString(++_dirtyCount, pObject.getFormat());
                }

                if (pObject.isUnitModified()) {
                      ps.setString(++_dirtyCount, pObject.getUnit());
                }

                if (pObject.isCommentModified()) {
                      ps.setString(++_dirtyCount, pObject.getComment());
                }

                if (pObject.isDescriptionModified()) {
                      ps.setString(++_dirtyCount, pObject.getDescription());
                }

                if (pObject.isRangeModified()) {
                      ps.setString(++_dirtyCount, pObject.getRange());
                }

                if (pObject.isDefaultValueModified()) {
                      ps.setString(++_dirtyCount, pObject.getDefaultValue());
                }

                if (pObject.isMandatoryModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getMandatory());
                }

                if (pObject.isZprotectedModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getZprotected());
                }
    
                if (_dirtyCount == 0) {
                     return pObject;
                }
    
                ps.executeUpdate();
                pObject.resetIsModified();
                afterUpdate(pObject); // listener callback
            }
    
            return pObject;
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    /**
     * Saves an array of DidParametersBean pObjects into the database.
     *
     * @param pObjects the DidParametersBean pObject table to be saved
     * @return the saved DidParametersBean array.
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //65
    public DidParameter[] save(DidParameter[] pObjects) throws SQLException, JdbcConnectionException 
    {
        for (int iIndex = 0; iIndex < pObjects.length; iIndex ++){
            save(pObjects[iIndex]);
        }
        return pObjects;
    }



    ///////////////////////////////////////////////////////////////////////
    // USING TEMPLATE 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Loads a unique DidParametersBean pObject from a template one giving a c
     *
     * @param pObject the DidParametersBean pObject to look for
     * @return the pObject matching the template
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //85
    public DidParameter loadUniqueUsingTemplate(DidParameter pObject) throws SQLException, JdbcConnectionException
    {
         DidParameter[] pReturn = loadUsingTemplate(pObject);
         if (pReturn.length == 0)
             return null;
         if (pReturn.length > 1)
             throw new SQLException("More than one element !!");
         return pReturn[0];
     }

    /**
     * Loads an array of DidParametersBean from a template one.
     *
     * @param pObject the DidParametersBean template to look for
     * @return all the DidParametersBean matching the template
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //88
    public DidParameter[] loadUsingTemplate(DidParameter pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer where = new StringBuffer("");
        StringBuffer _sql = new StringBuffer("SELECT " + ALL_FIELDS + " from did_parameters WHERE ");
        StringBuffer _sqlWhere = new StringBuffer("");
        try
        {
            int _dirtyCount = 0;
    
             if (pObject.isCodeModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("code= ?");
             }
    
             if (pObject.isParameterModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("parameter= ?");
             }
    
             if (pObject.isZclassModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("zclass= ?");
             }
    
             if (pObject.isContextModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("context= ?");
             }
    
             if (pObject.isTypeModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("type= ?");
             }
    
             if (pObject.isFormatModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("format= ?");
             }
    
             if (pObject.isUnitModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("unit= ?");
             }
    
             if (pObject.isCommentModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("comment= ?");
             }
    
             if (pObject.isDescriptionModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("description= ?");
             }
    
             if (pObject.isRangeModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("range= ?");
             }
    
             if (pObject.isDefaultValueModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("default_value= ?");
             }
    
             if (pObject.isMandatoryModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("mandatory= ?");
             }
    
             if (pObject.isZprotectedModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("zprotected= ?");
             }
    
             if (_dirtyCount == 0) {
                 throw new SQLException ("The pObject to look for is invalid : not initialized !");
             }
             _sql.append(_sqlWhere);
             c = getConnection();
             ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
             _dirtyCount = 0;
    
             if (pObject.isCodeModified()) {
                 DbConnManager.setLong(ps, ++_dirtyCount, pObject.getCode());
             }
    
             if (pObject.isParameterModified()) {
                 ps.setString(++_dirtyCount, pObject.getParameter());
             }
    
             if (pObject.isZclassModified()) {
                 ps.setString(++_dirtyCount, pObject.getZclass());
             }
    
             if (pObject.isContextModified()) {
                 ps.setString(++_dirtyCount, pObject.getContext());
             }
    
             if (pObject.isTypeModified()) {
                 ps.setString(++_dirtyCount, pObject.getType());
             }
    
             if (pObject.isFormatModified()) {
                 ps.setString(++_dirtyCount, pObject.getFormat());
             }
    
             if (pObject.isUnitModified()) {
                 ps.setString(++_dirtyCount, pObject.getUnit());
             }
    
             if (pObject.isCommentModified()) {
                 ps.setString(++_dirtyCount, pObject.getComment());
             }
    
             if (pObject.isDescriptionModified()) {
                 ps.setString(++_dirtyCount, pObject.getDescription());
             }
    
             if (pObject.isRangeModified()) {
                 ps.setString(++_dirtyCount, pObject.getRange());
             }
    
             if (pObject.isDefaultValueModified()) {
                 ps.setString(++_dirtyCount, pObject.getDefaultValue());
             }
    
             if (pObject.isMandatoryModified()) {
                 DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getMandatory());
             }
    
             if (pObject.isZprotectedModified()) {
                 DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getZprotected());
             }
    
             ps.executeQuery();
             return loadByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }
    /**
     * Deletes rows using a DidParametersBean template.
     *
     * @param pObject the DidParametersBean object(s) to be deleted
     * @return the number of deleted objects
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //63
    public int deleteUsingTemplate(DidParameter pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer sql = null;
    
        try 
        {
            sql = new StringBuffer("DELETE FROM did_parameters WHERE ");
            int _dirtyAnd = 0;
            if (pObject.isCodeInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("code").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isParameterInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("parameter").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isZclassInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("zclass").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isContextInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("context").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isTypeInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("type").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isFormatInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("format").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isUnitInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("unit").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isCommentInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("comment").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isDescriptionInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("description").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isRangeInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("range").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isDefaultValueInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("default_value").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isMandatoryInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("mandatory").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isZprotectedInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("zprotected").append("=?");
                _dirtyAnd ++;
            }
    
            c = getConnection();
            ps = c.prepareStatement(sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            int _dirtyCount = 0;
    
            if (pObject.isCodeInitialized()) {
                DbConnManager.setLong(ps, ++_dirtyCount, pObject.getCode());
            }
    
            if (pObject.isParameterInitialized()) {
                ps.setString(++_dirtyCount, pObject.getParameter());
            }
    
            if (pObject.isZclassInitialized()) {
                ps.setString(++_dirtyCount, pObject.getZclass());
            }
    
            if (pObject.isContextInitialized()) {
                ps.setString(++_dirtyCount, pObject.getContext());
            }
    
            if (pObject.isTypeInitialized()) {
                ps.setString(++_dirtyCount, pObject.getType());
            }
    
            if (pObject.isFormatInitialized()) {
                ps.setString(++_dirtyCount, pObject.getFormat());
            }
    
            if (pObject.isUnitInitialized()) {
                ps.setString(++_dirtyCount, pObject.getUnit());
            }
    
            if (pObject.isCommentInitialized()) {
                ps.setString(++_dirtyCount, pObject.getComment());
            }
    
            if (pObject.isDescriptionInitialized()) {
                ps.setString(++_dirtyCount, pObject.getDescription());
            }
    
            if (pObject.isRangeInitialized()) {
                ps.setString(++_dirtyCount, pObject.getRange());
            }
    
            if (pObject.isDefaultValueInitialized()) {
                ps.setString(++_dirtyCount, pObject.getDefaultValue());
            }
    
            if (pObject.isMandatoryInitialized()) {
                DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getMandatory());
            }
    
            if (pObject.isZprotectedInitialized()) {
                DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getZprotected());
            }
    
            int _rows = ps.executeUpdate();
            return _rows;
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // COUNT 
    ///////////////////////////////////////////////////////////////////////

    /**
     * Retrieves the number of rows of the table did_parameters.
     *
     * @return the number of rows returned
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //78
    public int countAll() throws SQLException, JdbcConnectionException
    {
        return countWhere("");
    }



    /**
     * Retrieves the number of rows of the table did_parameters with a 'where' clause.
     * It is up to you to pass the 'WHERE' in your where clausis.
     *
     * @param where the restriction clause
     * @return the number of rows returned
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    public int countWhere(String where) throws SQLException, JdbcConnectionException
    {
        String sql = "select count(*) as MCOUNT from did_parameters " + where;
        Connection c = null;
        Statement pStatement = null;
        ResultSet rs =  null;
        logger.debug(sql);
        try 
        {
            int iReturn = -1;    
            c = getConnection();
            pStatement = c.createStatement();
            rs =  pStatement.executeQuery(sql);
            if (rs.next())
            {
                iReturn = rs.getInt("MCOUNT");
            }
            if (iReturn != -1)
                return iReturn;
        }
        finally
        {
            getManager().close(pStatement, rs);
            freeConnection(c);
        }
       throw new SQLException("Error in countWhere");
    }

    /**
     * Retrieves the number of rows of the table did_parameters with a prepared statement.
     *
     * @param ps the PreparedStatement to be used
     * @return the number of rows returned
     */
    //82
    int countByPreparedStatement(PreparedStatement ps) throws SQLException
    {
        ResultSet rs =  null;
        try 
        {
            int iReturn = -1;
            rs = ps.executeQuery();
            if (rs.next())
                iReturn = rs.getInt("MCOUNT");
            if (iReturn != -1)
                return iReturn;
        }
        finally
        {
            getManager().close(rs);
        }
       throw new SQLException("Error in countByPreparedStatement");
    }

    /**
     * Looks for the number of elements of a specific DidParametersBean pObject given a c
     *
     * @param pObject the DidParametersBean pObject to look for
     * @return the number of rows returned
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    //83
    public int countUsingTemplate(DidParameter pObject) throws SQLException, JdbcConnectionException
    {
        StringBuffer where = new StringBuffer("");
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer _sql = null;
        StringBuffer _sqlWhere = null;
    
        try
        {
                _sql = new StringBuffer("SELECT count(*) as MCOUNT  from did_parameters WHERE ");
                _sqlWhere = new StringBuffer("");
                int _dirtyCount = 0;
    
                if (pObject.isCodeModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("code= ?");
                }
    
                if (pObject.isParameterModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("parameter= ?");
                }
    
                if (pObject.isZclassModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("zclass= ?");
                }
    
                if (pObject.isContextModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("context= ?");
                }
    
                if (pObject.isTypeModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("type= ?");
                }
    
                if (pObject.isFormatModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("format= ?");
                }
    
                if (pObject.isUnitModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("unit= ?");
                }
    
                if (pObject.isCommentModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("comment= ?");
                }
    
                if (pObject.isDescriptionModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("description= ?");
                }
    
                if (pObject.isRangeModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("range= ?");
                }
    
                if (pObject.isDefaultValueModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("default_value= ?");
                }
    
                if (pObject.isMandatoryModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("mandatory= ?");
                }
    
                if (pObject.isZprotectedModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("zprotected= ?");
                }
    
                if (_dirtyCount == 0)
                   throw new SQLException ("The pObject to look is unvalid : not initialized !");
    
                _sql.append(_sqlWhere);
                c = getConnection();
                ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
    
                _dirtyCount = 0;
    
                if (pObject.isCodeModified()) {
                    DbConnManager.setLong(ps, ++_dirtyCount, pObject.getCode());
                }
    
                if (pObject.isParameterModified()) {
                    ps.setString(++_dirtyCount, pObject.getParameter());
                }
    
                if (pObject.isZclassModified()) {
                    ps.setString(++_dirtyCount, pObject.getZclass());
                }
    
                if (pObject.isContextModified()) {
                    ps.setString(++_dirtyCount, pObject.getContext());
                }
    
                if (pObject.isTypeModified()) {
                    ps.setString(++_dirtyCount, pObject.getType());
                }
    
                if (pObject.isFormatModified()) {
                    ps.setString(++_dirtyCount, pObject.getFormat());
                }
    
                if (pObject.isUnitModified()) {
                    ps.setString(++_dirtyCount, pObject.getUnit());
                }
    
                if (pObject.isCommentModified()) {
                    ps.setString(++_dirtyCount, pObject.getComment());
                }
    
                if (pObject.isDescriptionModified()) {
                    ps.setString(++_dirtyCount, pObject.getDescription());
                }
    
                if (pObject.isRangeModified()) {
                    ps.setString(++_dirtyCount, pObject.getRange());
                }
    
                if (pObject.isDefaultValueModified()) {
                    ps.setString(++_dirtyCount, pObject.getDefaultValue());
                }
    
                if (pObject.isMandatoryModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getMandatory());
                }
    
                if (pObject.isZprotectedModified()) {
                    DbConnManager.setInteger(ps, ++_dirtyCount, pObject.getZprotected());
                }
    
                return countByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // DECODE RESULT SET 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Transforms a ResultSet iterating on the did_parameters on a DidParametersBean pObject.
     *
     * @param rs the ResultSet to be transformed
     * @return pObject resulting DidParametersBean pObject
     */
    //72
    public DidParameter decodeRow(ResultSet rs) throws SQLException
    {
        DidParameter pObject = createDidParametersBean();
        pObject.setCode(DbConnManager.getLong(rs, 1));
        pObject.setParameter(rs.getString(2));
        pObject.setZclass(rs.getString(3));
        pObject.setContext(rs.getString(4));
        pObject.setType(rs.getString(5));
        pObject.setFormat(rs.getString(6));
        pObject.setUnit(rs.getString(7));
        pObject.setComment(rs.getString(8));
        pObject.setDescription(rs.getString(9));
        pObject.setRange(rs.getString(10));
        pObject.setDefaultValue(rs.getString(11));
        pObject.setMandatory(DbConnManager.getInteger(rs, 12));
        pObject.setZprotected(DbConnManager.getInteger(rs, 13));

        pObject.isNew(false);
        pObject.resetIsModified();

        return pObject;
    }

    /**
     * Transforms a ResultSet iterating on the did_parameters table on a DidParametersBean pObject according to a list of fields.
     *
     * @param rs the ResultSet to be transformed
     * @param fieldList table of the field's associated constants
     * @return pObject resulting DidParametersBean pObject
     */
    //73
    public DidParameter decodeRow(ResultSet rs, int[] fieldList) throws SQLException
    {
        DidParameter pObject = createDidParametersBean();
        int pos = 0;
        for(int i = 0; i < fieldList.length; i++)
        {
            switch(fieldList[i]) {
                case ID_CODE:
                    ++pos;
                    pObject.setCode(DbConnManager.getLong(rs, pos));
                    break;
                case ID_PARAMETER:
                    ++pos;
                    pObject.setParameter(rs.getString(pos));
                    break;
                case ID_ZCLASS:
                    ++pos;
                    pObject.setZclass(rs.getString(pos));
                    break;
                case ID_CONTEXT:
                    ++pos;
                    pObject.setContext(rs.getString(pos));
                    break;
                case ID_TYPE:
                    ++pos;
                    pObject.setType(rs.getString(pos));
                    break;
                case ID_FORMAT:
                    ++pos;
                    pObject.setFormat(rs.getString(pos));
                    break;
                case ID_UNIT:
                    ++pos;
                    pObject.setUnit(rs.getString(pos));
                    break;
                case ID_COMMENT:
                    ++pos;
                    pObject.setComment(rs.getString(pos));
                    break;
                case ID_DESCRIPTION:
                    ++pos;
                    pObject.setDescription(rs.getString(pos));
                    break;
                case ID_RANGE:
                    ++pos;
                    pObject.setRange(rs.getString(pos));
                    break;
                case ID_DEFAULT_VALUE:
                    ++pos;
                    pObject.setDefaultValue(rs.getString(pos));
                    break;
                case ID_MANDATORY:
                    ++pos;
                    pObject.setMandatory(DbConnManager.getInteger(rs, pos));
                    break;
                case ID_ZPROTECTED:
                    ++pos;
                    pObject.setZprotected(DbConnManager.getInteger(rs, pos));
                    break;
            }
        }
        pObject.isNew(false);
        pObject.resetIsModified();

        return pObject;
    }

    //////////////////////////////////////
    // PREPARED STATEMENT LOADER
    //////////////////////////////////////

    /**
     * Loads all the elements using a prepared statement.
     *
     * @param ps the PreparedStatement to be used
     * @return an array of DidParametersBean 
     */
    //41
    public DidParameter[] loadByPreparedStatement(PreparedStatement ps) throws SQLException
    {
        return loadByPreparedStatement(ps, null);
    }

    /**
     * Loads all the elements using a prepared statement specifying a list of fields to be retrieved.
     *
     * @param ps the PreparedStatement to be used
     * @param fieldList table of the field's associated constants
     * @return an array of DidParametersBean 
     */
    public DidParameter[] loadByPreparedStatement(PreparedStatement ps, int[] fieldList) throws SQLException
    {
        ResultSet rs =  null;
        java.util.ArrayList v =  null;
        try
        {
            rs =  ps.executeQuery();
            v = new java.util.ArrayList();
            while(rs.next())
            {
                if(fieldList == null)
                    v.add(decodeRow(rs));
                else 
                    v.add(decodeRow(rs, fieldList));
            }
            return (DidParameter[])v.toArray(new DidParameter[0]);
        }
        finally
        {
            if (v != null) { v.clear(); v = null;}
            getManager().close(rs);
        }
    }

    ///////////////////////////////////////////////////////////////////////
    // LISTENER 
    ///////////////////////////////////////////////////////////////////////
    private DidParametersListener listener = null;

    /**
     * Registers a unique DidParametersListener listener.
     */
    //66.5
    public void registerListener(DidParametersListener listener) {
        this.listener = listener;
    }

    /**
     * Before the save of the DidParametersBean pObject.
     *
     * @param pObject the DidParametersBean pObject to be saved
     */
    //67
    void beforeInsert(DidParameter pObject) throws SQLException {
        if (listener != null)
            listener.beforeInsert(pObject);
    }

    /**
     * After the save of the DidParametersBean pObject.
     *
     * @param pObject the DidParametersBean pObject to be saved
     */
    //68
    void afterInsert(DidParameter pObject) throws SQLException {
        if (listener != null)
            listener.afterInsert(pObject);
    }

    /**
     * Before the update of the DidParametersBean pObject.
     *
     * @param pObject the DidParametersBean pObject to be updated
     */
    //69
    void beforeUpdate(DidParameter pObject) throws SQLException {
        if (listener != null)
            listener.beforeUpdate(pObject);
    }

    /**
     * After the update of the DidParametersBean pObject.
     *
     * @param pObject the DidParametersBean pObject to be updated
     */
    //70
    void afterUpdate(DidParameter pObject) throws SQLException {
        if (listener != null)
            listener.afterUpdate(pObject);
    }

    ///////////////////////////////////////////////////////////////////////
    // UTILS  
    ///////////////////////////////////////////////////////////////////////

    /**
     * Retrieves the manager object used to get connections.
     *
     * @return the manager used
     */
    //2
    DbConnManager getManager() {
        return DbConnManager.getInstance();
    }

    /**
     * Frees the connection.
     *
     * @param c the connection to release
     */
    void freeConnection(Connection c) {
        //getManager().releaseDicConnection(c); // back to pool
    }
    /**
     * Gets the connection.
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    Connection getConnection() throws SQLException, JdbcConnectionException {
        return getManager().getDicConnection();
    }

}
