package org.eso.oca.data;



/**
 * Class MetaColumn
 *
 * Business object representing a row on obs_columns table from raw database, 
 * or a row on calib_columns table from calib database
 *
 * @version $Revision: 1.4 $
 * @author pnunes, Mar 30, 2005
 */

public class MetaColumn {
    
    private String tname;
    private boolean tname_is_modified = false;
    private boolean tname_is_initialized = false;
    
    private String name;
    private boolean name_is_modified = false;
    private boolean name_is_initialized = false;
    
    private String type;
    private boolean type_is_modified = false;
    private boolean type_is_initialized = false;
    
    private Integer length;
    private boolean length_is_modified = false;
    private boolean length_is_initialized = false;
    
    private String format;
    private boolean format_is_modified = false;
    private boolean format_is_initialized = false;
    
    private String keyword;
    private boolean keyword_is_modified = false;
    private boolean keyword_is_initialized = false;
    
    private boolean _isNew = true;
    
    /**
     * Do not use this constructor directly, please use the factory method
     * available in the associated manager.
     */
    MetaColumn()
    {
    }
    
    /**
     * Getter method for tname.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_columns.tname
     * <li>column size: 30
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of tname
     */
    public String getTname()
    {
        return tname; 
    }

    /**
     * Setter method for tname.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to tname
     */
    public void setTname(String newVal) {
        if ((newVal != null && this.tname != null && (newVal.compareTo(this.tname) == 0)) || 
            (newVal == null && this.tname == null && tname_is_initialized)) {
            return; 
        } 
        this.tname = newVal; 
        tname_is_modified = true; 
        tname_is_initialized = true; 
    }

    /**
     * Determines if the tname has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isTnameModified() {
        return tname_is_modified; 
    }

    /**
     * Determines if the tname has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isTnameInitialized() {
        return tname_is_initialized; 
    }

    /**
     * Getter method for name.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_columns.name
     * <li>column size: 30
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of name
     */
    public String getName()
    {
        return name; 
    }

    /**
     * Setter method for name.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to name
     */
    public void setName(String newVal) {
        if ((newVal != null && this.name != null && (newVal.compareTo(this.name) == 0)) || 
            (newVal == null && this.name == null && name_is_initialized)) {
            return; 
        } 
        this.name = newVal; 
        name_is_modified = true; 
        name_is_initialized = true; 
    }

    /**
     * Determines if the name has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isNameModified() {
        return name_is_modified; 
    }

    /**
     * Determines if the name has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isNameInitialized() {
        return name_is_initialized; 
    }

    /**
     * Getter method for type.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_columns.type
     * <li>column size: 30
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of type
     */
    public String getType()
    {
        return type; 
    }

    /**
     * Setter method for type.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to type
     */
    public void setType(String newVal) {
        if ((newVal != null && this.type != null && (newVal.compareTo(this.type) == 0)) || 
            (newVal == null && this.type == null && type_is_initialized)) {
            return; 
        } 
        this.type = newVal; 
        type_is_modified = true; 
        type_is_initialized = true; 
    }

    /**
     * Determines if the type has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isTypeModified() {
        return type_is_modified; 
    }

    /**
     * Determines if the type has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isTypeInitialized() {
        return type_is_initialized; 
    }

    /**
     * Getter method for length.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_columns.length
     * <li>column size: 5
     * <li>jdbc type returned by the driver: Types.SMALLINT
     * </ul>
     *
     * @return the value of length
     */
    public Integer getLength()
    {
        return length; 
    }

    /**
     * Setter method for length.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to length
     */
    public void setLength(Integer newVal) {
        if ((newVal != null && this.length != null && (newVal.compareTo(this.length) == 0)) || 
            (newVal == null && this.length == null && length_is_initialized)) {
            return; 
        } 
        this.length = newVal; 
        length_is_modified = true; 
        length_is_initialized = true; 
    }

    /**
     * Setter method for length.
     * <br>
     * Convenient for those who do not want to deal with Objects for primary types.
     *
     * @param newVal the new value to be assigned to length
     */
    public void setLength(int newVal) {
        setLength(new Integer(newVal));
    }

    /**
     * Determines if the length has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isLengthModified() {
        return length_is_modified; 
    }

    /**
     * Determines if the length has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isLengthInitialized() {
        return length_is_initialized; 
    }

    /**
     * Getter method for format.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_columns.format
     * <li>column size: 10
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of format
     */
    public String getFormat()
    {
        return format; 
    }

    /**
     * Setter method for format.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to format
     */
    public void setFormat(String newVal) {
        if ((newVal != null && this.format != null && (newVal.compareTo(this.format) == 0)) || 
            (newVal == null && this.format == null && format_is_initialized)) {
            return; 
        } 
        this.format = newVal; 
        format_is_modified = true; 
        format_is_initialized = true; 
    }

    /**
     * Determines if the format has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isFormatModified() {
        return format_is_modified; 
    }

    /**
     * Determines if the format has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isFormatInitialized() {
        return format_is_initialized; 
    }

    /**
     * Getter method for keyword.
     * <br>
     * Meta Data Information (in progress):
     * <ul>
     * <li>full name: obs_columns.keyword
     * <li>column size: 50
     * <li>jdbc type returned by the driver: Types.VARCHAR
     * </ul>
     *
     * @return the value of keyword
     */
    public String getKeyword()
    {
        return keyword; 
    }

    /**
     * Setter method for keyword.
     * <br>
     * The new value is set only if compareTo() says it is different,
     * or if one of either the new value or the current value is null.
     * In case the new value is different, it is set and the field is marked as 'modified'.
     *
     * @param newVal the new value to be assigned to keyword
     */
    public void setKeyword(String newVal) {
        if ((newVal != null && this.keyword != null && (newVal.compareTo(this.keyword) == 0)) || 
            (newVal == null && this.keyword == null && keyword_is_initialized)) {
            return; 
        } 
        this.keyword = newVal; 
        keyword_is_modified = true; 
        keyword_is_initialized = true; 
    }

    /**
     * Determines if the keyword has been modified.
     *
     * @return true if the field has been modified, false if the field has not been modified
     */
    public boolean isKeywordModified() {
        return keyword_is_modified; 
    }

    /**
     * Determines if the keyword has been initialized.
     * <br>
     * It is useful to determine if a field is null on purpose or just because it has not been initialized.
     *
     * @return true if the field has been initialized, false otherwise
     */
    public boolean isKeywordInitialized() {
        return keyword_is_initialized; 
    }

    /**
     * Determines if the current object is new.
     *
     * @return true if the current object is new, false if the object is not new
     */
    public boolean isNew() {
        return _isNew;
    }

    /**
     * Specifies to the object if it has been set as new.
     *
     * @param isNew the boolean value to be assigned to the isNew field
     */
    public void isNew(boolean isNew) {
        this._isNew = isNew;
    }

    /**
     * Determines if the object has been modified since the last time this method was called.
     * <br>
     * We can also determine if this object has ever been modified since its creation.
     *
     * @return true if the object has been modified, false if the object has not been modified
     */
    public boolean isModified() {
        return tname_is_modified || 
		name_is_modified || 
		type_is_modified || 
		length_is_modified || 
		format_is_modified || 
		keyword_is_modified;
    }

    /**
     * Resets the object modification status to 'not modified'.
     */
    public void resetIsModified() {
        tname_is_modified = false;
        name_is_modified = false;
        type_is_modified = false;
        length_is_modified = false;
        format_is_modified = false;
        keyword_is_modified = false;
    }

    /**
     * Copies the passed bean into the current bean.
     *
     * @param bean the bean to copy into the current bean
     */
    public void copy(MetaColumn bean) {
        setTname(bean.getTname());
        setName(bean.getName());
        setType(bean.getType());
        setLength(bean.getLength());
        setFormat(bean.getFormat());
        setKeyword(bean.getKeyword());
    }

    /**
     * Returns the object string representation.
     *
     * @return the object as a string
     */
    public String toString() {
        return   "\n[obs_columns] "
                 + "\n - obs_columns.tname = " + (tname_is_initialized ? ("[" + (tname == null ? null : tname.toString()) + "]") : "not initialized") + ""
                 + "\n - obs_columns.name = " + (name_is_initialized ? ("[" + (name == null ? null : name.toString()) + "]") : "not initialized") + ""
                 + "\n - obs_columns.type = " + (type_is_initialized ? ("[" + (type == null ? null : type.toString()) + "]") : "not initialized") + ""
                 + "\n - obs_columns.length = " + (length_is_initialized ? ("[" + (length == null ? null : length.toString()) + "]") : "not initialized") + ""
                 + "\n - obs_columns.format = " + (format_is_initialized ? ("[" + (format == null ? null : format.toString()) + "]") : "not initialized") + ""
                 + "\n - obs_columns.keyword = " + (keyword_is_initialized ? ("[" + (keyword == null ? null : keyword.toString()) + "]") : "not initialized") + ""
            ;
    }

}
