package org.eso.oca.data;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;


/**
 * Class ObsTablesDAO
 * 
 * Handles database calls for the obs_tables table.
 *
 * @version $Revision: 1.6 $
 * @author pnunes, Mar 30, 2005
 */

public class ObsTablesDAO {

    /**
     * Column name of type Types.VARCHAR mapped to String.
     */
    public static final int ID_NAME = 0;
    public static final int TYPE_NAME = Types.VARCHAR;
    public static final String NAME_NAME = "name";

    /**
     * Column instrume of type Types.VARCHAR mapped to String.
     */
    public static final int ID_INSTRUME = 1;
    public static final int TYPE_INSTRUME = Types.VARCHAR;
    public static final String NAME_INSTRUME = "instrume";


    private static final String TABLE_NAME = "obs_tables";

    /**
     * Create an array of type string containing all the fields of the obs_tables table.
     */
    private static final String[] FIELD_NAMES = 
    {
        "obs_tables.name"
        ,"obs_tables.instrume"
    };

    /**
     * Field that contains the comma separated fields of the obs_tables table.
     */
    private static final String ALL_FIELDS = "obs_tables.name"
                            + ",obs_tables.instrume";

    private static ObsTablesDAO singleton = new ObsTablesDAO();

    /**
     * Get the ObsTablesManager singleton.
     *
     * @return ObsTablesManager 
     */
    synchronized public static ObsTablesDAO getInstance()
    {
        return singleton;
    }

    /**
     * Sets your own ObsTablesManager instance.
     <br>
     * This is optional, by default we provide it for you.
     */
    synchronized public static void setInstance(ObsTablesDAO instance)
    {
        singleton = instance;
    }


    /**
     * Creates a new ObsTablesBean instance.
     *
     * @return the new ObsTablesBean 
     */
    public MetaTable createObsTablesBean()
    {
        return new MetaTable();
    }



    //////////////////////////////////////
    // LOAD ALL
    //////////////////////////////////////

    /**
     * Loads all the rows from obs_tables.
     *
     * @return an array of ObsTablesManager pObject
     */
    //38
    public MetaTable[] loadAll() throws SQLException , JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        try 
        {
            c = getConnection();
            ps = c.prepareStatement("SELECT " + ALL_FIELDS + " FROM obs_tables",ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            return loadByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }

    //////////////////////////////////////
    // SQL 'WHERE' METHOD
    //////////////////////////////////////
    /**
     * Retrieves an array of ObsTablesBean given a sql 'where' clause.
     *
     * @param where the sql 'where' clause
     * @return the resulting ObsTablesBean table 
     */
    //49
    public MetaTable[] loadByWhere(String where) throws SQLException, JdbcConnectionException
    {
        return loadByWhere(where, null);
    }

    /**
     * Retrieves an array of ObsTablesBean given a sql where clause, and a list of fields.
     * It is up to you to pass the 'WHERE' in your where clausis.
     *
     * @param where the sql 'where' clause
     * @param fieldList table of the field's associated constants
     * @return the resulting ObsTablesBean table 
     */
    //51
    public MetaTable[] loadByWhere(String where, int[] fieldList) throws SQLException, JdbcConnectionException
    {
        String sql = null;
        if(fieldList == null)
            sql = "select " + ALL_FIELDS + " from obs_tables " + where;
        else
        {
            StringBuffer buff = new StringBuffer(128);
            buff.append("select ");
            for(int i = 0; i < fieldList.length; i++)
            {
                if(i != 0)
                    buff.append(",");
                buff.append(FIELD_NAMES[fieldList[i]]);
            }
            buff.append(" from obs_tables ");
            buff.append(where);
            sql = buff.toString();
            buff = null;
        }
        Connection c = null;
        Statement pStatement = null;
        ResultSet rs =  null;
        java.util.ArrayList v = null;
        try 
        {
            c = getConnection();
            pStatement = c.createStatement();
            rs =  pStatement.executeQuery(sql);
            v = new java.util.ArrayList();
            while(rs.next())
            {
                if(fieldList == null)
                    v.add(decodeRow(rs));
                else
                    v.add(decodeRow(rs, fieldList));
            }

            return (MetaTable[])v.toArray(new MetaTable[0]);
        }
        finally
        {
            if (v != null) { v.clear(); }
            getManager().close(pStatement, rs);
            freeConnection(c);
        }
    }


    /**
     * Deletes all rows from obs_tables table.
     * @return the number of deleted rows.
     */
    public int deleteAll() throws SQLException, JdbcConnectionException
    {
        return deleteByWhere("");
    }


    /**
     * Deletes rows from the obs_tables table using a 'where' clause.
     * It is up to you to pass the 'WHERE' in your where clausis.
     * <br>Attention, if 'WHERE' is omitted it will delete all records. 
     *
     * @param where the sql 'where' clause
     * @return the number of deleted rows
     */
    public int deleteByWhere(String where) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;

        try
        {
            c = getConnection();
            String delByWhereSQL = "DELETE FROM obs_tables " + where;
            ps = c.prepareStatement(delByWhereSQL);
            return ps.executeUpdate();
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // SAVE 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Saves the ObsTablesBean pObject into the database.
     *
     * @param pObject the ObsTablesBean pObject to be saved
     */
    //100
    public MetaTable save(MetaTable pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer _sql = null;

        try
        {
            c = getConnection();
            if (pObject.isNew())
            { // SAVE 
                beforeInsert(pObject); // listener callback
                int _dirtyCount = 0;
                _sql = new StringBuffer("INSERT into obs_tables (");
    
                if (pObject.isNameModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("name");
                    _dirtyCount++;
                }

                if (pObject.isInstrumeModified()) {
                    if (_dirtyCount>0) {
                        _sql.append(",");
                    }
                    _sql.append("instrume");
                    _dirtyCount++;
                }

                _sql.append(") values (");
                if(_dirtyCount > 0) {
                    _sql.append("?");
                    for(int i = 1; i < _dirtyCount; i++) {
                        _sql.append(",?");
                    }
                }
                _sql.append(")");

                //ps = c.prepareStatement(_sql.toString(), Statement.RETURN_GENERATED_KEYS);
                
                ps = c.prepareStatement(_sql.toString());
                
                
                _dirtyCount = 0;

                if (pObject.isNameModified()) {
                    ps.setString(++_dirtyCount, pObject.getName());
                }
    
                if (pObject.isInstrumeModified()) {
                    ps.setString(++_dirtyCount, pObject.getInstrume());
                }
    
                ps.executeUpdate();
    
                pObject.isNew(false);
                pObject.resetIsModified();
                afterInsert(pObject); // listener callback
            }
            else 
            { // UPDATE 
                beforeUpdate(pObject); // listener callback
                _sql = new StringBuffer("UPDATE obs_tables SET ");
                boolean useComma=false;

                if (pObject.isNameModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("name").append("=?");
                }

                if (pObject.isInstrumeModified()) {
                    if (useComma) {
                        _sql.append(",");
                    } else {
                        useComma=true;
                    }
                    _sql.append("instrume").append("=?");
                }
                _sql.append("");
                ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
                int _dirtyCount = 0;

                if (pObject.isNameModified()) {
                      ps.setString(++_dirtyCount, pObject.getName());
                }

                if (pObject.isInstrumeModified()) {
                      ps.setString(++_dirtyCount, pObject.getInstrume());
                }
    
                if (_dirtyCount == 0) {
                     return pObject;
                }
    
                ps.executeUpdate();
                pObject.resetIsModified();
                afterUpdate(pObject); // listener callback
            }
    
            return pObject;
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    /**
     * Saves an array of ObsTablesBean pObjects into the database.
     *
     * @param pObjects the ObsTablesBean pObject table to be saved
     * @return the saved ObsTablesBean array.
     */
    //65
    public MetaTable[] save(MetaTable[] pObjects) throws SQLException , JdbcConnectionException
    {
        for (int iIndex = 0; iIndex < pObjects.length; iIndex ++){
            save(pObjects[iIndex]);
        }
        return pObjects;
    }



    ///////////////////////////////////////////////////////////////////////
    // USING TEMPLATE 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Loads a unique ObsTablesBean pObject from a template one giving a c
     *
     * @param pObject the ObsTablesBean pObject to look for
     * @return the pObject matching the template
     */
    //85
    public MetaTable loadUniqueUsingTemplate(MetaTable pObject) throws SQLException, JdbcConnectionException
    {
         MetaTable[] pReturn = loadUsingTemplate(pObject);
         if (pReturn.length == 0)
             return null;
         if (pReturn.length > 1)
             throw new SQLException("More than one element !!");
         return pReturn[0];
     }

    /**
     * Loads an array of ObsTablesBean from a template one.
     *
     * @param pObject the ObsTablesBean template to look for
     * @return all the ObsTablesBean matching the template
     */
    //88
    public MetaTable[] loadUsingTemplate(MetaTable pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer where = new StringBuffer("");
        StringBuffer _sql = new StringBuffer("SELECT " + ALL_FIELDS + " from obs_tables WHERE ");
        StringBuffer _sqlWhere = new StringBuffer("");
        try
        {
            int _dirtyCount = 0;
    
             if (pObject.isNameModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("name= ?");
             }
    
             if (pObject.isInstrumeModified()) {
                 _dirtyCount ++; 
                 _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("instrume= ?");
             }
    
             if (_dirtyCount == 0) {
                 throw new SQLException ("The pObject to look for is invalid : not initialized !");
             }
             _sql.append(_sqlWhere);
             c = getConnection();
             ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
             _dirtyCount = 0;
    
             if (pObject.isNameModified()) {
                 ps.setString(++_dirtyCount, pObject.getName());
             }
    
             if (pObject.isInstrumeModified()) {
                 ps.setString(++_dirtyCount, pObject.getInstrume());
             }
    
             ps.executeQuery();
             return loadByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }
    /**
     * Deletes rows using a ObsTablesBean template.
     *
     * @param pObject the ObsTablesBean object(s) to be deleted
     * @return the number of deleted objects
     */
    //63
    public int deleteUsingTemplate(MetaTable pObject) throws SQLException, JdbcConnectionException
    {
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer sql = null;
    
        try 
        {
            sql = new StringBuffer("DELETE FROM obs_tables WHERE ");
            int _dirtyAnd = 0;
            if (pObject.isNameInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("name").append("=?");
                _dirtyAnd ++;
            }
    
            if (pObject.isInstrumeInitialized()) {
                if (_dirtyAnd > 0)
                    sql.append(" AND ");
                sql.append("instrume").append("=?");
                _dirtyAnd ++;
            }
    
            c = getConnection();
            ps = c.prepareStatement(sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
            int _dirtyCount = 0;
    
            if (pObject.isNameInitialized()) {
                ps.setString(++_dirtyCount, pObject.getName());
            }
    
            if (pObject.isInstrumeInitialized()) {
                ps.setString(++_dirtyCount, pObject.getInstrume());
            }
    
            int _rows = ps.executeUpdate();
            return _rows;
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // COUNT 
    ///////////////////////////////////////////////////////////////////////

    /**
     * Retrieves the number of rows of the table obs_tables.
     *
     * @return the number of rows returned
     */
    //78
    public int countAll() throws SQLException, JdbcConnectionException
    {
        return countWhere("");
    }



    /**
     * Retrieves the number of rows of the table obs_tables with a 'where' clause.
     * It is up to you to pass the 'WHERE' in your where clausis.
     *
     * @param where the restriction clause
     * @return the number of rows returned
     */
    public int countWhere(String where) throws SQLException, JdbcConnectionException
    {
        String sql = "select count(*) as MCOUNT from obs_tables " + where;
        Connection c = null;
        Statement pStatement = null;
        ResultSet rs =  null;
        try 
        {
            int iReturn = -1;    
            c = getConnection();
            pStatement = c.createStatement();
            rs =  pStatement.executeQuery(sql);
            if (rs.next())
            {
                iReturn = rs.getInt("MCOUNT");
            }
            if (iReturn != -1)
                return iReturn;
        }
        finally
        {
            getManager().close(pStatement, rs);
            freeConnection(c);
        }
       throw new SQLException("Error in countWhere");
    }

    /**
     * Retrieves the number of rows of the table obs_tables with a prepared statement.
     *
     * @param ps the PreparedStatement to be used
     * @return the number of rows returned
     */
    //82
    int countByPreparedStatement(PreparedStatement ps) throws SQLException
    {
        ResultSet rs =  null;
        try 
        {
            int iReturn = -1;
            rs = ps.executeQuery();
            if (rs.next())
                iReturn = rs.getInt("MCOUNT");
            if (iReturn != -1)
                return iReturn;
        }
        finally
        {
            getManager().close(rs);
        }
       throw new SQLException("Error in countByPreparedStatement");
    }

    /**
     * Looks for the number of elements of a specific ObsTablesBean pObject given a c
     *
     * @param pObject the ObsTablesBean pObject to look for
     * @return the number of rows returned
     */
    //83
    public int countUsingTemplate(MetaTable pObject) throws SQLException, JdbcConnectionException
    {
        StringBuffer where = new StringBuffer("");
        Connection c = null;
        PreparedStatement ps = null;
        StringBuffer _sql = null;
        StringBuffer _sqlWhere = null;
    
        try
        {
                _sql = new StringBuffer("SELECT count(*) as MCOUNT  from obs_tables WHERE ");
                _sqlWhere = new StringBuffer("");
                int _dirtyCount = 0;
    
                if (pObject.isNameModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("name= ?");
                }
    
                if (pObject.isInstrumeModified()) {
                    _dirtyCount++; 
                    _sqlWhere.append((_sqlWhere.length() == 0) ? " " : " AND ").append("instrume= ?");
                }
    
                if (_dirtyCount == 0)
                   throw new SQLException ("The pObject to look is unvalid : not initialized !");
    
                _sql.append(_sqlWhere);
                c = getConnection();
                ps = c.prepareStatement(_sql.toString(),ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
    
                _dirtyCount = 0;
    
                if (pObject.isNameModified()) {
                    ps.setString(++_dirtyCount, pObject.getName());
                }
    
                if (pObject.isInstrumeModified()) {
                    ps.setString(++_dirtyCount, pObject.getInstrume());
                }
    
                return countByPreparedStatement(ps);
        }
        finally
        {
            getManager().close(ps);
            freeConnection(c);
        }
    }



    ///////////////////////////////////////////////////////////////////////
    // DECODE RESULT SET 
    ///////////////////////////////////////////////////////////////////////
    /**
     * Transforms a ResultSet iterating on the obs_tables on a ObsTablesBean pObject.
     *
     * @param rs the ResultSet to be transformed
     * @return pObject resulting ObsTablesBean pObject
     */
    //72
    public MetaTable decodeRow(ResultSet rs) throws SQLException
    {
        MetaTable pObject = createObsTablesBean();
        pObject.setName(rs.getString(1));
        pObject.setInstrume(rs.getString(2));

        pObject.isNew(false);
        pObject.resetIsModified();

        return pObject;
    }

    /**
     * Transforms a ResultSet iterating on the obs_tables table on a ObsTablesBean pObject according to a list of fields.
     *
     * @param rs the ResultSet to be transformed
     * @param fieldList table of the field's associated constants
     * @return pObject resulting ObsTablesBean pObject
     */
    //73
    public MetaTable decodeRow(ResultSet rs, int[] fieldList) throws SQLException
    {
        MetaTable pObject = createObsTablesBean();
        int pos = 0;
        for(int i = 0; i < fieldList.length; i++)
        {
            switch(fieldList[i]) {
                case ID_NAME:
                    ++pos;
                    pObject.setName(rs.getString(pos));
                    break;
                case ID_INSTRUME:
                    ++pos;
                    pObject.setInstrume(rs.getString(pos));
                    break;
            }
        }
        pObject.isNew(false);
        pObject.resetIsModified();

        return pObject;
    }

    //////////////////////////////////////
    // PREPARED STATEMENT LOADER
    //////////////////////////////////////

    /**
     * Loads all the elements using a prepared statement.
     *
     * @param ps the PreparedStatement to be used
     * @return an array of ObsTablesBean 
     */
    //41
    public MetaTable[] loadByPreparedStatement(PreparedStatement ps) throws SQLException
    {
        return loadByPreparedStatement(ps, null);
    }

    /**
     * Loads all the elements using a prepared statement specifying a list of fields to be retrieved.
     *
     * @param ps the PreparedStatement to be used
     * @param fieldList table of the field's associated constants
     * @return an array of ObsTablesBean 
     */
    public MetaTable[] loadByPreparedStatement(PreparedStatement ps, int[] fieldList) throws SQLException
    {
        ResultSet rs =  null;
        java.util.ArrayList v =  null;
        try
        {
            rs =  ps.executeQuery();
            v = new java.util.ArrayList();
            while(rs.next())
            {
                if(fieldList == null)
                    v.add(decodeRow(rs));
                else 
                    v.add(decodeRow(rs, fieldList));
            }
            return (MetaTable[])v.toArray(new MetaTable[0]);
        }
        finally
        {
            if (v != null) { v.clear(); v = null;}
            getManager().close(rs);
        }
    }

    ///////////////////////////////////////////////////////////////////////
    // LISTENER 
    ///////////////////////////////////////////////////////////////////////
    private ObsTablesListener listener = null;

    /**
     * Registers a unique ObsTablesListener listener.
     */
    //66.5
    public void registerListener(ObsTablesListener listener) {
        this.listener = listener;
    }

    /**
     * Before the save of the ObsTablesBean pObject.
     *
     * @param pObject the ObsTablesBean pObject to be saved
     */
    //67
    void beforeInsert(MetaTable pObject) throws SQLException {
        if (listener != null)
            listener.beforeInsert(pObject);
    }

    /**
     * After the save of the ObsTablesBean pObject.
     *
     * @param pObject the ObsTablesBean pObject to be saved
     */
    //68
    void afterInsert(MetaTable pObject) throws SQLException {
        if (listener != null)
            listener.afterInsert(pObject);
    }

    /**
     * Before the update of the ObsTablesBean pObject.
     *
     * @param pObject the ObsTablesBean pObject to be updated
     */
    //69
    void beforeUpdate(MetaTable pObject) throws SQLException {
        if (listener != null)
            listener.beforeUpdate(pObject);
    }

    /**
     * After the update of the ObsTablesBean pObject.
     *
     * @param pObject the ObsTablesBean pObject to be updated
     */
    //70
    void afterUpdate(MetaTable pObject) throws SQLException {
        if (listener != null)
            listener.afterUpdate(pObject);
    }

    ///////////////////////////////////////////////////////////////////////
    // UTILS  
    ///////////////////////////////////////////////////////////////////////

    /**
     * Retrieves the manager object used to get connections.
     *
     * @return the manager used
     */
    //2
    DbConnManager getManager() {
        return DbConnManager.getInstance();
    }

    /**
     * Frees the connection.
     *
     * @param c the connection to release
     */
    void freeConnection(Connection c) {
        //getManager().releaseObsConnection(c); // back to pool
    }
    /**
     * Gets the connection.
     * @throws SQLException
     * @throws JdbcConnectionException
     * @throws OCAPropertiesException
     */
    Connection getConnection() throws SQLException, JdbcConnectionException {
        return getManager().getObsConnection();
    }


}
