package org.eso.oca.fitsmapper;

import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;

import org.eso.oca.data.CalibColumnsDAO;
import org.eso.oca.data.CalibTablesDAO;
import org.eso.oca.data.JdbcConnectionException;
import org.eso.oca.data.MetaColumn;
import org.eso.oca.data.MetaTable;

/**
 * Class DBFitsMapper
 *
 * Implements the DBFitsMapper for the calib database
 *
 * @version $Revision: 1.3 $
 * @author pnunes, Mar 30, 2005
 */

// $Id: CalibDbFitsMapper.java,v 1.3 2005/06/06 10:31:15 szampier Exp $ 
public class CalibDbFitsMapper extends DbFitsMapper {
    

    /**
     * Singleton
     */
    private static CalibDbFitsMapper _instance; 

    
    // For lazy initialization 
    public static synchronized CalibDbFitsMapper getInstance() throws SQLException, JdbcConnectionException { 
       if (_instance==null) { 
          _instance = new CalibDbFitsMapper(); 
         } 
         return _instance; 
    } 
    
    
    /**
     * @throws JdbcConnectionException
     * @throws SQLException
     * 
     */
    private CalibDbFitsMapper() throws SQLException, JdbcConnectionException {
        load();
    }
    
    /**
     * Loads the configuration from the database
     * 
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    private void load() throws SQLException, JdbcConnectionException {
        
        // load the calibTables and store them in cache        
        CalibTablesDAO ctDAO = CalibTablesDAO.getInstance();
        tables = new Hashtable();
        
        MetaTable [] metaTables = ctDAO.loadAll();               
        for (int i = 0; i < metaTables.length; i++) {
            MetaTable metaTable = metaTables[i];
            
            
            //insert in the hashtable
            //check if it is already there
            Vector tabs;
            
            String instrum = metaTable.getInstrume();
            
            // if the hashtable already contains this instrument, 
            // get the vector, otherwise create a new one            
            if ( tables.containsKey(instrum )) {
                tabs = (Vector)tables.get(instrum);                
                
            } else {
                tabs = new Vector();
            }
                        
            //add the new ObsTable object to the vector and put it in the hashtable
            tabs.add(metaTable);            
            tables.put( instrum , tabs );
        }
        
        // load the calibColumns and store them in cache        
        CalibColumnsDAO ccDAO = CalibColumnsDAO.getInstance();
        columnsFromKW = new Hashtable();
        columnsFromColumnName = new Hashtable();
        
        
        // iterate obscolumns
        MetaColumn [] metaColumns = ccDAO.loadAll();               
        for (int i = 0; i < metaColumns.length; i++) {
            MetaColumn metaColumn = metaColumns[i];
            
            // insert in the hashtables
            columnsFromKW.put(  metaColumn.getTname() + metaColumn.getKeyword(), metaColumn );
            columnsFromColumnName.put (  metaColumn.getTname()+  metaColumn.getName() , metaColumn );
        } 
                
    }
                
   

}





