package org.eso.oca.parser;

import java.lang.Integer;
import java.lang.String;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.Hashtable;
import java.util.LinkedList;
import java.util.ListIterator;
import java.util.TreeMap;

import org.apache.log4j.Logger;

/** 
 * 
 * @author Neil Ferguson ESO/DMD/DFS <nferguso@eso.org> 
 */
public class DispatchHelper
{
    /**  */
    private static Hashtable dispatchCache = new Hashtable(); 

    /** Returns a list of all methods having the specified basic 'methodName' 
     * which are available to an object of type 'targetClass', including any 
     * matching methods inherited from superclasses of 'targetClass'. 
     */ 
    static LinkedList getMethodSet(Class targetClass, String methodName) 
    {
        /* The class currently being examined. */ 
        Class currentClass = targetClass;

        /* All methods declared by currentClass (but not inherited ones). */ 
        Method[] classMethods; 

        /* The method currently being considered. */
        Method currentMethod; 

        /* The access modifiers for the method currently being considered. */ 
        int accessModifiers; 

        /* The list of methods to return. */ 
        LinkedList methodList = new LinkedList(); 
        
        do
        {
            /* Get all of the methods declared by currentClass. */ 
            classMethods = currentClass.getDeclaredMethods(); 

            /* Loop through each method in turn. */ 
            for(int i = 0; i < classMethods.length; ++i)
            {
                currentMethod = classMethods[i]; 

                if(currentMethod.getName().equals(methodName)) 
                {
                    /* This method has the correct name - check its access 
                     * modifiers. 
                     */ 
                    accessModifiers = currentMethod.getModifiers(); 

                    /* Skip unreachable private methods. */ 
                    if((currentClass != targetClass) 
                       && Modifier.isPrivate(accessModifiers)) 
                    {
                        continue; 
                    }

                    /* Skip unreachable methods in other packages. */ 
                    if((currentClass.getPackage() != targetClass.getPackage())
                       && !Modifier.isPublic(accessModifiers)
                       && !Modifier.isProtected(accessModifiers)) 
                    {
                        continue; 
                    }

                    /* Add this method to the list. */ 
                    methodList.add(currentMethod); 
                }
            }

            /* Move up a level in the inheritance hierarchy and repeat. */ 
            currentClass = currentClass.getSuperclass(); 
        } 
        while(currentClass != null); 

        return(methodList); 
    }

    /** Not in use, but document it and redo the KA-BOOM! message anyway... 
     */ 
    static Class getDispatchCallingClass()
    {
        try
        {
            throw new Exception();
        }
        catch(Exception e)
        {
            StackTraceElement[] elements = e.getStackTrace();
            
            if(elements.length < 3)
            {
                throw new DispatchReflectionError("KA-BOOM!"); 
            }

            try
            {
                return(Class.forName(elements[2].getClassName())); 
            }
            catch(ClassNotFoundException cnfe)
            {
                return(Object.class); 
            }
        }
    }

    /** 
     */ 
    static Method getNearestMethod(Class targetClass, 
                                   String methodName, 
                                   Object[] args) 
    {
        Method result = null; 

        String methodKey = targetClass.getName() + "::" + methodName + "("; 

        for(int i = 0; i < args.length; ++i)
        {
            if(i > 0)
            {
                methodKey += ", "; 
            }

            methodKey += args[i].getClass().getName(); 
        }

        methodKey += ")"; 

        if((result = (Method) dispatchCache.get(methodKey)) == null) 
        {
            Class currentArgType; 
            LinkedList[] argTypes = new LinkedList[args.length]; 

            for(int i = 0; i < args.length; ++i)
            {
                argTypes[i] = new LinkedList(); 
                currentArgType = args[i].getClass(); 

                do
                {
                    argTypes[i].add(currentArgType); 
                    currentArgType = currentArgType.getSuperclass(); 
                }
                while(currentArgType != null); 
            }

            LinkedList methodList = getMethodSet(targetClass, methodName); 
            ListIterator methodIterator = methodList.listIterator(0); 
            TreeMap scoredMethods = new TreeMap(); 
            Method currentMethod; 
            Class[] currentParams; 
            ListIterator paramIterator; 
            int currentScore;
        
        METHODLOOP:
            for(int i = 0; i < methodList.size(); ++i)
            {
                currentMethod = (Method) methodIterator.next(); 
                currentParams = currentMethod.getParameterTypes(); 
                currentScore = 0;
            
                if(currentParams.length != argTypes.length)
                {
                    continue METHODLOOP; 
                }

            PARAMLOOP:
                for(int j = 0; j < argTypes.length; ++j)
                {
                    paramIterator = argTypes[j].listIterator(0); 
                
                    for(int k = 0; k < argTypes[j].size(); ++k)
                    {
                        if(currentParams[j] == paramIterator.next())
                        {
                            currentScore += k * k; 
                            continue PARAMLOOP; 
                        }
                    }

                    continue METHODLOOP; 
                }

                if(currentScore == 0)
                {
                    result = currentMethod;
                    break METHODLOOP; 
                }

                if(scoredMethods.get(new Integer(currentScore)) == null)
                {
                    scoredMethods.put(new Integer(currentScore), currentMethod); 
                }
            }

            if(result == null)
            {
                if(scoredMethods.size() > 0)
                {
                    result = (Method) scoredMethods.get(scoredMethods.firstKey()); 
                }
            }

            dispatchCache.put(methodKey, result); 
        }

        return(result); 
    }
}