package org.eso.oca.parser;

import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.oca.dal.DBMetaData;
import org.eso.oca.dal.TableMetaData;
import org.eso.oca.fits.OCAFile;

/**
 * This class is used to store the SQL representation of
 * an OCA "action" rule, which is basically a container
 * for select/associate rules.
 * 
 * The main usage of this class is in connection with the
 * SqlVisitor, where the SQL is incrementally built
 * by the various visit() methods.
 * 
 * @author szampier
 */
public class OcaVisitorState {

	/** The Apache Log4j logger. */
	static Logger logger = Logger.getLogger(OcaVisitorState.class);

	/** The instrument name. */
	String instrument;

    /** The exemplar frame is used as a reference for searching
     * associated frames.
     */
	OCAFile exemplar;

	TableMetaData rawFilesMetaData;

	TableMetaData calibFilesMetaData;

	/** The recipe name. */
	String recipe;

	/** The action name. */
	String action;

	/** Collection of select/associate rules converted to SQL. */
	Vector<SelectAssociateState> selects = new Vector<SelectAssociateState>();

	/** Temporary store for minRet value. */
	Integer minRet = new Integer("1");

	/** Temporary store for maxRet value. */
	Integer maxRet = new Integer("1");
	
	/** The name of the current keyword being processed by the visitor. */
	String kwName = null;
	
	/** The value of the current keyword being processed by the visitor. */
	String kwValue = null;

	/**
	 * In some special cases (database views with static column values)
	 * one needs to cast the NULL value to the correct type ...
	 * 
	 * @param column the column name.
	 * @param dataSource the data source (rawFiles or calibFiles).
	 * @return the cast expression - for example "convert(int, null)". 
	 */
	public String getNullValueForColumn(String column, String dataSource) {
		logger.trace("getNullValueForColumn(" + column + "," + dataSource + ")");
		String result = null;
		if (dataSource.equals(ASTSelectAssociateStatement.CALIB_FILES)) {
			result = DBMetaData.getNullValueForColumn(calibFilesMetaData, column);
		} else if (dataSource.equals(ASTSelectAssociateStatement.RAW_FILES)) {
			result = DBMetaData.getNullValueForColumn(rawFilesMetaData,
					column);
		} else {
			result = "null";
		}
		return result;
	}

	/**
	 * Returns string representation of this object.
	 */
	public String toString() {
		StringBuffer buf = new StringBuffer();
		buf.append("action " + action + "\n");
		buf.append("instrument " + instrument + "\n");
		for (int i = 0; i < selects.size(); ++i) {
			buf.append(selects.elementAt(i).toString());
		}
		return buf.toString();
	}

	public String getAction() {
		return action;
	}

	public void setAction(String action) {
		this.action = action;
	}

	public TableMetaData getCalibFilesMetaData() {
		return calibFilesMetaData;
	}

	public void setCalibFilesMetaData(TableMetaData calibFilesMetaData) {
		this.calibFilesMetaData = calibFilesMetaData;
	}

	public OCAFile getExemplar() {
		return exemplar;
	}

	public void setExemplar(OCAFile exemplar) {
		this.exemplar = exemplar;
	}

	public String getInstrument() {
		return instrument;
	}

	public void setInstrument(String instrument) {
		this.instrument = instrument;
	}

	public TableMetaData getRawFilesMetaData() {
		return rawFilesMetaData;
	}

	public void setRawFilesMetaData(TableMetaData rawFilesMetaData) {
		this.rawFilesMetaData = rawFilesMetaData;
	}

	public String getRecipe() {
		return recipe;
	}

	public void setRecipe(String recipe) {
		this.recipe = recipe;
	}

	public SelectAssociateState[] getSelects() {
		return selects.toArray(new SelectAssociateState[selects.size()]);
	}

	public void setSelects(Vector<SelectAssociateState> selects) {
		this.selects = selects;
	}
}
