package org.eso.oca.fitsmapper;

import java.io.File;
import java.sql.SQLException;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eso.oca.data.CalibColumnsDAO;
import org.eso.oca.data.CalibTablesDAO;
import org.eso.oca.data.JdbcConnectionException;
import org.eso.oca.data.MetaColumn;
import org.eso.oca.data.MetaTable;
import org.eso.oca.data.ObsColumnsDAO;
import org.eso.oca.data.ObsTablesDAO;
import org.eso.oca.properties.OCAProperties;
import org.eso.oca.properties.OCAPropertiesException;


/**
 * Class TestDBFitsMapper
 *
 * Unit tests for <code>RawDbFitsMapper</code> and <code>CalibDbFitsMapper</code>
 *
 * @version $Revision: 1.3 $
 * @author pnunes, Apr 4, 2005
 */


public class TestDBFitsMapper extends TestCase {
    
    // test variables
    private String testInstrument = "TEST_INSTRUMENT"; 
    private String testTable = "test_table";
    private String testTableX = "test_tab";
    private String testKW = "TESTKW";
    private String testKW2 = "TESTKW2";
    private String testCol1 = "test_column_1";
    private String testCol2 = "test_column_2";
    private String testType = "char";
    
    
    private RawDbFitsMapper rawMapper;
    private CalibDbFitsMapper calibMapper;
    
    private ObsTablesDAO obsTablesDAO;
    private ObsColumnsDAO obsColumnsDAO;
       
    private CalibTablesDAO calibTablesDAO;
    private CalibColumnsDAO calibColumnsDAO;
    
    
    private static final String configDir =
    	"org" + File.separator + 
    	"eso" + File.separator + 
    	"oca" + File.separator + 
    	"config" + File.separator;
    
    protected static final String propertiesFile =
        configDir + ".ocarc";
    
    
    protected void setUp() throws Exception {
    
        super.setUp();
       
        
    }

    protected void tearDown() throws Exception {

        super.tearDown();
    }
    
    
    
    /**
     * Test the <code>getColumn</code> method of <code>DbFitsMapper</code> class
     *
     * @param mapper
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    private void testGetColumn( DbFitsMapper mapper ) throws SQLException, JdbcConnectionException {

        // simple case
        String columnName = null;
        boolean excepThrown = false; 
        try {
            // gets the table name for the table
            columnName = mapper.getColumn(testInstrument,testKW);
            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }
        
        // exception should have not been thrown
        assertTrue(!excepThrown);
        // should be equal        
        assertTrue(testCol1.equals(columnName));        
        
        
        // cases where an exception is expected
        excepThrown = false; 
        
        
        //case in which there's no table for an instrument
        try {
            // gets the table name for the table
            columnName = mapper.getColumn("FAKE_INSTRUMENT",testKW);            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }        
        assertTrue(excepThrown);
        
        
        //  case in which there's no column for a keyword and instrument
        excepThrown = false; 
        try {
            // gets the table name for the table            
            columnName = mapper.getColumn(testInstrument,"FAKE_KW");            
            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }        
        assertTrue(excepThrown);
        
        
        
        // case of bad configuration

        
    }
    
    /**
     * 
     * Test the <code>getTable</code> method of <code>DbFitsMapper</code> class
     * 
     * @param mapper
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    private void testGetTable( DbFitsMapper mapper ) throws SQLException, JdbcConnectionException {

        
        // simple case
        String tableName = null;
        boolean excepThrown = false; 
        try {
            // gets the table name for the table
            tableName = mapper.getTable(testInstrument,testKW);
            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }
        
        // exception should have not been thrown
        assertTrue(!excepThrown);
        // should be equal        
        assertTrue(tableName.equals(testTable));        
        
        
        // cases where an exception is expected
        excepThrown = false; 
        
        
        //case in which there's no table for an instrument
        try {
            // gets the table name for the table
            tableName = mapper.getTable("FAKE_INSTRUMENT",testKW);            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }        
        assertTrue(excepThrown);
        
        
        //  case in which there's no column for a keyword and instrument
        excepThrown = false; 
        try {
            // gets the table name for the table            
            tableName = mapper.getTable(testInstrument,"FAKE_KW");            
            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }        
        assertTrue(excepThrown);
        
        
        
        // case of bad configuration

        
    }
    
    /**
     * Test the <code>getTableAndColumn</code> method of <code>DbFitsMapper</code> class
     * 
     * @param mapper
     * @throws SQLException
     * @throws JdbcConnectionException
     */
    private void testGetTableAndColumn ( DbFitsMapper mapper) throws SQLException, JdbcConnectionException  {
        
        // simple case
        String tableAndColumn = null;
        boolean excepThrown = false; 
        try {
            // gets the table name for the table
            tableAndColumn = mapper.getTableAndColumn(testInstrument,testKW);
            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }
        
        // exception should have not been thrown
        assertTrue(!excepThrown);
        // should be equal        
        assertTrue(tableAndColumn.equals(testTable + "." + testCol1 ));        
        
        
        // cases where an exception is expected
        excepThrown = false; 
        
        
        //case in which there's no table for an instrument
        try {
            // gets the table name for the table
            tableAndColumn = mapper.getTableAndColumn("FAKE_INSTRUMENT",testKW);            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }        
        assertTrue(excepThrown);
        
        
        //  case in which there's no column for a keyword and instrument
        excepThrown = false; 
        try {
            // gets the table name for the table            
            tableAndColumn = mapper.getTableAndColumn(testInstrument,"FAKE_KW");            
            
        } catch (DbFitsMapperException e) {
            excepThrown = true;
        }        
        assertTrue(excepThrown);
        
        
        
        // case of bad configuration
        
        
    }
    
    
    public void testGetRawColumn () throws SQLException, JdbcConnectionException {
        insertRawTestData();
        rawMapper = RawDbFitsMapper.getInstance();
        testGetColumn(rawMapper);
        deleteRawTestData();
        
    }
    
    public void testGetRawTable () throws SQLException, JdbcConnectionException {
        insertRawTestData();
        rawMapper = RawDbFitsMapper.getInstance();
        testGetTable(rawMapper);        
        deleteRawTestData();
    }
    
    public void testGetRawTableAndColumn() throws SQLException, JdbcConnectionException {
        insertRawTestData();
        rawMapper = RawDbFitsMapper.getInstance();
        testGetTableAndColumn(rawMapper);
        deleteRawTestData();
    }
    
    public void testGetCalibColumn () throws SQLException, JdbcConnectionException {
        insertCalibTestData();
        calibMapper = CalibDbFitsMapper.getInstance();
        testGetColumn(calibMapper);
        deleteCalibTestData();
        
    }
    
    public void testGetCalibTable () throws SQLException, JdbcConnectionException {
        insertCalibTestData();
        calibMapper = CalibDbFitsMapper.getInstance();
        testGetTable(calibMapper);
        deleteCalibTestData();
    }
    
    public void testGetCalibTableAndColumn() throws SQLException, JdbcConnectionException {
        insertCalibTestData();
        calibMapper = CalibDbFitsMapper.getInstance();
        testGetTableAndColumn(calibMapper);
        deleteCalibTestData();
    }
    
    
    
    
    
    private void deleteRawTestData() throws SQLException, JdbcConnectionException {
        obsTablesDAO = ObsTablesDAO.getInstance();
        obsColumnsDAO = ObsColumnsDAO.getInstance();
        
        //  deletes any test records created rows
        obsColumnsDAO.deleteByWhere("where tname='test_table' ");
        obsTablesDAO.deleteByWhere("where name='test_table'");
        obsTablesDAO.deleteByWhere("where name='test_tab'");
    }
    
    private void insertRawTestData() throws SQLException, JdbcConnectionException {        
        obsTablesDAO = ObsTablesDAO.getInstance();
        obsColumnsDAO = ObsColumnsDAO.getInstance();
        
        
        // insert rows in the raw database 
        
        //  deletes any test records created 
        deleteRawTestData();
        
        
        
        // insert a row in obs_tables
        
        MetaTable obsTable2 = obsTablesDAO.createObsTablesBean();        
        obsTable2.setInstrume(testInstrument);
        obsTable2.setName(testTableX);        
        obsTablesDAO.save(obsTable2);
        
        MetaTable obsTable = obsTablesDAO.createObsTablesBean();        
        obsTable.setInstrume(testInstrument);
        obsTable.setName(testTable);        
        obsTablesDAO.save(obsTable);
        
        
        

        // insert 2 rows in obs_columns
        MetaColumn obsColumn = obsColumnsDAO.createObsColumnsBean();        
        obsColumn.setKeyword(testKW);
        obsColumn.setTname(testTable);
        obsColumn.setName(testCol1);
        obsColumn.setType(testType);
        obsColumnsDAO.save(obsColumn);
        
        MetaColumn obsColumn2 = obsColumnsDAO.createObsColumnsBean();        
        obsColumn2.setKeyword(testKW2);
        obsColumn2.setTname(testTable);
        obsColumn2.setName(testCol2);
        obsColumn2.setType(testType);
        obsColumnsDAO.save(obsColumn2);
        
        
        
    }
    
    private void deleteCalibTestData() throws SQLException, JdbcConnectionException {
        calibTablesDAO = CalibTablesDAO.getInstance();
        calibColumnsDAO = CalibColumnsDAO.getInstance();
        
        //  deletes any test records created rows
        calibColumnsDAO.deleteByWhere("where tname='test_table' ");
        calibTablesDAO.deleteByWhere("where name='test_table'");
        
    }
    
    private void insertCalibTestData() throws SQLException, JdbcConnectionException {
        
        calibTablesDAO = CalibTablesDAO.getInstance();
        calibColumnsDAO = CalibColumnsDAO.getInstance();
        
        
        // insert rows in the raw database 
        
        //     deletes any test records created 
        deleteCalibTestData();
        
        // insert a row in obs_tables
        MetaTable calibTable = calibTablesDAO.createCalibTablesBean();        
        calibTable.setInstrume(testInstrument);
        calibTable.setName(testTable);        
        calibTablesDAO.save(calibTable);
        

        // insert 2 rows in obs_columns
        MetaColumn calibColumn = calibColumnsDAO.createCalibColumnsBean();        
        calibColumn.setKeyword(testKW);
        calibColumn.setTname(testTable);
        calibColumn.setName(testCol1);
        calibColumn.setType(testType);
        calibColumnsDAO.save(calibColumn);
        
        MetaColumn calibColumn2 = calibColumnsDAO.createCalibColumnsBean();        
        calibColumn2.setKeyword(testKW2);
        calibColumn2.setTname(testTable);
        calibColumn2.setName(testCol2);
        calibColumn2.setType(testType);
        calibColumnsDAO.save(calibColumn2);
        
        
        
    }
    
    
   
    public static TestSuite suite() {
        TestSuite suite = new TestSuite();

        suite.addTest( new TestSuite(TestDBFitsMapper.class) );
        return suite;
    }
    
    public static void main(String[] args) throws OCAPropertiesException {
        OCAProperties ocaProp = OCAProperties.getInstance();
        ocaProp.loadProperties(TestDBFitsMapper.propertiesFile);
        TestRunner.run(suite());  
    }

}





