package org.eso.oca.fitsmapper;

import java.sql.SQLException;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.eso.oca.data.JdbcConnectionException;

/**
 * Class TestDbKeywordDictionary
 *
 * Unit test for the <code>TestDbKeywordDictionary</code> methods
 *
 * @version $Revision: 1.3 $
 * @author pnunes, Apr 26, 2005
 */

// $Id: TestDbKeywordDictionary.java,v 1.3 2005/06/06 10:32:19 szampier Exp $ 
public class TestDbKeywordDictionary extends TestCase {

        
    /*
     * Logger
     */
    private static Logger logger = Logger.getLogger(TestDbKeywordDictionary.class);
    private DbKeywordDictionary dic;   
        
   
    /**
     * Constructor for TestDbKeywordDictionary.
     * @param arg0
     * @throws JdbcConnectionException
     * @throws SQLException
     */
    public TestDbKeywordDictionary(String arg0) throws SQLException, JdbcConnectionException {
        super(arg0);
        dic = new DbKeywordDictionary();
    }

    public void testGetFitsType() throws SQLException, JdbcConnectionException, DbKeywordDictionaryException {
        
        boolean exceptionOcurred = false;
        
        // simplest cases
        String type = dic.getFitsType("TEST KEYWORD DOUBLE");
        assertTrue( type.equals("double"));
        
        type = dic.getFitsType("TEST KEYWORD INTEGER");
        assertTrue( type.equals("integer"));
        
        type = dic.getFitsType("TEST KEYWORD LOGICAL");
        assertTrue( type.equals("logical"));
        
        
        // case in which keyword is not there
        exceptionOcurred = false;
        try {                   
            type = dic.getFitsType("TEST BLA BLA");   
        } catch ( DbKeywordDictionaryException e ) {
            exceptionOcurred = true;
        }
        assertTrue(exceptionOcurred);
        
        
        
        // case in which there's more than one result     
        exceptionOcurred = false;
        try {                   
            type = dic.getFitsType("TEST KEYWORD STRING");   
        } catch ( DbKeywordDictionaryException e ) {
            exceptionOcurred = true;
        }
        assertTrue(exceptionOcurred);        
        
    }

    
    public void testGetSQLType() throws SQLException, JdbcConnectionException, DbKeywordDictionaryException {
        
        
        boolean exceptionOcurred = false;
        
        //      simplest cases
        String type = dic.getSQLType("TEST KEYWORD DOUBLE");
        assertTrue( type.equals("real"));
        
        type = dic.getSQLType("TEST KEYWORD INTEGER");
        assertTrue( type.equals("int"));
        
        type = dic.getSQLType("TEST KEYWORD LOGICAL");
        assertTrue( type.equals("char(1)"));
        
        
        // case in which keyword is not there
        exceptionOcurred = false;
        try {                   
            type = dic.getSQLType("TEST BLA BLA");   
        } catch ( DbKeywordDictionaryException e ) {
            exceptionOcurred = true;
        }
        assertTrue(exceptionOcurred);
        
        
        
        // case in which there's more than one result     
        exceptionOcurred = false;
        try {                   
            type = dic.getSQLType("TEST KEYWORD STRING");   
        } catch ( DbKeywordDictionaryException e ) {
            exceptionOcurred = true;
        }
        assertTrue(exceptionOcurred);        
    }
    
    

}





