package org.eso.oca.fitsmapper;

import java.io.File;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import junit.extensions.TestSetup;
import junit.framework.Test;
import junit.framework.TestSuite;

import org.apache.log4j.Logger;
import org.eso.oca.data.DbConnManager;
import org.eso.oca.data.JdbcConnectionException;
import org.eso.oca.properties.OCAProperties;
import org.eso.oca.properties.OCAPropertiesException;

/**
 * Class AllTests
 *
 * Test decorator for TestDbKeywordsDictionary class
 * Sets up the data environment for these tests and runs them
 *
 * @version $Revision: 1.3 $
 * @author pnunes, Apr 26, 2005
 */

// $Id: TestDbKeywordsDictionarySetup.java,v 1.3 2005/06/06 10:32:20 szampier Exp $ 
public class TestDbKeywordsDictionarySetup extends TestSetup {

    
    /**
     * Database connection 
     */
    private DbConnManager connManager;    
    private Connection conn; 
    
    /*
     * configuration files 
     */
    private final static String configDir = 
        	"org" + File.separator + 
        	"eso" + File.separator + 
        	"oca" + File.separator + 
        	"config" + File.separator;
    
    private final static String propertiesFile =
        configDir + ".ocarc";
    
    
    /*
     * Logger
     */    
    private static Logger logger = Logger.getLogger(TestDbKeywordsDictionarySetup.class);
    
               
    /*
     *  Test data for TestDbKeywordDictionary class
     * 
     * 	The following table is created in the raw database just for testing
     */
    private String sqlCreateDbKwDictionary = 

        "CREATE TABLE did_parameters (				 "+
        "        code numeric (10) NOT NULL,		 "+
        "        parameter varchar (100) NOT NULL,	 "+
        "        class varchar (100) NOT NULL,		 "+
        "        context varchar (32) NOT NULL,		 "+
        "        type varchar (16) NOT NULL,		 "+
        "        format varchar (8) NOT NULL,		 "+
        "        unit varchar (16) NULL,			 "+
        "        comment varchar (100) NOT NULL,     "+
        "        description varchar (255) NULL, "+
        "        range varchar (255) NULL,           "+
        "        default_value varchar (255) NULL,   "+
        "        mandatory tinyint  NOT NULL,	 "+
        "        protected tinyint  NOT NULL      "+
        "    )   ";

    private String [] sqlInsertsDbKwDictionary = {
            "insert into did_parameters values ( 1 , 'TEST KEYWORD DOUBLE'  , '' , '', 'double'     ,'','' , '' , '', '', '',0,0) ",
            "insert into did_parameters values ( 2 , 'TEST KEYWORD INTEGER' , '' , '', 'integer'        ,'','' , '' , '', '', '',0,0) ",
            "insert into did_parameters values ( 3 , 'TEST KEYWORD LOGICAL' , '' , '', 'logical'    ,'','' , '' , '', '', '',0,0) ",
            "insert into did_parameters values ( 4 , 'TEST KEYWORD STRING'  , '' , '', 'string','','' , '' , '', '', '',0,0) ",
            "insert into did_parameters values ( 5 , 'TEST KEYWORD STRING'  , '' , '', 'string','','' , '' , '', '', '',0,0) "
    };
    
    private String sqlDropDbKwDictionary = "drop table did_parameters";
    /* **/


    private void setUpDataDbKwDictionary() throws SQLException, JdbcConnectionException {

        /* 
         * test data for TestDbKeywordDictionary tests 
         * */        
        // drops in case it already exists
        dropDataDbKwDictionary();
        
        connManager = DbConnManager.getInstance();
        conn = connManager.getObsConnection();
    
        
    
        // 	create the temporary table
        PreparedStatement ps = null;
        StringBuffer _sql = new StringBuffer();
    
        _sql.append(sqlCreateDbKwDictionary);
    
        logger.debug(_sql.toString());
        ps = conn.prepareStatement(_sql.toString());        
        ps.executeUpdate();
    
    
        // 	insert the test data        
        for (int i = 0; i < sqlInsertsDbKwDictionary.length; i++) {
            String sql = sqlInsertsDbKwDictionary[i];
        
            _sql = new StringBuffer();
            _sql.append(sql);
        
            logger.debug(_sql.toString());
            ps = conn.prepareStatement(_sql.toString());        
            ps.executeUpdate();

        }
        
        /* **/             
    }

    private void dropDataDbKwDictionary() throws SQLException, JdbcConnectionException {
        /* 
         * drops the test data for TestDbKeywordDictionary tests 
         * */
        connManager = DbConnManager.getInstance();
        conn = connManager.getObsConnection();
    
    
        // 	create the temporary table
        PreparedStatement ps = null;
        StringBuffer _sql = new StringBuffer();
    
        _sql.append(sqlDropDbKwDictionary);
    
        logger.debug(_sql.toString());
        ps = conn.prepareStatement(_sql.toString());        
        ps.executeUpdate();
    }
    
    
    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     * @throws Exception
     */
    protected void setUp() throws Exception {
        setUpDataDbKwDictionary();
    }
    
    
    
    
    /* (non-Javadoc)
     * @see junit.extensions.TestSetup#tearDown()
     */
    protected void tearDown() throws Exception {
        dropDataDbKwDictionary();
    }
    /**
     * Add TestCase to a suite and return the suite.
     * @return test suite
     */

    public static TestSuite suite() {
	   TestSuite suite = new TestSuite();	   	   	   
	   suite.addTest(
	           new TestDbKeywordsDictionarySetup(new TestSuite(TestDbKeywordDictionary.class)));
	   
	       
	return suite;
    }   
    
    /**
     * @param arg0
     */
    public TestDbKeywordsDictionarySetup(Test arg0) {
        super(arg0);
       
    }

    public static void main(String[] args) throws OCAPropertiesException {
        
        OCAProperties props = OCAProperties.getInstance();
        props.loadProperties(propertiesFile);
        junit.textui.TestRunner.run(suite());
    }

    



}





