/*
 * Created on Jan 03, 2006
 */
package org.eso.oca.parser;

import java.io.FileInputStream;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;
import org.eso.oca.fits.OCAFile;
import org.eso.oca.fits.OCAFileFactory;
import org.eso.oca.fits.TypeDeterminationException;
import org.eso.oca.fits.TypedHeaderCard;

/**
 * @author szampier
 */
public class TestOCAFileToDB {
    private static Logger logger = Logger.getLogger(TestOCAFileToDB.class);

    final static String USAGE = "Usage: TestOCAFileToDB <oca rules> <instrume> <dir>";

    final static int VARCHAR_LENGTH = 50;

    final static String GO = "go";

    public static void main(String[] args) {
        String ocaRules = "";
        String dataDir = "";
        String instrumentName = "";

        try {
            ocaRules = new String(args[0]);
            instrumentName = new String(args[1]);
            dataDir = new String(args[2]);
        } catch (Throwable t) {
            System.out.println(USAGE);
            System.exit(1);
        }

        try {
            /* Configure logging. */
            // BasicConfigurator.configure();
            // ocaLogger.setLevel(Level.INFO);
            // logger.setLevel(Level.INFO);
            logger.info("OCA Rules: " + ocaRules);
            logger.info("Instrument: " + instrumentName);
            logger.info("Dir: " + dataDir);

            /* Parse the rules. */
            OcaParser parser = new OcaParser(new FileInputStream(ocaRules));
            ASTStart start = parser.Start();

            /*
             * Find out which keyword values are needed from the HDR file.
             */
            String[] requestedData = start.getRequestedData();

            OCAFileFactory rawFactory = new OCAFileFactory(dataDir);
            OCAFile[] rawFiles = rawFactory.createOCAFiles(0, requestedData,
                    null);

            Hashtable values = new Hashtable();
            Hashtable types = new Hashtable();
            Hashtable mapper = new Hashtable();
            Vector insertIST = new Vector();
            for (int i = 0; i < rawFiles.length; ++i) {
                StringBuffer dbFields = new StringBuffer();
                StringBuffer dbValues = new StringBuffer();
                values = rawFiles[i].getCards();
                for (Iterator it = values.keySet().iterator(); it.hasNext();) {
                    String fitsKey = (String) it.next();
                    Object value = values.get(fitsKey);
                    String dbKey = fitsKey.replace('.', '_').toLowerCase();
                    dbKey = dbKey.replace('-', '_');
                    mapper.put(fitsKey, dbKey);
                    if (dbKey.equals("INSTRUME"))
                        value = instrumentName;
                    String valueType = TypedHeaderCard
                            .findFITSTypeString(value);
                    String keyType = (String) types.get(fitsKey);
                    if (keyType == null
                            || valueType.equals("UNDEFINED") == false) {
                        types.put(fitsKey, valueType);
                    }
                    dbFields.append(dbKey + ",");
                    if (valueType.equals("UNDEFINED")) {
                        dbValues.append("NULL,");
                    } else if (valueType.equals("STRING")) {
                        dbValues.append("'" + value + "',");
                    } else {
                        dbValues.append(value + ",");
                    }
                }
                // remove trailing ","
                dbValues.setLength(dbValues.length() - 1);
                dbFields.setLength(dbFields.length() - 1);
                String sql = "INSERT " + instrumentName + "(" + dbFields
                        + ") VALUES (" + dbValues + ")";
                insertIST.add(sql);
            }

            Vector deleteTablesAndColumns = new Vector();
            Vector insertTablesAndColumns = new Vector();

            deleteTablesAndColumns.add("DELETE tables WHERE instrume = '"
                    + instrumentName + "'");
            deleteTablesAndColumns.add("DELETE columns WHERE tname = '"
                    + instrumentName + "'");
            insertTablesAndColumns
                    .add("INSERT tables (instrume,name) VALUES ('"
                            + instrumentName + "','" + instrumentName + "')");

            String dropTable = "DROP TABLE " + instrumentName;
            StringBuffer createTable = new StringBuffer();
            createTable.append("CREATE TABLE " + instrumentName + " (");
            for (Iterator it = types.keySet().iterator(); it.hasNext();) {
                String fitsKey = (String) it.next();
                String dbKey = (String) mapper.get(fitsKey);
                String fitsType = (String) types.get(fitsKey);
                String dbType = null;
                String dbTypeLength = null;
                String defaultValue = "NULL";
                if (fitsType.equals("UNDEFINED") || fitsType.equals("UNTYPED")) {
                    logger.warn("Could not determine type of keyword "
                            + fitsKey + ". Using varchar.");
                    dbType = "varchar";
                    dbTypeLength = "50";
                } else if (fitsType.equals("STRING")) {
                    dbType = "varchar";
                    dbTypeLength = "50";
                } else if (fitsType.equals("INTEGER")) {
                    dbType = "integer";
                } else if (fitsType.equals("FLOAT")) {
                    dbType = "float";
                } else {
                    throw new TypeDeterminationException("Unsupported type : "
                            + fitsType);
                }
                if (dbTypeLength == null) {
                    dbTypeLength = "NULL";
                }
                insertTablesAndColumns
                        .add("INSERT columns (tname,name,type,length,keyword) VALUES ('"
                                + instrumentName
                                + "','"
                                + dbKey
                                + "','"
                                + dbType
                                + "',"
                                + dbTypeLength
                                + ",'"
                                + fitsKey
                                + "')");
                if (dbKey.equals("DP_ID")) {
                    defaultValue = "NOT NULL";
                }
                if (dbType.equals("varchar")) {
                    createTable.append(dbKey + " " + dbType + "("
                            + dbTypeLength + ") " + defaultValue + ",");
                } else {
                    createTable.append(dbKey + " " + dbType + " "
                            + defaultValue + ",");
                }
            }
            createTable.setLength(createTable.length() - 1);
            createTable.append(")");

            System.out.println(dropTable);
            System.out.println(GO);
            System.out.println(createTable);
            System.out.println(GO);
            for (Iterator it = deleteTablesAndColumns.iterator(); it.hasNext();) {
                System.out.println(it.next());
            }
            System.out.println(GO);
            for (Iterator it = insertTablesAndColumns.iterator(); it.hasNext();) {
                System.out.println(it.next());
            }
            System.out.println(GO);
            for (Iterator it = insertIST.iterator(); it.hasNext();) {
                System.out.println(it.next());
            }
            System.out.println(GO);
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }
}
