package org.eso.util.dal;

import java.util.List;

/**
 * DAO interface for NgasDisk objects
 * 
 * @author Vincenzo Forchi` <vforchi@eso.org>
 *
 */

public interface NgasDiskDAO {
	
	/**
	 * Find all records in ngas_disks table
	 * 
	 * @return a list of NgasDisk objects
	 * @throws DAOException
	 */
	public List<NgasDisk> findAll() throws DAOException;
	
	/**
	 * Find all records in ngas_disks table matching a given Id
	 *  
	 * @param Id the unique identifier of the disk
	 * @return a list of NgasDisk objects
	 * @throws DAOException
	 */
	public List<NgasDisk> findByDiskId(String id) throws DAOException ;
	
	/**
	 * Find all records in ngas_disks table matching a given Label
	 * 
	 * @param Label The human readable disk label
	 * @return a list of NgasDisk objects
	 * @throws DAOException
	 */
	public List<NgasDisk> findByLogicalName(String name) throws DAOException ;
	
	/**
	 * Find all entries in ngas_files relative to a given disk
	 * @param id the disk_id
	 * @return a list of NgasFile objects
	 * @throws DAOException
	 */
	public List<NgasFile> findFilesByDiskId(String id) throws DAOException;
	
	/**
	 * Find last inserted entry
	 * @return the last NgasDisk inserted
	 * @throws DAOException
	 */
	public NgasDisk findLastEntry() throws DAOException;
	
	/**
	 * Saves a new entry in ngas_disks table
	 * 
	 * @param entry The new entry
	 * @throws DAOException
	 */
	public void save(NgasDisk entry) throws DAOException;

	/**
	 * Saves a new entry in ngas_disks table if it doesn't already exist
	 * 
	 * @param entry The new entry
	 * @throws DAOException
	 */
	public void saveIfNotFound(NgasDisk entry) throws DAOException;
	
	/**
	 * Updates a new entry in ngas_disks table
	 * 
	 * @param entry The entry to be updated
	 * @throws DAOException
	 */
	public void update(NgasDisk entry) throws DAOException;
	
	/**
	 * Deletes a entry from ngas_disks table
	 * 
	 * @param entry The entry to be deleted
	 * @throws DAOException
	 */
	public void delete(NgasDisk entry) throws DAOException;
	
}
