package org.eso.util.dal;

import java.util.List;
import java.util.ArrayList;
import java.math.BigDecimal;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Connection;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import org.eso.util.dal.SybaseConnectionService;

/**
 * Sybase implementation of the NgasDisk DAO
 * 
 * @author Vincenzo Forchi` <vforchi@eso.org>
 *
 */
public class NgasDiskDAOSyb implements NgasDiskDAO {
	
	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(NgasDiskDAOSyb.class);
    
    /** database's connection parameters */
    private String d_dbUrl			= null;
    private String d_dbUsername		= null;
    private String d_dbPassword		= null;
    
    /** ngas_disks table */
    private String d_dbTable		= null;
    /** ngas_files table */
    private String d_dbFilesTable	= null;
    
    private Connection d_connection	= null;
    
	public NgasDiskDAOSyb(String dbUrl, String dbUsername, String dbPassword, String dbTable, String dbFilesTable) {
		d_dbUrl			= dbUrl;
		d_dbUsername	= dbUsername;
		d_dbPassword	= dbPassword;
		d_dbTable		= dbTable;
		d_dbFilesTable	= dbFilesTable;
	}
	
	public NgasDiskDAOSyb(Connection connection, String dbTable, String dbFilesTable) {
		d_connection	= connection;
		d_dbTable		= dbTable;
		d_dbFilesTable	= dbFilesTable;
	}
		
	public List<NgasDisk> findAll() throws DAOException {
		String query = "select * from " + d_dbTable;
		return executeQuery(query);
	}
	
	public List<NgasDisk> findByDiskId(String id) throws DAOException {
		String query = "select * from " + d_dbTable + " where disk_id='" + id + "'";
		return executeQuery(query); 
	}
	
	public List<NgasDisk> findByLogicalName(String name) throws DAOException {
		String query = "select * from " + d_dbTable + " where logical_name='" + name + "'";
		return executeQuery(query);		
	}
	
	public List<NgasFile> findFilesByDiskId(String Id) throws DAOException {
		String query = "select * from " + d_dbFilesTable + " where disk_id='" + Id + "'";
		try {
			List<NgasFile> res = null;
			if (d_connection == null)
			{
				Connection conn = SybaseConnectionService.getConnection(d_dbUrl, d_dbUsername, d_dbPassword); 
				res = NgasFileDAOSyb.toNgasFileList(SybaseConnectionService.executeStatement(conn, query));
				SybaseConnectionService.closeConnection(conn);
			}
			else
				res = NgasFileDAOSyb.toNgasFileList(SybaseConnectionService.executeStatement(d_connection, query));
			return res;
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public NgasDisk findLastEntry() throws DAOException {
		String query = "select * from " + d_dbTable + " having installation_date=max(installation_date)";
		List<NgasDisk> disks = executeQuery(query);
		return disks.get(disks.size()-1);
	}
	
	public void save(NgasDisk entry) throws DAOException {
		NgasDisk.validate(entry);
		String query = "insert into " + d_dbTable + " " + formatForInsert(entry);
		executeQuery(query);
	}
	
	public void saveIfNotFound(NgasDisk entry) throws DAOException {
		NgasDisk.validate(entry);
		if (findByDiskId(entry.getDiskId()).size() == 0)
			save(entry);
		else
			throw new DAOException("Record for disk_id=" + entry.getDiskId() + " is already in the database");
	}
	
	public void update(NgasDisk entry) throws DAOException {
		NgasDisk.validate(entry);
		String query = "update " + d_dbTable + " " + formatForUpdate(entry) + " where disk_id='" + entry.getDiskId() + "'";
		executeQuery(query);
	}
	
	public void delete(NgasDisk entry) throws DAOException {
		NgasDisk.validate(entry);
		String query = "delete from " + d_dbTable + " where disk_id='" + entry.getDiskId() + "'";
		executeQuery(query);
	}
	
	/**
	 * Convenience method used for database access
	 * @param query
	 * @return a List of NgasDisk objects
	 * @throws DAOException
	 */
	private List<NgasDisk> executeQuery(String query) throws DAOException {
		try {
			List<NgasDisk> res = null;
			if (d_connection == null)
			{
				Connection conn = SybaseConnectionService.getConnection(d_dbUrl, d_dbUsername, d_dbPassword); 
				res = toNgasDiskList(SybaseConnectionService.executeStatement(conn, query));
				SybaseConnectionService.closeConnection(conn);
			}
			else
				res = toNgasDiskList(SybaseConnectionService.executeStatement(d_connection, query));
			return res;
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	/**
	 * Converts the result of a database query to a list of NgasDisk objects
	 * 
	 * @param rs the result of the query
	 * @return a list of NgasDisk objects
	 * @throws DAOException
	 */
	public static List<NgasDisk> toNgasDiskList(ResultSet rs) throws DAOException {
		List<NgasDisk> res = new ArrayList<NgasDisk>();
		try {
			while (rs.next()) {
				NgasDisk disk = new NgasDisk();

				disk.setDiskId(rs.getString("disk_id"));
				disk.setArchive(rs.getString("archive"));
				disk.setInstallationDate(rs.getDate("installation_date"));
				disk.setType(rs.getString("type"));
				disk.setManufacturer(rs.getString("manufacturer"));
				disk.setLogicalName(rs.getString("logical_name"));
				disk.setHostId(rs.getString("host_id"));
				disk.setSlotId(rs.getString("slot_id"));
				disk.setMounted(rs.getShort("mounted"));
				disk.setMountPoint(rs.getString("mount_point"));
				disk.setNumberOfFiles(rs.getInt("number_of_files"));
				disk.setAvailableMb(rs.getInt("available_mb"));
				disk.setBytesStored(rs.getBigDecimal("bytes_stored"));
				disk.setCompleted(rs.getShort("completed"));
				disk.setCompletionDate(rs.getDate("completion_date"));
				disk.setChecksum(rs.getString("checksum"));
				disk.setTotalDiskWriteTime(rs.getFloat("total_disk_write_time"));
				disk.setLastCheck(rs.getDate("last_check"));
				disk.setLastHostId(rs.getString("last_host_id"));

				res.add(disk);
			}
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (NullPointerException e) {}		
		return res;
	}
	
	/**
	 * Converts an NgasDisk object to a string that can be use in an update query 
	 * @param disk
	 * @return
	 */
	public static String formatForUpdate(NgasDisk disk)
	{
		String res = "";

		if (disk.getDiskId() != null)
			res += " set disk_id='" + disk.getDiskId() + "'";
		if (disk.getArchive() != null)
			res += ", archive='" + disk.getArchive() + "'";
		if (disk.getInstallationDate() != null)
			res += ", installation_date='" + disk.getInstallationDate() + "'";
		if (disk.getType() != null)
			res += ", type='" + disk.getType() + "'";
		if (disk.getManufacturer() != null)
			res += ", manufacturer='" + disk.getManufacturer() + "'";
		if (disk.getLogicalName() != null)
			res += ", logical_name='" + disk.getLogicalName() + "'";
		if (disk.getHostId() != null)
			res += ", host_id='" + disk.getHostId() + "'";
		else
			res += ", host_id=null";
		if (disk.getSlotId() != null)
			res += ", slot_id='" + disk.getSlotId() + "'";
		if (disk.getMounted() != null)
			res += ", mounted=" + disk.getMounted();
		if (disk.getMountPoint() != null)
			res += ", mount_point='" + disk.getMountPoint() + "'";
		if (disk.getNumberOfFiles() != null)
			res += ", number_of_files=" + disk.getNumberOfFiles();
		if (disk.getAvailableMb() != null)
			res += ", available_mb=" + disk.getAvailableMb();
		if (disk.getBytesStored() != null)
			res += ", bytes_stored=" + disk.getBytesStored();
		if (disk.getCompleted() != null)
			res += ", completed=" + disk.getCompleted();
		if (disk.getCompletionDate() != null)
			res += ", completion_date='" + disk.getCompletionDate() + "'";
		if (disk.getChecksum() != null)
			res += ", checksum='" + disk.getChecksum() + "'";
		if (disk.getTotalDiskWriteTime() != null)
			res += ", total_disk_write_time=" + disk.getTotalDiskWriteTime();
		if (disk.getLastCheck() != null)
			res += ", last_check='" + disk.getLastCheck() + "'";
		if (disk.getLastHostId() != null)
			res += ", last_host_id='" + disk.getLastHostId() + "'";
		
		return res;
	}
	
	/**
	 * Converts an NgasDisk object to a String that can be used in an insert query
	 * @param disk
	 * @return
	 */
	public static String formatForInsert(NgasDisk disk)
	{
		String first = "(";
		String second = "(";

		if (disk.getDiskId() != null)
		{
			first	+= "disk_id";
			second	+= "'" + disk.getDiskId() + "'";
		}
		if (disk.getArchive() != null)
		{
			first	+= ", archive";
			second	+= ", '" + disk.getArchive() + "'";
		}
		if (disk.getInstallationDate() != null)
		{
			first	+= ", installation_date";
			second	+= ", '" + disk.getInstallationDate() + "'"; 
		}
		if (disk.getType() != null)
		{
			first	+= ", type";
			second	+= ", '" + disk.getType() + "'";
		}
		if (disk.getManufacturer() != null)
		{
			first	+= ", manufacturer";
			second	+= ", '" + disk.getManufacturer() + "'";
		}
		if (disk.getLogicalName() != null)
		{
			first	+= ", logical_name";
			second	+= ", '" + disk.getLogicalName() + "'";
		}
		if (disk.getHostId() != null)
		{
			first	+= ", host_id";
			second	+= ", '" + disk.getHostId() + "'";
		}
		if (disk.getSlotId() != null)
		{
			first	+= ", slot_id";
			second	+= ", '" + disk.getSlotId() + "'";
		}
		if (disk.getMounted() != null)
		{
			first	+=	", mounted";
			second	+= 	", " + disk.getMounted();
		}
		if (disk.getMountPoint() != null)
		{
			first	+= ", mount_point";
			second	+= ", '" + disk.getMountPoint() + "'";
		}
		if (disk.getNumberOfFiles() != null)
		{
			first	+=	", number_of_files";
			second	+= 	", " + disk.getNumberOfFiles();
		}
		if (disk.getAvailableMb() != null)
		{
			first	+=	", available_mb";
			second	+= 	", " + disk.getAvailableMb();
		}
		if (disk.getBytesStored() != null)
		{
			first	+=	", bytes_stored";
			second	+= 	", " + disk.getBytesStored();
		}
		if (disk.getCompleted() != null)
		{
			first	+= ", completed";
			second	+= ", " + disk.getCompleted();
		}
		if (disk.getCompletionDate() != null)
		{
			first	+=	", completion_date";
			second	+= 	", '" + disk.getCompletionDate() + "'";
		}
		if (disk.getChecksum() != null)
		{
			first	+= ", checksum";
			second	+= ", '" + disk.getChecksum() + "'";
		}
		if (disk.getTotalDiskWriteTime() != null)
		{
			first	+=	", total_disk_write_time";
			second	+= 	", " + disk.getTotalDiskWriteTime();
		}
		if (disk.getLastCheck() != null)
		{
			first	+=	", last_check";
			second	+= 	", '" + disk.getLastCheck() + "'";
		}
		if (disk.getLastHostId() != null)
		{
			first	+=	", last_host_id";
			second	+= 	", '" + disk.getLastHostId() + "'";
		}
		
		first	+= ")";
		second	+= ")";
		
		return first + " values " + second;
	}
	
	/**
	 * Test program
	 * @param args
	 */
	public static void main (String[] args) {
		BasicConfigurator.configure();
		NgasDiskDAO dao = new NgasDiskDAOSyb("acdbdev.hq.eso.org:6789", "ngas", "ngas_pw", "ngastst1..ngas_disks", null);
		try {
			/*for (NgasDisk d: dao.findAll())
				System.out.println(d.toString());*/
			
			for (NgasDisk d: dao.findByDiskId("tmp-NGAS_ngasdev3-Volume3-Rep-6"))
				System.out.println(d.toString());
			
			
			NgasDisk disk = new NgasDisk();
			disk.setDiskId("test-disk-01");
			disk.setArchive("TEST");
			disk.setInstallationDate(Date.valueOf("2000-01-01"));
			disk.setType("");
			disk.setLogicalName("USB-000001");
			disk.setNumberOfFiles(10);
			disk.setAvailableMb(80000);
			disk.setBytesStored(BigDecimal.valueOf(2000000000));
			disk.setCompleted(Integer.valueOf(0).shortValue());
			
			//dao.saveIfNotFound(disk);
			dao.update(disk);
			
			for (NgasDisk d: dao.findByLogicalName("USB-000001"))
				System.out.println(d.toString());
			
			
			//dao.update(dao.findByLogicalName("M-001006").get(0));
			//dao.save(dao.findByLogicalName("M-001006").get(0));
			//dao.delete(dao.findByLogicalName("USB-000001").get(0));
			//dao.saveIfNotFound(dao.findByLogicalName("M-001006").get(0));
			
		} catch (DAOException e) {
			System.out.println(e.getMessage());
		}
	}
}
