package org.eso.util.dal;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;

import org.apache.log4j.Logger;
import org.eso.util.dal.ConnectionServiceException;

public class PackageInfoDAOSyb implements PackageInfoDAO {

	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(PackageInfoDAOSyb.class);
    
    /** The name of this class for logging purposes. */
	private static final String classLogName = "PackageInfoDAOSyb";
	
	/* table columns */
	public static final String PACKAGEID_KW = "pkg_id";
	public static final String RUNID_KW = "run_id";
	public static final String INSTRUMENT_KW = "instrume";
	public static final String ASSOCIATIONDATA_KW = "assoc_data";
	public static final String STATUS_KW = "pkg_status";
	
	private Connection	d_connection = null;
	private String		d_dbTable = null;
	private String		d_idGenProcedure = null;
	
	public PackageInfoDAOSyb(Connection conn, String dbTable, String idGenProcedure) {
		d_connection 	= conn;
		d_dbTable		= dbTable;
		d_idGenProcedure = idGenProcedure;
	}

	public Integer getPackageId(String runId, String instrument) throws DAOException {
		PackageInfo info;
		if ((info = find(runId, instrument)) != null)
			return info.getPackageId();
		else {
			try {
				CallableStatement cstmt = null;
				d_connection.setAutoCommit(true);
				cstmt = d_connection.prepareCall("{ call " + d_idGenProcedure + "(?,?)}");

				cstmt.setString(1, "PI_PACK");
				cstmt.registerOutParameter(2, Types.INTEGER);

				cstmt.execute();
				return cstmt.getInt(2);
			} catch (SQLException e) {
				throw new DAOException(e.getMessage());
			}
		}
	}
	
	public void delete(PackageInfo pkg) throws DAOException {
		String query = "DELETE FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + " = " + pkg.getPackageId();
		
		try {
			SybaseConnectionService.executeStatement(d_connection, query);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public PackageInfo find(String run_id, String instrument) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + RUNID_KW + " = '" + run_id + "' AND " + INSTRUMENT_KW + " = '" + instrument + "'";
		
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			if (rs.next()) {
				PackageInfo result = new PackageInfo();
				
				result.setPackageId(rs.getInt(PACKAGEID_KW));
				result.setRunId(rs.getString(RUNID_KW));
				result.setInstrument(rs.getString(INSTRUMENT_KW));
				result.setAssocData(rs.getString(ASSOCIATIONDATA_KW));
				result.setStatus(PackageInfo.Status.valueOf(rs.getString(STATUS_KW)));
				
				return result;
			} else { 
				return null;
			}
			
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void insert(PackageInfo pkg) throws DAOException {

		String columns = "(";
		String values  = "(";
		
		columns += PACKAGEID_KW + "," + RUNID_KW + "," + INSTRUMENT_KW + "," + STATUS_KW;
		if (pkg.getAssocData() != null)
			columns += "," + ASSOCIATIONDATA_KW;
		columns += ")";
		
		values += pkg.getPackageId() + ",'" + pkg.getRunId() + "','" + pkg.getInstrument() + "','" + pkg.getStatus().toString() + "'";
		if (pkg.getAssocData() != null)
			values += ",'" + pkg.getAssocData() + "'";
		values += ")";
			
		String query = "INSERT INTO " + d_dbTable + " " + columns + " VALUES " + values;
		
		try {
			SybaseConnectionService.executeStatement(d_connection, query);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public void insertIfNotFound(PackageInfo pkg) throws DAOException {
		if (find(pkg.getRunId(), pkg.getInstrument()) != null) {
			logger.info("Package information already in the database");
		} else {
			logger.info("Ingesting package information");
			insert(pkg);
		}
	}
}

