package org.eso.util.dal;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;

public class PackageReportsDAOSyb implements PackageReportsDAO {
	
	/** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(PackageReportsDAOSyb.class);
    
    /** The name of this class for logging purposes. */
	private static final String classLogName = "PackageReportsDAOSyb";
	
	/* table columns */
	public static final String PACKAGEID_KW = "pkg_id";
	public static final String FILEURL_KW = "file_url";
	public static final String REPORT_KW = "report";
	
	private Connection	d_connection = null;
	private String		d_dbTable = null;
	
	public PackageReportsDAOSyb(Connection conn, String dbTable) throws DAOException {
		d_connection 	= conn;
		d_dbTable		= dbTable;
	}
	
	public void delete(PackageReport report) throws DAOException {
		String query = "DELETE FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + report.getPackageId();
		query += " AND " + FILEURL_KW + "='" + report.getFileUrl() + "'"; 

		try {
			SybaseConnectionService.executeStatement(d_connection, query);
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public void delete(List<PackageReport> reports) throws DAOException {
		/* TODO: improve this */
		for (PackageReport report: reports)
			delete(report);
	}

	public PackageReport find(Integer pkgId, String url) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + pkgId;
		query += " AND " + FILEURL_KW + "='" + url + "'"; 

		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			
			PackageReport report = null;
			
			if (rs.next()) {
				report = new PackageReport();
				report.setPackageId(rs.getInt(PACKAGEID_KW));
				report.setFileUrl(rs.getString(FILEURL_KW));
				report.setReport(readAll(rs.getBinaryStream(REPORT_KW)));
			}
			
			return report;
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (IOException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void insert(PackageReport report) throws DAOException {
		String columns = "(" + PACKAGEID_KW + "," + FILEURL_KW + "," + REPORT_KW + ")";
		String values = "(" + report.getPackageId() + ",'" + report.getFileUrl() + "', ? )";
		String query = "INSERT INTO " + d_dbTable + " " + columns + " VALUES " + values;

		try {
			//SybaseConnectionService.executeStatement(d_connection, query);
			
			PreparedStatement stmt = d_connection.prepareStatement(query);
			
			InputStream binaryStream = new ByteArrayInputStream(report.getReport());
			stmt.setBinaryStream(1, binaryStream, (int) report.getReport().length);
            stmt.execute();
            binaryStream.close();
			
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (FileNotFoundException e) {
			throw new DAOException(e.getMessage());
		} catch (IOException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public void insertOrUpdate(PackageReport report) throws DAOException {
		if (find(report.getPackageId(), report.getFileUrl()) == null)
			insert(report);
		else
			update(report);
	}

	public void update(PackageReport report) throws DAOException {
		String query = "UPDATE " + d_dbTable + " SET " + REPORT_KW + " = ? WHERE "; 
		query += PACKAGEID_KW + "=" + report.getPackageId() + " AND " + FILEURL_KW + "='" + report.getFileUrl() + "'";

		try {
			//SybaseConnectionService.executeStatement(d_connection, query);
			
			PreparedStatement stmt = d_connection.prepareStatement(query);
			
			InputStream binaryStream = new ByteArrayInputStream(report.getReport());
			stmt.setBinaryStream(1, binaryStream, (int) report.getReport().length);
            stmt.execute();
            binaryStream.close();
			
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (FileNotFoundException e) {
			throw new DAOException(e.getMessage());
		} catch (IOException e) {
			throw new DAOException(e.getMessage());
		}
	}

	public List<PackageReport> find(Integer pkgId) throws DAOException {
		String query = "SELECT * FROM " + d_dbTable + " WHERE " + PACKAGEID_KW + "=" + pkgId;
	
		List<PackageReport> result = new ArrayList<PackageReport>();
		
		try {
			ResultSet rs = SybaseConnectionService.executeStatement(d_connection, query);
			
			while (rs.next()) {
				PackageReport report = new PackageReport();
				report.setPackageId(rs.getInt(PACKAGEID_KW));
				report.setFileUrl(rs.getString(FILEURL_KW));
				report.setReport(readAll(rs.getBinaryStream(REPORT_KW)));
				result.add(report);
			}
			
			return result;
			
		} catch (ConnectionServiceException e) {
			throw new DAOException(e.getMessage());
		} catch (SQLException e) {
			throw new DAOException(e.getMessage());
		} catch (IOException e) {
			throw new DAOException(e.getMessage());
		}
	}
	
	public static byte[] readAll(InputStream inputStream) throws IOException {
		ByteArrayOutputStream bout = new ByteArrayOutputStream();
		byte [] buffer = new byte[1024];
		while (true) {
			int len = inputStream.read(buffer);
			if(len < 0)
				break;
			bout.write(buffer, 0, len);
		}
		return bout.toByteArray();
	}
}
