package org.eso.util.dal;

import java.sql.Date;
import java.util.Map;
import java.util.HashMap;
import java.util.EnumSet;

public class ReplicationRequest {

	public enum Status {
		SCHEDULED(0),
		PENDING(1),
		REQUESTED(2),
		READY(3),
		REPLICATING(4),
		REPLICATED(5),
		REPORTED(6),
		ERROR(-1);

		private static final Map<Integer,Status> lookup = new HashMap<Integer,Status>();

		static {
			for(Status s : EnumSet.allOf(Status.class))
				lookup.put(s.getCode(), s);
		}

		private int code;

		private Status(int code) {
			this.code = code;
		}

		public int getCode() { return code; }

		public static Status get(int code) { 
			return lookup.get(code); 
		}
	}
	
	private String		d_fileId			= null;
	private Integer		d_fileVersion		= null;
	private String		d_diskId			= null;
	private String		d_submitterId		= null;
	private Date		d_submissionTime	= null;
	private Integer		d_priority			= null;
	private Integer		d_statusId			= null;
	private Date		d_lastUpdateTime	= null;
	private String 		d_message			= null;
	
	public String getFileId() {
		return d_fileId;
	}
	
	public void setFileId(String id) {
		d_fileId = id;
	}
	
	public Integer getFileVersion() {
		return d_fileVersion;
	}
	
	public void setFileVersion(Integer version) {
		d_fileVersion = version;
	}
	
	public String getDiskId() {
		return d_diskId;
	}
	
	public void setDiskId(String id) {
		d_diskId = id;
	}
	
	public String getSubmitterId() {
		return d_submitterId;
	}
	
	public void setSubmitterId(String id) {
		d_submitterId = id;
	}
	
	public Date getSubmissionTime() {
		return d_submissionTime;
	}
	
	public void setSubmissionTime(Date time) {
		d_submissionTime = time;
	}
	
	public Integer getPriority() {
		return d_priority;
	}
	
	public void setPriority(Integer d_priority) {
		this.d_priority = d_priority;
	}
	
	public Status getStatusId() {
		return Status.get(d_statusId);
	}
	
	public void setStatusId(Status status) {
		d_statusId = status.getCode();
	}
	
	public Date getLastUpdateTime() {
		return d_lastUpdateTime;
	}
	
	public void setLastUpdateTime(Date updateTime) {
		d_lastUpdateTime = updateTime;
	}
	
	public String getMessage() {
		return d_message;
	}
	
	public void setMessage(String d_message) {
		this.d_message = d_message;
	}
	
	/**
	 * Checks a ReplicationRequest instance for mandatory fields
	 * 
	 * @param rr the instance to be checked
	 * @throws DAOException if there are some missing fields
	 */
	public static void validate(ReplicationRequest disk) throws DAOException {
		String fields = "";
		if (disk.getFileId() == null)
			fields += ", file_id";
		if (disk.getSubmitterId() == null)
			fields += ", submitter_id";
		
		if (fields.length() > 0)
			throw new DAOException("Mandatory field(s) " + fields.substring(2) + " missing");	
	}
	
	@Override
	public String toString() {
		String res = "ReplicationRequest instance";
		res += "\n" + "\tfile_id " + d_fileId;
		res += "\n" + "\tfile_version " + d_fileVersion;
		res += "\n" + "\tdisk_id " + d_diskId;
		res += "\n" + "\tsubmitter_id " + d_submitterId;
		res += "\n" + "\tsubmission_time " + d_submissionTime;
		res += "\n" + "\tpriority " + d_priority;
		res += "\n" + "\tstatus_id " + d_statusId;
		res += "\n" + "\tlast_update_time " + d_lastUpdateTime;
		res += "\n" + "\tmessage " + d_message;
		return res;
	}

}
