package org.eso.util.datatransfer;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

/**
 * This class implements basic TransferRequest validation. It checks if the mandatory
 * keywords are present.
 * 
 * @author szampier
 * 
 */
public class DefaultTransferRequestValidator implements TransferRequestValidator {

	/** The mandatory keywords */
	static private final String[] mandatoryKeywords = { TransferRequest.FILEID,
		TransferRequest.FILENAME, TransferRequest.SIZE };

	/** The Log4j logger */
	static Logger logger = Logger.getLogger(DefaultTransferRequestValidator.class);

	/**
	 * Returns true if all the mandatory keywords are present in the given
	 * TransferRequest. False otherwise.
	 */
	public boolean isValid(TransferRequest tr) {
		boolean ret = true;
		for (int i = 0; i < mandatoryKeywords.length; i++) {
			String keyword = mandatoryKeywords[i];
			if (!tr.containsKey(keyword)) {
				ret = false;
				String message = "DefaultTransferRequestValidator::isValid() - "
						+ " Mandatory keyword " + keyword + " is missing.";
				logger.error(message);
			}
		}
		return ret;
	}

	/**
	 * Test the main functionality of the DefaultTransferRequestValidator class.
	 * 
	 * @param args
	 * @throws TransferRequestException 
	 */
	public static void main(String[] args) throws TransferRequestException {
		// Basic Log4j configuration
		BasicConfigurator.configure();

		TransferRequest tr = new TransferRequest();
		tr.setProperty(TransferRequest.FILENAME, "pippo");
		DefaultTransferRequestValidator trv = new DefaultTransferRequestValidator();
		if (!trv.isValid(tr))
			logger.error("TransferRequest is not valid");
		else
			logger.info("TransferRequest is valid.");
	}
}
