package org.eso.util.datatransfer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import org.apache.log4j.Logger;

/**
 * This class reads the given input buffer into a string buffer.
 * 
 * @author szampier
 * 
 */
class StreamConsumer extends Thread {
	/** The input stream */
	InputStream inputStream;

	/** The input string buffer */
	StringBuffer inputStringBuffer = new StringBuffer();

	/** This flag indicates the status of this object */
	boolean good = true;

	/** The Log4j logger */
	static Logger logger = Logger.getLogger(StreamConsumer.class);

	/** The name of this class for logging purposes. */
	private static final String classLogName = "StreamConsumer";

	/** End Of Line */
	private static final String EOL = System.getProperty("line.separator");

	/**
	 * Constructor
	 * 
	 * @param is
	 *            The input stream to read from.
	 */
	StreamConsumer(InputStream is) {
		this.inputStream = is;
	}

	/**
	 * Return the string representation of this object.
	 * 
	 * @return the string representation of this object.
	 */
	public String toString() {
		final String methodLogName = classLogName + "::toString()";

		if (isAlive()) {
			try {
				join();
			} catch (InterruptedException e) {
				logger.error(methodLogName
						+ " - InterruptedException caught while "
						+ " waiting for the thread to finish: "
						+ e.getMessage());
			}
		}
		return inputStringBuffer.toString();
	}

	/**
	 * Return the status of this object after reading the input stream.
	 * 
	 * @return true if everything went fine, false if an error has occourred.
	 */
	public boolean good() {
		return good;
	}

	/**
	 * Read each line of the input stream into a string buffer. Note that this
	 * is performed in a separate thread of execution.
	 */
	public void run() {
		InputStreamReader isr = new InputStreamReader(inputStream);
		BufferedReader br = new BufferedReader(isr);
		// TODO: Use a character buffer instead?
		String line;
		try {
			while ((line = br.readLine()) != null) {
				inputStringBuffer.append(line + EOL);
			}
		} catch (IOException e) {
			logger.error(e.getMessage());
			good = false;
		}
	}
}