package org.eso.util.datatransfer;

import org.apache.log4j.Logger;
import org.eso.util.misc.SystemCommand;

public class UnpackCommand extends SystemCommand {

    /** Apache Log4J logger. */
    static final Logger logger = Logger.getLogger(UnpackCommand.class);
    /** The name of this class for logging purposes. */
    private static final String classLogName = "UnpackCommand";

    private final String DIR_KEYWORD = "\\$DIR";

    private final String PACKAGE_KEYWORD = "\\$PACKAGE";

    /**
     * Constructor
     * 
     * @param unpackCmdFormat
     *            format of the command to extract files in the package
     * @param unpackDir
     *            directory where extracted files will be stored
     * @param packageName
     *            name of the package 
     * @param timeout
     *            command timeout
     */
    public UnpackCommand(String unpackCmdFormat, String unpackDir,
            String packageName, long timeout) {
        super();
        final String methodLogName = classLogName + "::UnpackCommand()";
        if ((unpackCmdFormat == null) || (unpackCmdFormat.length() == 0)
                || (unpackDir == null) || (packageName == null)) {
            String message = methodLogName + " - Some parameters are missing";
            logger.error(message);
            throw new IllegalStateException(message);
        }
        String command = unpackCmdFormat;
        command = command.replaceAll(PACKAGE_KEYWORD, packageName);
        command = command.replaceAll(DIR_KEYWORD, unpackDir);
        setCommandLine(command);
        setTimeout(timeout);
    }
}
