/**
 * @author N. Kornweibel
 * @version 1.0
 * A FileSelectionModel based on returning files for the set of specified
 * directories.
 */

package org.eso.util.filesystem;

import java.io.File;
import java.util.Vector;

import org.eso.util.filesystem.FileSelectionModel;

public class DirectoryFileSelectionModel extends FileSelectionModel {

    private String[] dirs = null;

    public DirectoryFileSelectionModel() {
        dirs = new String[0];
    }

    /**
     * @param dirlist The array of directory path string.
     */
    public DirectoryFileSelectionModel(String[] dirlist) {
        dirs = dirlist;
    }

    public DirectoryFileSelectionModel(String filePath) {
        dirs = new String[1];
        dirs[0] = filePath;
    }

    public String[] getDirs() {
        return dirs;
    }
 
    /**
     * @return The Vector of Strings for all valid directories in this model.
     */
    public Vector getFileList() {
        return getComponentFiles();
    }

    /**
     * @return The Vector of String objects for all valid files and directories
     * in this model.
     */
    private Vector getComponentFiles() {
        Vector rc = new Vector();
        if (dirs == null) {
            return rc;
        }
        for (int i = 0; i < dirs.length; i++) {
            String files[] = null;
            if (dirs[i] == null) {
                continue;
            }
            File dir = new File(dirs[i]);
            if (dir.isDirectory()) {
                try {
                    files = dir.list();
                } catch (SecurityException e) {
                    System.out.println(e.toString());
                }
                if (files == null) {
                    System.out.println("No files found under: "+dirs[i]);
                    continue;
                }
                for (int x = 0; x < files.length; x++) {
                    rc.addElement(dir.getPath()+File.separator+files[x]);
                }
            } else {
                    rc.addElement(dirs[i]);
            }
        }
        return rc;
    }

} // end of class DirectoryFileSelectionModel.

// end of file DirectoryFileSelectionModel.java
