package org.eso.util.requesthandler;

import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

/**
 * Implementation of a request to get data set. An instance of this class will
 * be instantiated each time a user makes a new request to get data set. A
 * request will be identified by a unique request number and will also contain
 * the meta data of the data set files required by the user.
 * 
 * @author cmoins
 * 
 */
public class DatasetRequest {

    // request identifier
    private String reqNum;
    // request user identifier
    private String userId;
    // archive where requested dataset are stored
    // RIL use an enum instead and define a hibernate UserType for mapping to /
    // from DB. Enums add an extra level of typing.
    // REPLY : use of an enum type implemented
    private ArchiveName archive;
    // submission date
    private Date date;
    // TODO : define enum type for status
    // REPLY : use of enum type implemented
    // state of the request
    private DatasetRequestStatus status;
    // medium the request content will be sent to
    // RIL use an enum instead and define a hibernate UserType for mapping to /
    // from DB. Enums add an extra level of typing.
    // Enum type implemented
    private DatasetRequestOptionName medium;
    // request priority
    // RIL use an enum instead and define a hibernate UserType for mapping to /
    // from DB. 
    // REPLY Enums add an extra level of typing.
    // RIL: add comment: who can set this priority, how, what does it control?
    // REPLY It will be set at the service layer according to the number of files and 
    // the selected medium. TODO
    private DatasetRequestPriority priority;
    // total size of the request (kByte)
    private Integer reqSize;
    // bit mask of the action associated to the request
    private byte[] action;
    // Set of data set file meta data included in the request
    private Set<DatasetFile> datasetFiles = new HashSet<DatasetFile>();

    public String getReqNum() {
        return reqNum;
    }

    public void setReqNum(String reqNum) {
        this.reqNum = reqNum;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public ArchiveName getArchive() {
        return archive;
    }

    public void setArchive(ArchiveName archive) {
        this.archive = archive;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public DatasetRequestStatus getStatus() {
        return status;
    }

    public void setStatus(DatasetRequestStatus status) {
        this.status = status;
    }

    public DatasetRequestOptionName getMedium() {
        return medium;
    }

    public void setMedium(DatasetRequestOptionName medium) {
        this.medium = medium;
    }

    public DatasetRequestPriority getPriority() {
        return priority;
    }

    public void setPriority(DatasetRequestPriority priority) {
        this.priority = priority;
    }

    public Integer getReqSize() {
        return reqSize;
    }

    public void setReqSize(Integer reqSize) {
        this.reqSize = reqSize;
    }

    public byte[] getAction() {
        return action;
    }

    public void setAction(byte[] bitmask) {
        action = bitmask;
    }

    /**
     * High level setter to set the actions associated to a request. A bit wise
     * logical OR is performed with the bit masks of all the provided options
     * to generate the request action mask.
     * 
     * @param optionList
     *            list of actions associated to the request
     */
    public void setAction(List<DatasetRequestOption> optionList) {
        Iterator<DatasetRequestOption> optionIt = optionList.iterator();
        action = new byte[DatasetRequestOptionId.OPTION_BIT_MASK_BYTES_NUMBER];
        // Logical OR of the option bit masks. The LSB has the highest index
        while (optionIt.hasNext()) {
            byte curMask[] = optionIt.next().getId().getReqOption();
            int minMaskLength =
                    (action.length < curMask.length
                            ? action.length
                            : curMask.length);
            for (int i = 0; i < minMaskLength; i++) {
                action[action.length - i - 1] |=
                        curMask[curMask.length - i - 1];
            }
        }
    }

    public Set<DatasetFile> getDatasetFiles() {
        return datasetFiles;
    }

    public void setDatasetFiles(Set<DatasetFile> datasetFiles) {
        this.datasetFiles = datasetFiles;
    }
}
