package org.eso.util.requesthandler;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateTemplate;

/** Implementation of a dataset request dao for sybase */
public class DatasetRequestDAOSyb implements DatasetRequestDAO {

    // initializes logger
    private final static Logger LOGGER =
            Logger.getLogger(DatasetRequestDAOSyb.class);

    // Hibernate helper
    private HibernateTemplate hibernateTemplate;

    public void save(DatasetRequest request) throws DAOException {
        if (request == null) {
            String message = " input parameters are missing";
            throw new IllegalArgumentException(message);
        }
        try {
            hibernateTemplate.save(request);
        } catch (DataAccessException e) {
            throw new DAOException(e.getMessage());
        }
    }

    public List<DatasetRequest> findByRequestNumber(String requestNumber)
            throws DAOException {
        String methodLogName = "findByRequestNumber()";
        if (requestNumber == null) {
            String message = "input parameters are missing";
            throw new IllegalArgumentException(message);
        }
        try {
            List<DatasetRequest> dsRequestList =
                    new ArrayList<DatasetRequest>();
            DatasetRequest dsRequest =
                    (DatasetRequest) hibernateTemplate.get(DatasetRequest.class,
                            requestNumber);
            if (dsRequest != null) {
                dsRequestList.add(dsRequest);
            } else {
                LOGGER.error(methodLogName
                        + "-No request found for request number "
                        + requestNumber);
            }
            return (dsRequestList);
        } catch (DataAccessException e) {
            throw new DAOException(e.getMessage());
        }
    }

    public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
        this.hibernateTemplate = hibernateTemplate;
    }
}
