package org.eso.util.requesthandler;

import java.util.HashMap;
import java.util.Map;

/**
 * Implementation of an enum type that specifies the different categories of
 * options that may be associated to a data set request : RETRIEV, MEDIA for
 * options specifying the medium to be used to store a request content, PROCESS for
 * options specifying processes to be applied on dataset requests, FORMATTING
 * for options specifying how to format data set files
 * 
 * @author cmoins
 * 
 */
public enum DatasetRequestOptionCategory {

    RETRIEV("RETRIEV"), MEDIA("MEDIA"), PROCESS("PROCESS"), FORMATTING(
            "FORMATTI");

    // string representation of an option category
    String optionCategory;

    private static final Map<String, DatasetRequestOptionCategory> INSTANCES =
            new HashMap<String, DatasetRequestOptionCategory>();

    static {
        DatasetRequestOptionCategory allCategory[] =
                DatasetRequestOptionCategory.values();
        for (int i = 0; i < allCategory.length; i++) {
            INSTANCES.put(allCategory[i].toString(), allCategory[i]);
        }
    }

    DatasetRequestOptionCategory(String optionCategory) {
        this.optionCategory = optionCategory;
    }

    /** returns the string representation of a medium */
    public String toString() {
        return optionCategory;
    }

    /** returns the option category enum type element associated to a string */
    public static DatasetRequestOptionCategory getInstance(String optionCategory) {
        return INSTANCES.get(optionCategory.trim());
    }
}
