package org.eso.util.requesthandler;

import java.util.List;

import org.apache.log4j.Logger;
import org.springframework.dao.DataAccessException;
import org.springframework.orm.hibernate3.HibernateTemplate;

/** Implementation of a dataset request option dao for sybase */
public class DatasetRequestOptionDAOSyb implements DatasetRequestOptionDAO {

    // initializes logger
    private final static Logger LOGGER =
            Logger.getLogger(DatasetRequestOptionDAOSyb.class);

    // hibernate helper
    private HibernateTemplate hibernateTemplate;

    @SuppressWarnings("unchecked")
    public DatasetRequestOption findByMediumAndArchive(
            DatasetRequestOptionName mediaName, ArchiveName archive)
            throws DAOException {
        
        if ((mediaName == null) || (archive == null)) {
            String message = " - Some input parameters are missing";
            throw new IllegalArgumentException(message);
        }
        // checks if the medium given as a parameter is valid
        if ((mediaName != DatasetRequestOptionName.CDR)
                && (mediaName != DatasetRequestOptionName.DVR)
                && (mediaName != DatasetRequestOptionName.MAG)
                && (mediaName != DatasetRequestOptionName.LAN)
                && (mediaName != DatasetRequestOptionName.FTP)
                && (mediaName != DatasetRequestOptionName.AUT)) {
            throw new DAOException(mediaName + " is not a media type");
        }
        return findByOptionNameAndArchive(mediaName, archive);
    }

    @SuppressWarnings("unchecked")
    public DatasetRequestOption findByOptionNameAndArchive(
            DatasetRequestOptionName optionName, ArchiveName archive)
            throws DAOException {

        final String QUERY_NAME = "findByOptionNameAndArchive";
        String methodLogName = "findByOptionNameAndArchive()";
        DatasetRequestOption dsRequestOption = null;

        if ((optionName == null) || (archive == null)) {
            String message = " - Some input parameters are missing";
            throw new IllegalArgumentException(message);
        }

        try {
            // execution of a query to retrieve the meta data of the option
            // associated to the archive and medium given as a parameter
            String paramName[] = { "archive", "optionname" };
            Object value[] = { archive, optionName };
            List<DatasetRequestOption> dsRequestOptionList =
                    (List<DatasetRequestOption>) hibernateTemplate.findByNamedQueryAndNamedParam(QUERY_NAME,
                            paramName,
                            value);
            if (dsRequestOptionList.size() == 1) {
                dsRequestOption = dsRequestOptionList.get(0);
            } else if (dsRequestOptionList.size() > 1) {
                // Only one option should be associated to an archive and an
                // option name
                throw new DAOException("One than more record found for "
                        + archive + " " + optionName);
            } else {
                LOGGER.error(methodLogName + " - No record found for "
                        + archive + " " + optionName);
            }
        } catch (DataAccessException e) {
            throw new DAOException(e.getMessage());
        }
        return (dsRequestOption);
    }

    public void setHibernateTemplate(HibernateTemplate hibernateTemplate) {
        this.hibernateTemplate = hibernateTemplate;
    }

}
