package org.eso.util.requesthandler;

import java.util.HashMap;
import java.util.Map;

/**
 * Possible states of a request to get data set : INITIAL when a request is
 * created, ERROR when a request has failed, LOCKED when a request is excluded
 * from automatic processing, FILE_PROCESSING_COMPLETE when all the files of a
 * request have been individually processed, READY_TO_CLOSE when request wide
 * actions are complete, FINISHED when a request is closed.
 * 
 * @author cmoins
 * 
 */
public enum DatasetRequestStatus {

    INITIAL("P"), ERROR("E"), LOCKED("Y"), FILE_PROCESSING_COMPLETE("R"), READY_TO_CLOSE(
            "T"), FINISHED("Z");

    // String representation of a request status
    private String status;
    private static final Map<String, DatasetRequestStatus> INSTANCES =
            new HashMap<String, DatasetRequestStatus>();

    static {
        DatasetRequestStatus[] statusArray = DatasetRequestStatus.values();
        for (int i = 0; i < statusArray.length; i++) {
            INSTANCES.put(statusArray[i].toString(), statusArray[i]);
        }
    }

    DatasetRequestStatus(String status) {
        this.status = status;
    }

    /** returns the string representation of a datafile status enum type element */
    public String toString() {
        return status;
    }

    /** returns the status enum type element associated to a string */
    public static DatasetRequestStatus getInstance(String status) {
        return INSTANCES.get(status.trim());
    }
}
