package org.eso.util.requesthandler;

import java.io.Serializable;

/**
 * Implementation of a ngas file composite key which is made of a disk id, a
 * file id and a file version
 * 
 * @author cmoins
 * 
 */

// RIL: Question: Why do we need to know low-level details such as disk?
// REPLY : the disk id is necessary if we want to identify a unique file record
// in the ngas file meta data repository. Furhtermore, the host id is needed for a 
// join query on ngas_files, ngas_hosts and ngas_disks tables to retrieve files stored
// on disks belonging to the headquarters domain.
public class NgasFileId implements Serializable {

    private static final long serialVersionUID = 1L;
    // Id of the disk where the file is stored
    private String diskId;
    // data product id of the file
    private String fileId;
    // version of the file
    private Integer fileVersion;

    public NgasFileId() {
    }

    public NgasFileId(String diskId, String fileName, Integer fileVersion) {
        this.diskId = diskId;
        this.fileId = fileName;
        this.fileVersion = fileVersion;
    }

    public boolean equals(Object o) {
        if (this == o)
            return true;
        if (o == null)
            return false;
        if (!(o instanceof NgasFileId))
            return false;
        final NgasFileId ngasFileId = (NgasFileId) o;
        if (!diskId.equals(ngasFileId.getDiskId()))
            return false;
        if (!fileId.equals(ngasFileId.getFileId()))
            return false;
        if (!fileVersion.equals(ngasFileId.getFileVersion()))
            return false;
        return true;
    }

    public int hashCode() {
        return (diskId.hashCode() + fileVersion.hashCode());
    }

    public String getDiskId() {
        return diskId;
    }

    public void setDiskId(String diskId) {
        this.diskId = diskId;
    }

    public String getFileId() {
        return fileId;
    }

    public void setFileId(String fileId) {
        this.fileId = fileId;
    }

    public Integer getFileVersion() {
        return fileVersion;
    }

    public void setFileVersion(Integer fileVersion) {
        this.fileVersion = fileVersion;
    }
}
