package org.eso.util.requesthandler;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Date;
import java.util.List;

import javax.sql.DataSource;

import org.eso.util.requesthandler.DAOException;
import org.eso.util.requesthandler.DatasetFile;
import org.eso.util.requesthandler.DatasetRequest;
import org.eso.util.requesthandler.DatasetRequestDAO;
import org.springframework.test.AbstractDependencyInjectionSpringContextTests;

public class DatasetRequestDAOTest extends
        AbstractDependencyInjectionSpringContextTests {

    final static String USER_ID = "test01";
    private DatasetRequestDAO requestDAO;
    private DataSource dataSource;

    public void setDataSource(DataSource dataSource) {
        this.dataSource = dataSource;
    }

    public DatasetRequestDAOTest(String name) {
        super(name);
    }

    @Override
    protected String[] getConfigLocations() {
        // TODO Auto-generated method stub
        return new String[] { "config/spring/db.xml",
                "config/spring/hibernate.xml", "config/spring/dao.xml",
                "config/spring/transaction.xml" };
    }

    /** Nominal case */
    public void testSave_01() throws DAOException, SQLException {

        DatasetRequest testRequest = new DatasetRequest();

        testRequest.setUserId(USER_ID);
        testRequest.setArchive(ArchiveName.SAF);
        testRequest.setDate(new Date());
        testRequest.setStatus(DatasetRequestStatus.INITIAL);
        testRequest.setMedium(DatasetRequestOptionName.DVR);
        testRequest.setAction(new byte[] { 0x17, 0x0, 0, 0x17, 0x0, 0 });
        testRequest.setPriority(DatasetRequestPriority.HIGH);
        DatasetFile dsFile1 = new DatasetFile();
        dsFile1.setDatasetName("DSNAMETEST1");
        dsFile1.setName("FNAMETEST1");
        dsFile1.setStatus(DatasetFileStatus.MISSING);
        testRequest.getDatasetFiles().add(dsFile1);
        DatasetFile dsFile2 = new DatasetFile();
        dsFile2.setDatasetName("DSNAMETEST2");
        dsFile2.setName("FNAMETEST2");
        dsFile2.setStatus(DatasetFileStatus.PROPRIETARY);
        testRequest.getDatasetFiles().add(dsFile2);

        requestDAO.save(testRequest);

        // reqnum is computed by the dao class
        String sql =
                "select * from rt_requests where reqnum='"
                        + testRequest.getReqNum() + "'";
        ResultSet selectedRows =
                dataSource.getConnection().createStatement().executeQuery(sql);
        int count = 0;
        while (selectedRows.next()) {
            count++;
        }
        assertEquals(1, count);

        sql =
                "select * from rt_reqfiles where reqnum='"
                        + testRequest.getReqNum() + "'";
        selectedRows =
                dataSource.getConnection().createStatement().executeQuery(sql);
        count = 0;
        while (selectedRows.next()) {
            count++;
        }
        assertEquals(2, count);
    }

    /**
     * transaction test
     * 
     * @throws DAOException
     * @throws SQLException
     */
    public void testSave_02() throws DAOException, SQLException {

        DatasetRequest testRequest = new DatasetRequest();

        testRequest.setUserId(USER_ID);
        testRequest.setArchive(ArchiveName.SAF);
        testRequest.setDate(new Date());
        testRequest.setStatus(DatasetRequestStatus.INITIAL);
        testRequest.setMedium(DatasetRequestOptionName.DVR);
        testRequest.setAction(new byte[] { 0x0, 0x0, 0x0, 0x0, 0x1, (byte) 0xff });
        DatasetFile dsFile1 = new DatasetFile();

        dsFile1.setDatasetName(null);
        dsFile1.setName("FNAMETEST1");
        dsFile1.setStatus(DatasetFileStatus.MISSING);
        testRequest.getDatasetFiles().add(dsFile1);

        try {
            requestDAO.save(testRequest);
            fail("No exception was thrown");
        } catch (DAOException e) {
            // expected behaviour
        }

        // Checks that no request was inserted
        String sql =
                "select * from rt_requests where user_id='" + USER_ID + "'";
        ResultSet selectedRows =
                dataSource.getConnection().createStatement().executeQuery(sql);
        int count = 0;
        while (selectedRows.next()) {
            count++;
        }
        assertEquals(0, count);

    }

   
    /**
     * Nominal case for the find dataset request method
     * 
     * @throws DAOException
     * @throws SQLException
     */
    public void testFind_01() throws DAOException, SQLException {

        final String REQNUM = USER_ID + "01";
        Connection connection = dataSource.getConnection();
        // insertion of one requests and two files
        String sql =
                " insert into archive..rt_requests   "
                        + "(user_id, archive, date, status, action, medium, priority, "
                        + "destination, destuname, destpswd, completion, reqsize, reqnum) "
                        + "values('" + USER_ID
                        + "', 'SAF',getdate(),'T', 0x110000000000,'CDR', 'I', "
                        + "'DEST', 'UNAME','PASSWd', getdate(), 1000, '"
                        + REQNUM + "')";
        connection.createStatement().executeUpdate(sql);

        sql =
                " insert into archive..rt_reqfiles "
                        + "(reqnum,dataset_name,fname,status,fsize,diskname,"
                        + " dirsectoff,tapenum,output_fname) "
                        + " values('"
                        + REQNUM
                        + "','DSNAMETEST1','FNAMETEST1','P',1000,'DISKNAME1',1,1,'OUTFNAMETEST1')";
        connection.createStatement().executeUpdate(sql);

        sql =
                " insert into archive..rt_reqfiles "
                        + "(reqnum,dataset_name,fname,status,fsize,diskname,"
                        + " dirsectoff,tapenum,output_fname) "
                        + " values('"
                        + REQNUM
                        + "','DSNAMETEST2','FNAMETEST2','F',1000,'DISKNAME2',1,1,'OUTFNAMETEST2')";
        connection.createStatement().executeUpdate(sql);

        try {
            List<DatasetRequest> dsReqList =
                    requestDAO.findByRequestNumber(REQNUM);
            assertEquals(1, dsReqList.size());
            DatasetRequest resultDR = dsReqList.iterator().next();
            assertEquals(2, resultDR.getDatasetFiles().size());

        } catch (DAOException e) {
            fail("An exception was thrown" + e.getMessage());
        }

    }

    @Override
    protected void onSetUp() throws Exception {
        Connection connection = dataSource.getConnection();
        String sql = "delete from rt_requests where user_id='" + USER_ID + "'";
        connection.createStatement().executeUpdate(sql);

        sql = "delete from rt_reqfiles where reqnum like '" + USER_ID + "%'";
        connection.createStatement().executeUpdate(sql);
    }

    public void setRequestDAO(DatasetRequestDAO requestDAO) {
        this.requestDAO = requestDAO;
    }
}
