package org.eso.util.dal.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.TimeZone;

import org.eso.util.dal.PackageFile;
import org.eso.util.dal.PackageFilesDAOSyb;
import org.eso.util.dal.PackageInfoDAOSyb;
import org.eso.util.dal.SybaseConnectionService;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class PackageFilesDAOSybTest {
	
	private class PackageFileComparator implements Comparator {
		public int compare(Object o1, Object o2) {
			PackageFile p1 = (PackageFile) o1;
			PackageFile p2 = (PackageFile) o2;
			return p1.getFileName().compareTo(p2.getFileName());
		}
	}
	
	private static PackageFilesDAOSyb dao = null;
	private static Statement stmt = null;
	
	private static String DBTABLE = "rhandler..pi_pkg_files";
	
	private static final Integer pkgId = 22222222;
	
	private static final Integer nrFiles = 1000; 
	
	private static List<PackageFile> testFiles = new ArrayList<PackageFile>();
	
	private static Connection conn1;
	private static Connection conn2;
		
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
		conn1 = SybaseConnectionService.getConnection("acdbdev:6789", "pipack", "pipack_ps");
		dao = new PackageFilesDAOSyb(conn1, DBTABLE);
		conn2 = SybaseConnectionService.getConnection("acdbdev:6789", "pipack", "pipack_ps");
		stmt = conn2.createStatement();
		
		for (long i = 0; i < nrFiles; i++) {
			String formattedNumber = new DecimalFormat("0000").format(i);
			PackageFile file = new PackageFile();
			file.setPackageId(pkgId);
			file.setFileName("FILENAME" + formattedNumber);
			file.setFileId("FILEID" + formattedNumber);
			file.setFilePath("PATH" + formattedNumber);
			file.setFileType(PackageFile.Type.SCIENCEPROD);
			file.setIngestionDate(new java.sql.Timestamp(i*100000));
			file.setAcquisitionDate(new java.sql.Timestamp(i*1000000));
			testFiles.add(file);
		}
		
		truncateTable();
		String query = "delete from rhandler..pi_pkgs where pkg_id=" + pkgId;
		stmt.execute(query);
		query = "insert into rhandler..pi_pkgs values (" + pkgId + ", 'RUN182712613', 'INSTRU75764563', 'OPEN', '')";
		stmt.execute(query);
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		truncateTable();
		
		final String query = "delete from rhandler..pi_pkgs where pkg_id=" + pkgId;
		stmt.execute(query);
		
		conn1.close();
		conn2.close();
	}

	@Before
	public void setUp() throws Exception {		
	}

	@Test
	public void testInsert() {
		try {
			truncateTable();
						
			/***************************************************************/
			dao.insert(testFiles);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageFilesDAOSyb.PACKAGEID_KW + "=" + pkgId + "ORDER BY " + PackageFilesDAOSyb.FILENAME_KW));
			
			compare(stmt.getResultSet(), testFiles);
							
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	@Test
	public void testDelete() {
		try {
			truncateTable();
			insertTestEntry(nrFiles);
			/********************************/
			dao.delete(testFiles);
			/********************************/
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageInfoDAOSyb.PACKAGEID_KW + "=" + pkgId));
			ResultSet rs = stmt.getResultSet();
			assertFalse(rs.next());
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testFind1() {
		try {
			truncateTable();
			insertTestEntry(nrFiles);
			/********************************/
			/* TODO check dates */
			List<PackageFile> files = dao.find(pkgId, new Timestamp(0), new Timestamp(System.currentTimeMillis()));
			Collections.sort(files, new PackageFileComparator());
			
			/********************************/
			assertEquals((Integer) nrFiles, (Integer) files.size());
			for (int i = 0; i < nrFiles; i++) {
				assertEquals(testFiles.get(i).getPackageId(), files.get(i).getPackageId());
				assertEquals(testFiles.get(i).getFileName(), files.get(i).getFileName());
				assertEquals(testFiles.get(i).getFileId(), files.get(i).getFileId());
				assertEquals(testFiles.get(i).getFilePath(), files.get(i).getFilePath());
				assertEquals(testFiles.get(i).getFileType(), files.get(i).getFileType());
				assertEquals(testFiles.get(i).getIngestionDate(), files.get(i).getIngestionDate());
				assertEquals(testFiles.get(i).getAcquisitionDate(), files.get(i).getAcquisitionDate());
			}
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	@Test
	public void testFind2() {
		try {
			truncateTable();
			insertTestEntry(nrFiles);
			/********************************/
			List<PackageFile> files = dao.find(testFiles);
			Collections.sort(files, new PackageFileComparator());
			/********************************/
			assertEquals((Integer) nrFiles, (Integer) files.size());
			for (int i = 0; i < nrFiles; i++) {
				assertEquals(testFiles.get(i).getPackageId(), files.get(i).getPackageId());
				assertEquals(testFiles.get(i).getFileName(), files.get(i).getFileName());
				assertEquals(testFiles.get(i).getFileId(), files.get(i).getFileId());
				assertEquals(testFiles.get(i).getFilePath(), files.get(i).getFilePath());
				assertEquals(testFiles.get(i).getFileType(), files.get(i).getFileType());
				assertEquals(testFiles.get(i).getIngestionDate(), files.get(i).getIngestionDate());
				assertEquals(testFiles.get(i).getAcquisitionDate(), files.get(i).getAcquisitionDate());
			}
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	@Test
	public void testInsertIfNotFound1() {
		try {
			truncateTable();
						
			/***************************************************************/
			dao.insertIfNotFound(testFiles);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageFilesDAOSyb.PACKAGEID_KW + "=" + pkgId + "ORDER BY " + PackageFilesDAOSyb.FILENAME_KW));
			compare(stmt.getResultSet(), testFiles);
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testInsertIfNotFound2() {
		try {
			truncateTable();
			insertTestEntry(nrFiles/2);
			
			/***************************************************************/
			dao.insertIfNotFound(testFiles);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageFilesDAOSyb.PACKAGEID_KW + "=" + pkgId + " ORDER BY " + PackageFilesDAOSyb.FILENAME_KW));
			compare(stmt.getResultSet(), testFiles);
				
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testfindByQCFilename() {
		try {
			truncateTable();
			insertTestEntry(1);
			
			/***************************************************************/
			assertTrue(dao.findByQCFilename(pkgId, testFiles.get(0).getFilePath(), testFiles.get(0).getFileName()));
			/****************************************************************/
			
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	public static void truncateTable() throws Exception {
		stmt.execute("DELETE FROM " + DBTABLE + " WHERE " + PackageFilesDAOSyb.PACKAGEID_KW + "=" + pkgId);
	}
	
	public void insertTestEntry(int n) throws Exception {
		for (int i = 0; i < n; i++) {
			String query = "";
			query += "INSERT INTO " + DBTABLE + " (" + PackageFilesDAOSyb.PACKAGEID_KW + "," + PackageFilesDAOSyb.FILENAME_KW + ",";
			query += PackageFilesDAOSyb.FILEID_KW + "," + PackageFilesDAOSyb.INGESTIONDATE_KW + "," + PackageFilesDAOSyb.FILEPATH_KW + ",";
			query += PackageFilesDAOSyb.ACQUISITIONDATE_KW + "," + PackageFilesDAOSyb.FILETYPE_KW + ") ";
			String formattedNumber = new DecimalFormat("0000").format(i);
			query += "VALUES (" + pkgId + ", 'FILENAME" + formattedNumber + "', 'FILEID" + formattedNumber + "', '" + new java.sql.Timestamp(i*100000) + "', 'PATH" + formattedNumber + "' , '" +  new java.sql.Timestamp(i*1000000) + "', 'S')";
			stmt.execute(query);
		}
	}
	
	public static void compare(ResultSet rs, List<PackageFile> files) throws SQLException {
		for (int i = 0; i < nrFiles; i++) {
			assertTrue(rs.next());
			assertEquals((Integer) files.get(i).getPackageId(), (Integer) rs.getInt(PackageFilesDAOSyb.PACKAGEID_KW));
			assertEquals(files.get(i).getFileName(), rs.getString(PackageFilesDAOSyb.FILENAME_KW));
			assertEquals(files.get(i).getFileId(), rs.getString(PackageFilesDAOSyb.FILEID_KW));
			assertEquals(files.get(i).getFilePath(), rs.getString(PackageFilesDAOSyb.FILEPATH_KW));
			assertEquals(files.get(i).getFileType().toString(), rs.getString(PackageFilesDAOSyb.FILETYPE_KW));
			assertEquals(files.get(i).getIngestionDate(), rs.getTimestamp(PackageFilesDAOSyb.INGESTIONDATE_KW));
			assertEquals(files.get(i).getAcquisitionDate(), rs.getTimestamp(PackageFilesDAOSyb.ACQUISITIONDATE_KW));
		}
		assertFalse(rs.next());
	}
}
