package org.eso.util.dal.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.TimeZone;

import org.eso.util.dal.PackageInfo;
import org.eso.util.dal.PackageInfoDAOSyb;
import org.eso.util.dal.SybaseConnectionService;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class PackageInfoDAOSybTest {
	
	private static PackageInfoDAOSyb dao = null;
	private static Statement stmt = null;
	
	private static String DBTABLE = "rhandler..pi_pkgs";
	
	private static PackageInfo testPkg = null;
	
	private static Connection conn1;
	private static Connection conn2;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
		TimeZone.setDefault(TimeZone.getTimeZone("UTC"));
		conn1 = SybaseConnectionService.getConnection("acdbdev:6789", "pipack", "pipack_ps");
		dao = new PackageInfoDAOSyb(conn1, DBTABLE, "");
		conn2 = SybaseConnectionService.getConnection("acdbdev:6789", "pipack", "pipack_ps");
		stmt = conn2.createStatement();
		
		testPkg = new PackageInfo();
		testPkg.setPackageId(11111111);
		testPkg.setRunId("TESTRUN");
		testPkg.setInstrument("TESTINSTRU");
		testPkg.setAssocData("TESTDATA");
		testPkg.setStatus(PackageInfo.Status.OPEN);
	}

	@AfterClass
	public static void tearDownAfterClass() throws Exception {
		truncateTable();
		conn1.close();
		conn2.close();
	}

	@Before
	public void setUp() throws Exception {		
	}

	@Test
	public void testInsert() {
		try {
			truncateTable();
						
			/***************************************************************/
			dao.insert(testPkg);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageInfoDAOSyb.PACKAGEID_KW + "=" + testPkg.getPackageId()));
			ResultSet rs = stmt.getResultSet();
			compare(rs, testPkg);
	
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	@Test
	public void testDelete() {
		try {
			truncateTable();
			insertTestEntry();
			/********************************/
			PackageInfo pkg = new PackageInfo();
			pkg.setPackageId(12345678);
			pkg.setRunId("RUN");
			pkg.setInstrument("INSTRU");
			pkg.setAssocData("DATA");
			pkg.setStatus(PackageInfo.Status.OPEN);
			dao.delete(pkg);
			/********************************/
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageInfoDAOSyb.PACKAGEID_KW + "=" + testPkg.getPackageId()));
			compare(stmt.getResultSet(), testPkg);
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testFind1() {
		try {
			truncateTable();
			insertTestEntry();
			/********************************/
			PackageInfo pkg = dao.find(testPkg.getRunId(), testPkg.getInstrument());
			/********************************/
			compare(testPkg, pkg);

		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	@Test
	public void testInsertIfNotFound1() {
		try {
			truncateTable();
			
			/***************************************************************/
			dao.insertIfNotFound(testPkg);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageInfoDAOSyb.PACKAGEID_KW + "=" + testPkg.getPackageId()));
			compare(stmt.getResultSet(), testPkg);
			
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}

	@Test
	public void testInsertIfNotFound2() {
		try {
			truncateTable();
			insertTestEntry();
			
			/***************************************************************/
			dao.insertIfNotFound(testPkg);
			/****************************************************************/
			
			assertTrue(stmt.execute("SELECT * FROM " + DBTABLE + " WHERE " + PackageInfoDAOSyb.PACKAGEID_KW + "=" + testPkg.getPackageId()));
			compare(stmt.getResultSet(), testPkg);
			
		} catch (Exception e) {
			fail(e.getMessage());
		}
	}
	
	public static void truncateTable() throws Exception {
		stmt.execute("DELETE FROM " + DBTABLE + " WHERE " + PackageInfoDAOSyb.PACKAGEID_KW + "=" + testPkg.getPackageId());
	}
	
	public void insertTestEntry() throws Exception {
		String query = "";
		query += "INSERT INTO " + DBTABLE + " (" + PackageInfoDAOSyb.PACKAGEID_KW + "," + PackageInfoDAOSyb.RUNID_KW + ",";
		query += PackageInfoDAOSyb.INSTRUMENT_KW + "," + PackageInfoDAOSyb.ASSOCIATIONDATA_KW + "," + PackageInfoDAOSyb.STATUS_KW + ") ";
		query += "VALUES (" + testPkg.getPackageId() + ", '" + testPkg.getRunId() + "', '" + testPkg.getInstrument() + "', '" + testPkg.getAssocData() + "', '" + testPkg.getStatus() + "')";
		stmt.execute(query);
	}
	
	public void compare(ResultSet rs, PackageInfo pkg) throws Exception {
		assertTrue(rs.next());
		assertEquals((Integer) testPkg.getPackageId(), (Integer) rs.getInt(PackageInfoDAOSyb.PACKAGEID_KW));
		assertEquals(testPkg.getRunId(), rs.getString(PackageInfoDAOSyb.RUNID_KW));
		assertEquals(testPkg.getInstrument(), rs.getString(PackageInfoDAOSyb.INSTRUMENT_KW));
		assertEquals(testPkg.getAssocData(), rs.getString(PackageInfoDAOSyb.ASSOCIATIONDATA_KW));
		assertEquals(testPkg.getStatus().toString(), rs.getString(PackageInfoDAOSyb.STATUS_KW));
		assertFalse(rs.next());	
	}
	
	public void compare(PackageInfo pkg1, PackageInfo pkg2) {
		assertEquals(pkg1.getPackageId(),pkg2.getPackageId());
		assertEquals(pkg1.getRunId(),pkg2.getRunId());
		assertEquals(pkg1.getInstrument(),pkg2.getInstrument());
		assertEquals(pkg1.getAssocData(),pkg2.getAssocData());
		assertEquals(pkg1.getStatus().toString(), pkg2.getStatus().toString());
	}
}
