package org.eso.dfs.gui;

import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JList;
import javax.swing.ListModel;
import javax.swing.TransferHandler;

/**
 * @author Nick Kornweibel
 * A JList with (in my opinion) smoother, more intuitive Drag/Drop.
 * See: http://forum.java.sun.com/thread.jsp?forum=57&thread=470025
 * for details.
 */

public class DnDJList extends JList {
    
    private boolean canSelect = true;
    private boolean isDragging;
    private MouseEvent pressEvent = null;
    
    /**
     * Constructor for DnDJList.
     * @param dataModel
     */
    public DnDJList(ListModel dataModel) {
        super(dataModel);
    }

    /**
     * Constructor for DnDJList.
     * @param listData
     */
    public DnDJList(Object[] listData) {
        super(listData);
    }

    /**
     * Constructor for DnDJList.
     * @param listData
     */
    public DnDJList(Vector listData) {
        super(listData);
    }

    /**
     * Constructor for DnDJList.
     */
    public DnDJList() {
        super();
    }

    /**
     * @see java.awt.Component#processMouseMotionEvent(java.awt.event.MouseEvent)
     */
    protected void processMouseMotionEvent(MouseEvent e) {
        handleMouseEvent(e);
        super.processMouseMotionEvent(e);
    }

    /**
     * Only permit row selection if we are not dragging.
     * @see javax.swing.JList#setSelectionInterval(int, int)
     */
    public void setSelectionInterval(int anchor, int lead) {
        if (canSelect) {
            super.setSelectionInterval(anchor, lead);
        } 
    }
    
    /**
     * @see java.awt.Component#processMouseEvent(java.awt.event.MouseEvent)
     */
    protected void processMouseEvent(MouseEvent e) {
        handleMouseEvent(e);
        super.processMouseEvent(e);
    }
    
    /**
     * Performs two tasks:
     * 1. Start Drag/Drop.
     * Commence drag/drop operation if we are not already dragging.
     * Commencing a drag/drop requires the mouse event of the MOUSE_PRESSED
     * kind - which has hopefully been recorded before the MOUSE_DRAGGED mouse event
     * is received.
     * 
     * 2. Disable row selection while dragging.
     * 
     * @param e the mouse event
     */
    protected void handleMouseEvent(MouseEvent e) {
        canSelect = (e.getID() != MouseEvent.MOUSE_DRAGGED);
        if (e.getID() == MouseEvent.MOUSE_PRESSED) {
            isDragging = false;
            pressEvent = e;
        } else if (e.getID() == MouseEvent.MOUSE_DRAGGED) {
            if (!isDragging && pressEvent != null) {
                TransferHandler handler = getTransferHandler();
                handler.exportAsDrag(this, pressEvent, TransferHandler.COPY);
            }
            pressEvent = null;
            isDragging = true;
        } else if (e.getID() == MouseEvent.MOUSE_RELEASED) {
            isDragging = false;
            pressEvent = null;
        }       
    }
}