package org.eso.dfs.gui;

import java.awt.event.ActionEvent;
import java.util.Iterator;

import javax.swing.JComboBox;

import org.eso.dfs.util.SharedList;
import org.eso.dfs.util.SharedListListener;

/**
 * @author Nick Kornweibel
 * 
 * A RecallingComboBox that uses a shared list of its contents.
 * The listSort member is false. If elements in the list are to 
 * be maintained in sorted order, the SharedList should have
 * setSortList() set to true.
 */
public class SharedListComboBoxModel extends RecallingComboBoxModel implements SharedListListener {

    private SharedList globalList;

    /**
     * Constructor for NComboBoxModel.
     */
    public SharedListComboBoxModel(SharedList list) {
        super();
        globalList = list;
        globalList.addListListener(this);
    }
    
    public void detach() {
        // remove this from the globalList listeners
        globalList.removeListListener(this);
    }

    /**
     * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
     */
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() instanceof JComboBox) {
            JComboBox cb = (JComboBox) e.getSource();
            // add the combo box item to the list, if not already present
            Object o = cb.getSelectedItem();
            boolean add = (o != null && o.toString().length() > 0);
            for (int i = 0; add && i < getSize(); i++) {
                if (o.equals(getElementAt(i))) {
                    add = false;
                }
            }
            if (add) {
                // add it to the globalList, then we will reload it into our list
                // through the call back (listChanged())
                globalList.add(o);
            }
        }
    }

    /**
     * @see SharedListListener#listChanged(SharedList)
     */
    public void listChanged(SharedList l) {
        // reload from the globalList as its contents have changed.
        // preserve the current selected object
        Object currentSel = getSelectedItem();
        boolean found = false;
        super.removeAllElements();
        Iterator i = globalList.iterator();
        while (i.hasNext()) {
            Object o = i.next();
            if (o == currentSel) {
                currentSel = o;
                found = true;
            }
            addElement(o);
        }
        if (found) {
            setSelectedItem(currentSel);
        } else {
            setSelectedItem(null);
        }
    }
}
