package org.eso.dfs.services.data;

import java.io.File;

/**
 * @author Nick Kornweibel
 *
 * Basic class for submitting a request to the AssociationService and
 * receiving a response. 
 * 
 * The class represents an implementation of the AssociationNode interface
 * for a frame on the filesystem.
 * 
 * 
 */
public class FileAssociationNode implements AssociationNode {

    /**
     * the path of file. 
     */
    private File address;
    
    /**
     * The classification(s) of the frame.
     */
    private String[] classification;
    
    /**
     * The group(s) of the frame.
     */
    private String[] grouping;
    
    /**
     * The category of the frame.
     */
    private String category;
    
    /**
     * The instrument associated with the frame.
     */
    private String instrumentName;

    /**
     * Create an association node for the specified file.
     * Association data is not set until the object is passed
     * through the AssociationService. 
     */
    public FileAssociationNode(File f) {
        address = f;
    }
    
    /**
     * Returns the address.
     * @return File
     */
    public File getAddress() {
        return address;
    }

    /**
     * Returns the classification.
     * @see org.eso.gasgano.services.data.AssociationNode#getClassification()
     * @return String[]
     */
    public String[] getClassification() {
        return classification;
    }

    /**
     * Returns the grouping.
     * @see org.eso.gasgano.services.data.AssociationNode#getGrouping()
     * @return String[]
     */   
    public String[] getGrouping() {
        return grouping;
    }

    /**
     * Returns the instrumentName.
     * @see org.eso.gasgano.services.data.AssociationNode#getInstrumentName()
     * @return String
     */
    public String getInstrumentName() {
        return instrumentName;
    }

    /**
     * Returns the category.
     * @see org.eso.gasgano.services.data.AssociationNode#getCategory()
     * @return String
     */
    public String getCategory() {
        return category;
    }

    /**
     * Sets the address.
     * @see org.eso.gasgano.services.data.AssociationNode#setAddress(java.io.File)
     * @param address The address to set
     */
    public void setAddress(File address) {
        this.address = address;
    }

    /**
     * Sets the classification.
     * @see org.eso.gasgano.services.data.AssociationNode#setClassification(java.lang.String)
     * @param classification The classification to set
     */
    public void setClassification(String[] classification) {
        this.classification = classification;
    }

    /**
     * Sets the grouping.
     * @see org.eso.gasgano.services.data.AssociationNode#setGrouping(java.lang.String)
     * @param grouping The grouping to set
     */
    
    public void setGrouping(String[] grouping) {
        this.grouping = grouping;
    }

    /**
     * Sets the instrumentName.
     * @see org.eso.gasgano.services.data.AssociationNode#setInstrumentName(java.lang.String)
     * @param instrumentName The instrumentName to set
     */

    public void setInstrumentName(String instrumentName) {
        this.instrumentName = instrumentName;
    }

    /**
     * Sets the category.
     * @param type The category to set
     *
     * @see org.eso.gasgano.services.data.AssociationNode#setCategory(String)
     */
    public void setCategory(String category) {
        this.category = category;
    }
}
