/**
 * @author N. Kornweibel
 * @version 1.0
 * OBComponent representing a file which is not located within the model.
 */

package org.eso.gasgano.datamodel;

import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.Box;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import org.eso.gasgano.datamodel.filesystem.DFSFile;
import org.eso.gasgano.datamodel.gui.OBComponent;
import org.eso.gasgano.gui.GUIUtils;

public class NotFoundOBComponent extends OBComponent {

    private String fileName = null;
    private static final String fileMissingMessage = 
        "This file has not been included in the "
        +"set of files currently in the application.";

    public NotFoundOBComponent(String name) {
        fileName = name;
    }
    /**
     * @return The status of this component.
     * @see org.eso.gasgano.DFSFile
     */
    public int getStatus() {
        return DFSFile.FILE_NOT_FOUND;
    }

    public DFSFile getSourceFile() {
        return null;
    }

    /**
     * @return The current status after executing the update.
     * Check if the file requires updating, update the data
     * and return the new DFSFile status.
     * @see org.eso.gasgano.DFSFile
     */
    public int update() {
        return getStatus();
    }

    /**
     * @return The JComponent representing the component data.
     */
    public JComponent getDisplayComponent() {
        JPanel panel = new JPanel(new FlowLayout());
        JLabel label = new JLabel(toString(), SwingConstants.CENTER);
        label.setForeground(Color.black);
        Box center = Box.createVerticalBox();
        center.add(label);
        label = new JLabel(fileMissingMessage, SwingConstants.CENTER);
        label.setForeground(Color.black);
        center.add(label);
        label.setForeground(Color.black);
        panel.add(center);
        return panel;
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = new JLabel(toString(), new ImageIcon(GUIUtils.dataGif),
            (int)JLabel.LEFT_ALIGNMENT);
        cell.setFont(getLabelFont());
        return cell;
    }

    /**
     * @return The unique id for this component 
     * - typically the file name, without the path.
     */
    public String getId() {
        return fileName;
    }

} // end of class NotFoundOBComponent.java

// end of file NotFoundOBComponent.java
