/**
 * @author N. Kornweibel
 * @version 1.0
 * A simple interface to encalsulate the generation of observation descriptions
 * in the DataModel.
 */
package org.eso.gasgano.datamodel;

import java.util.Vector;

import org.eso.gasgano.datamodel.filesystem.DFSDataModel;
import org.eso.gasgano.datamodel.gui.FitsFileOBComponent;
import org.eso.gasgano.datamodel.gui.OBComponent;
import org.eso.gasgano.keyword.Keyword;
import org.eso.gasgano.properties.PropertyDB;
import org.eso.gasgano.tools.PasswordSource;

public class ObsStatusDescription implements ObservationDescription {

  private String currentProgram = null;
  private String password = null;
  private OBStatusDictionary obStatusDic = null;
  private PasswordSource passwordSource = null;
  private static final String NO_STATUS_INFO = "no status available";
  private final String dbTable = "obrep2..obs_blocks";
  private final String dbStatusField = "status";
  private final String dbPidField = "run_id";
  private final String dbObIdField = "ob_id";

    public ObsStatusDescription() {
        Keyword k[] = new Keyword[1];
        k[0] = new Keyword(Keyword.OBS_NAME, null);
        FitsFileOBComponent.addToDefaultKeywordQuery(k);
        obStatusDic = new OBStatusDictionary();
    }
    
    public void setPasswordSource(PasswordSource ps) {
		passwordSource = ps;
	}
	
	private String getPassword() {
		String rc = null;
		if (passwordSource == null) {
			rc = new String("");
		} else {
			String userName = PropertyDB.getInstance().getProperty("DB_USER_NAME");
			rc = passwordSource.getPassword(userName, "Enter OB Database password", "OB_STATUS_FROM_DB_REQUIRED");
		}
		return rc;
	}
 
    /**
     * @param o The Observation to have its description set.
     * @param obc The OBComponent used to config Observation o.
     */
    public void setDescription(Observation o,
            OBComponent obc) {
        if (obc instanceof FitsFileOBComponent) {
            FitsFileOBComponent ffob = (FitsFileOBComponent)obc;
            Keyword[] k = FitsFileOBComponent.getDefaultKeywordQuery();
            String obsId = o.getId();
            String obsName = null;
            String programId = null;
            String status = null;
            for (int i = 0; i < k.length; i++) {
                if (k[i].getName().equals(Keyword.OBS_NAME)) {
                    if (k[i].getValue() != null) {
                        obsName = k[i].getValue();
                    }
                } else if (k[i].getName().equals(Keyword.PROG_ID)) {
                    if (k[i].getValue() != null) {
                        programId = k[i].getValue();
                    }
                }
                if (programId != null && obsName != null) {
                    break;
                }
            }
            if (obsName == null) {
                obsName = new String("");
            }
            status = getStatus(programId, obsId);
            // format our description.
            o.setDescription(new String(obsId+"    "+obsName+" "+status));
        }
    }

    private String getStatus(String programId, String obsId) {
        String rc = null;
        if (programId == null 
                || programId.equals(DFSDataModel.UNKNOWN_PROGRAM_ID)) {
            rc = new String("");
        } else {
            String statusDesc = null;
            if (currentProgram == null || !currentProgram.equals(programId)) {
                Vector progVec = new Vector(1);
                DFSDataModel dm = DFSDataModel.getDataModel();
                progVec.addElement(programId);
                obStatusDic.clear();
                currentProgram = programId;
                String userName = PropertyDB.getInstance().getProperty("DB_USER_NAME");
                String serverUrl = PropertyDB.getInstance().getProperty("DB_SERVER_URL");
                if (!obStatusDic.fillFromDatabase
						(serverUrl, userName, getPassword(),
                        dbTable, dbStatusField, dbPidField,
                        dbObIdField, progVec)) {
                    System.out.println("Database error: "
                        +obStatusDic.getErrorMessage());
                }
            }
            rc = (String)obStatusDic.get(obsId);
            if (rc == null) {
                rc = new String("("+NO_STATUS_INFO+")");
            } else {
                statusDesc = obStatusDic.getStatusDescription(rc);
                if (statusDesc == null) {
                    rc = new String("("+rc+")");
                } else {
                    rc = new String("("+statusDesc+")");
                }
            }
        }
        return rc;
    }
}
// end of file ObsStatusDescription.java.
