/**
 * @author N. Kornweibel
 * @version 1.0
 * A FileSelectionModel based on returning files for the set of specified
 * directories.
 */

package org.eso.gasgano.datamodel.filesystem;

import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.Set;
import java.util.LinkedHashSet;

public class DirectoryFileSelectionModel extends FileSelectionModel {

    private final String[] dirs;

    // TODO: this constructor is probably useless
    public DirectoryFileSelectionModel() throws IOException {
        this( new String[] {} );
    }

    /**
     * @param dirsAndFiles The array of directory path strings and files.
     */
    public DirectoryFileSelectionModel(final String[] dirsAndFiles) throws IOException {

    	final StringBuilder errorMessage = new StringBuilder("");
    	
    	// use the LinkedHashSet to get rid of any duplicate from the list of
    	// files and directories while keeping the elements original order

    	Set<String> dirsAndFilesSet = new LinkedHashSet<String>(dirsAndFiles.length);
    	for(final String dirOrFile : dirsAndFiles) {
    		
    		// don't ask me why by the array of strings may contain nulls !!
    		if(dirOrFile != null) {
        		try {
        			final File file = new File(dirOrFile);
        			if(file.exists()) {
        				dirsAndFilesSet.add( file.getCanonicalPath() );
        			}
    			} catch (IOException e) {
    				errorMessage.append(
    						"\nCannot resolve path of " + dirOrFile + ". Reason: " + e.getMessage()
    						);
    			}
    		}
    	}
    	
    	dirs = new String[dirsAndFilesSet.size()];
    	
    	dirsAndFilesSet.toArray(this.dirs);
    	
    	if(errorMessage.length() > 0) {
        	throw new IOException();
    	}
    }


    public String[] getDirs() {
        return dirs;
    }
 
    /**
     * @return The Vector of Strings for all valid directories in this model.
     */
    @SuppressWarnings("unchecked")
	public Vector getFileList() {
    	
        // use the LinkedHashSet to get rid of any duplicate from the list of
        // files and directories while keeping the elements original order

        final Set<String> dirsAndFilesSet = new LinkedHashSet<String>();

        for(final String path : dirs) {
        	final File f = new File(path);
        	if(f.exists()) {
            	if(f.isFile()) {
            		// it's a file, just add its canonical path to the set
            		try {
    					dirsAndFilesSet.add(f.getCanonicalPath());
    				} catch (IOException e) {
    					// TODO : this should be handled like in the constructor, but it would require
    					//        changing the signature of the method and thus also the interface ...
    					System.err.println("\nCannot resolve path of " + f + ". Reason: " + e.getMessage());
    				}
            	} else {
            		// it's a directory: add all the content to the set
            		for(final String dirOrFile : f.list()) {
                    	final File file = new File(path + File.separator + dirOrFile);
    					try {
    						dirsAndFilesSet.add(file.getCanonicalPath());
    					} catch (IOException e) {
    						// TODO : this should be handled like in the constructor, but it would require
    						//        changing the signature of the method and thus also the interface ...
    						System.err.println("\nCannot resolve path of " + file + ". Reason: " + e.getMessage());
    					}
            		}
            	}
        	} // if(f.exists())
        }
        
        return new Vector<String>(dirsAndFilesSet);
    }

} // end of class DirectoryFileSelectionModel.

// end of file DirectoryFileSelectionModel.java
