/**
 * class ResultsContainer
 *
 * adds just the bit necessary to 
 * the SortedHashtable, in order to 
 * be used for Tree representation<p>
 * 
 * it is thought as a replacement of 
 * Program and Observation in the
 * DFSFile world.<p>
 *
 * Reference is 'r';
 * Label is 'l';
 * 
 * a char variable is used to keep track of the
 * type of this container<p>
 * 
 * @author: Michele Zamparelli
 * @see: SortedHashtable
 * 
 * TODO: refactor: there is a a dependence on the GUI
 * layer (getTreeCellComponent()) which needs to be factored out. 
 */

package org.eso.gasgano.datamodel.gui;

import java.awt.Color;
import java.util.Enumeration;

import javax.swing.JComponent;
import javax.swing.JLabel;

import org.eso.gasgano.gui.GUIUtils;
import org.eso.gasgano.tools.SortedHashtable;

public class ResultsContainer {
    private ResultsContainer father = null;
    private int filesCount = 0;
    private String desc = null;

    private SortedHashtable frames = null;
    private SortedHashtable labels = null;

    private char containerType = 'a';

    public ResultsContainer() {
        frames = new SortedHashtable();
        labels = new SortedHashtable();
    }

    public ResultsContainer(char type, String label) {
        frames = new SortedHashtable();
        labels = new SortedHashtable();

        setDescription(label);
        containerType = type;
    }

    public void setFather(ResultsContainer f) {
        father = f;
    }

    public ResultsContainer getFather() {
        return father;
    }

    // LABELS

    public int getLabelsCount() {
        return labels.size();
    }

    public ResultsContainer getLabelAt(int i) {
        return (ResultsContainer) (labels.elementAt(i));
    }

    public int getIndexOfLabel(ResultsContainer ref) {
        return labels.indexOf(ref);
    }

    public SortedHashtable getLabelsSet() {
        return labels;
    }

    public ResultsContainer findLabel(String id) {
        return findLabel(labels, id);
    }

    public ResultsContainer findLabel(SortedHashtable progSet, String id) {
        return (ResultsContainer) progSet.get(id);
    }

    public ResultsContainer addLabel(String id) {
        return addLabel(labels, id);
    }

    private ResultsContainer addLabel(SortedHashtable progSet, String id) {
        ResultsContainer rc = findLabel(progSet, id);
        id = id.replace('\t', ' ').trim();
        if (rc == null) {
            rc = new ResultsContainer('l', id);
            progSet.put(id, rc);
            rc.setFather(this);
        }
        return rc;
    }

    public void removeLabel(ResultsContainer label) {
        label.setFather(null);
        labels.remove(label.getId());
    }

    // FRAMES

    public int getFramesCount() {
        return frames.size();
    }

    public ResultsComponent getFrameAt(int i) {
        return (ResultsComponent) (frames.elementAt(i));
    }

    public int getIndexOfFrame(ResultsComponent ref) {
        return frames.indexOf(ref);
    }

    public SortedHashtable getFramesSet() {
        return frames;
    }

    public ResultsComponent findFrame(String id) {
        return findFrame(frames, id);
    }

    public ResultsComponent findFrame(SortedHashtable progSet, String id) {
        return (ResultsComponent) progSet.get(id);
    }

    public void addFrame(String id) {
        addFrame(frames, id);
    }

    private void addFrame(SortedHashtable progSet, String id) {
        id = id.replace('\t', ' ').trim();
        ResultsComponent rc = findFrame(progSet, id);
        if (rc == null) {
            ResultsComponent resComp = new ResultsComponent(id);
            progSet.put(id, resComp);
            //System.out.println(id+" " + resComp.getId() + " " + progSet.indexOf(resComp) );
            resComp.setFather(this);
        }
    }

    public void removeFrame(String id) {
        frames.remove(id);
    }

    // IDENTIFICATION

    public boolean isReference() {
        if (containerType == 'r') {
            return true;
        }
        return false;
    }

    public boolean isLabel() {
        if (containerType == 'l') {
            return true;
        }
        return false;
    }

    public void setDescription(String s) {
        desc = s;
    }

    /**
     * this method is used in particular for the 
     * visualization of the Observation as a JLabel component,<p>
     * therefore it must contain more than the mere id number<p>
     */

    public String toString() {
        return desc;
    }

    /**
     * @return: the identification number for this observation
     */

    public String getId() {
        return super.toString();
    }

    public SortedHashtable getDescendants() {
        SortedHashtable tmpHash = null;

        tmpHash = new SortedHashtable();
        for (Enumeration e = frames.keys(); e.hasMoreElements();) {
            Object tmpObject = e.nextElement();
            tmpHash.put(tmpObject, frames.get(tmpObject));
        }
        for (Enumeration e = labels.keys(); e.hasMoreElements();) {
            Object tmpObject = e.nextElement();
            tmpHash.put(tmpObject, labels.get(tmpObject));
        }
        return tmpHash;
    }

    public JComponent getTreeCellComponent() {
        JLabel cell = new JLabel();
        // Use default colors for now.
        Color foreGround = null;
        cell.setIcon(GUIUtils.defaultLabel.getIcon());

        cell.setText(new String(toString()));
        if (isReference()) {
            cell.setForeground(Color.red);
        } else {
            cell.setForeground(Color.black);
        }
        return cell;
    }
}
