package org.eso.gasgano.dfspipeline;

import java.awt.GridLayout;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;

import javax.swing.JFrame;
import javax.swing.JRadioButton;

import org.eso.gasgano.tools.Exec;

/*
 * This class handles the periodic checking for the existence of processes
 * related to the DFS pipeline infrastructure
 * The  executables filenames are hard-coded and not subject to properties
 * 
 * A number of checks must be made before any member of this class is 
 * instantiated, you have to be on a HP-UX system and all the 
 * variables needed to run the pipeline infrastructure have to be defined and 
 * well defined.
 *
 */

public class InfrastructureWatch extends Thread {

    private String commandOne = null;
    private String doProcess;
    private String rbsProcess;
    int updateSec = 10;
    private Vector outString = null;
    private Vector errString = null;
    
    /**
     * Vector of all DFSPipelineListeners.
     */
    private Vector pipelineListeners = new Vector();

    /**
     * this method executes the specified string in a shell,
       * spawning a process via the RunTime.exec method
       *
       * Unfortunately it relies upon class-wide static 
       * variables for returning output   and error streams
       */

    public InfrastructureWatch() {
        commandOne = new String("");
        if (System.getProperty("os.name").equals("HP-UX")) {
            commandOne = commandOne.concat("/usr/bin/ps -fu ");
        }
        if (System.getProperty("os.name").equals("Solaris")) {
            commandOne = commandOne.concat("/usr/bin/ps -fu ");
        }
        commandOne =
            commandOne.concat(System.getProperty("user.name").toString());

        if (System.getProperty("os.name").equals("HP-UX")) {
            commandOne = commandOne.concat(" -o comm");
        }

        doProcess = new String("pilDataOrganizer");
        rbsProcess = new String("pilScheduler");

        outString = new Vector();
        errString = new Vector();

    }
    
    /**
     * Add a listener to the set of listeners.
     * @param l
     */
    public void addDFSPipelineStatusListener(DFSPipelineStatusListener l) {
        pipelineListeners.add(l);
    }
    
    /**
     * Remove a listener from the set of listeners.
     * @param l
     */
    public void removeDFSPipelineStatusListener(DFSPipelineStatusListener l) {
        pipelineListeners.remove(l);
    }

    /**
     * The periodic infrastrcutre (i.e. filesystem) check 
     * performed to detect new products from the pipeline.
     */
    void fireCheck() {
        boolean doStatus = false;
        boolean rbsStatus = false;
        if (Exec.shellCommand(commandOne, errString, outString) != 0) {
            System.err.println(
                ">> external shell command failed: notify parent class");
            System.err.println(">> command was: " + commandOne);
        }

        for (Enumeration e = outString.elements(); e.hasMoreElements();) {
            String lineTmp = (String) e.nextElement();
            // here all the checks to guarantee that the line we pick up is the right one
            // at the moment the bare grep, to be revised

            if (lineTmp.indexOf(doProcess) != -1) {
                doStatus = true;
            }
            if (lineTmp.indexOf(rbsProcess) != -1) {
                rbsStatus = true;
            }
            for (int i = 0; i < pipelineListeners.size(); i++) {
                ((DFSPipelineStatusListener)pipelineListeners.get(i)).notifyDORunning(doStatus);
                ((DFSPipelineStatusListener)pipelineListeners.get(i)).notifyRBSRunning(rbsStatus);
            }
        }

        for (Enumeration e = errString.elements(); e.hasMoreElements();) {
            System.out.println(e.nextElement());
        }
    }

    /**
     * The run method - sleep and periodically perform fireCheck().
     * @see java.lang.Runnable#run()
     */
    public void run() {
        while (true) {
            try {
                Thread.sleep(updateSec * 1000);
                fireCheck();
            } catch (InterruptedException e) {
                System.err.println("InterruptedException");
            }
        }
    }

    /**
     * Test Harness.
     * @param args
     */
    public static void main(String[] args) {
        Properties Environment = null;
        Enumeration e = null;

        JFrame frame = new JFrame();
        frame.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                System.exit(0);
            }
        });

        JRadioButton RBS_Button = new JRadioButton("RBS");
        RBS_Button.setBorderPainted(false);
        //RBS_Button.setBackground(
        frame.getContentPane().setLayout(new GridLayout(2, 1));
        frame.getContentPane().add(RBS_Button);
        // frame.getContentPane().add(new JCheckBox("RBS"));
        // frame.getContentPane().add(new JToggleButton("RBS"));
        frame.setLocation(200, 200);
        frame.pack();
        frame.setVisible(true);

        // InfrastructureWatch infWatch = new InfrastructureWatch();
        // infWatch.start();
    }
}
