package org.eso.gasgano.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.InputEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.border.EmptyBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableColumn;
import javax.swing.table.TableModel;

import org.eso.gasgano.properties.GasProp;
import org.eso.gasgano.properties.InstGroupTableModel;
import org.eso.gasgano.properties.PropertyTableModel;
import org.eso.gasgano.viewers.ExternalViewerFactory;

/**
 * @author nkornwei
 * 
 * This panel is for configuring the display options (for the filesystem view), printer
 * options, external viewers and the instrument grouping configuration.
 */
public class DisplayPanel extends PropertyEditorSubPanel implements ActionListener {
    private GasProp gpi;
    private JTextField tfitsToViewer;
    private JTextField fitsToViewer;
    private JTextField cubeToViewer;
    private JComboBox tfitsViewerType;
    private JComboBox fitsViewerType;
    private JComboBox cubeViewerType;
    private JTextField printJtf;

    
    private JTable fsDispKeys;

    /*
     * Instrument grouping table, model and buttons. 
     */
    private JTable instrTable;
    private InstGroupTableModel instrTableModel;
    private JButton instTableInsertRow;
    private JButton instTableDeleteRow;

    private KeywordListAdaptor fsDispKeysModel;

    private String fsHeaderLabel =
        new String("Filesystem - Displayed Keywords");
    private JButton insertRow1;
    private JButton deleteRow1;
    private JButton fontChooser;
    private JRadioButton shortName;
    private JRadioButton radecConversion;
    private JRadioButton showExtensions;
    private JRadioButton mjdSorting;

    private JComboBox printOrient;
    private String[] orients = { "landscape", "portrait" };
    private JLabel fontInUse;
    private Font selectedFont;

    /*
     * Is set to non-null if a new font is selected.
     */
    private String newFontProperty = null;

    /*
     * Constructor
     */
    public DisplayPanel(GasProp gp) {

        JPanel jpl;
        JPanel jpr;
        JPanel jpTable1;
        JPanel jplower;
        JPanel jpOptions;
        JPanel jpRadio = new JPanel(new GridLayout(4, 1));
        JPanel jpElse = new JPanel(new BorderLayout());
        JPanel jpLeft = new JPanel(new BorderLayout());
        
        // get the viewer types
        Vector viewerTypes =
            ExternalViewerFactory.getInstance().getViewerTypes();

        printOrient = new JComboBox(orients);
        // JTextFields
        printJtf = new JTextField(12);
        printJtf.addActionListener(this);
        tfitsToViewer = new JTextField(12);
        tfitsToViewer.setToolTipText("Path to viewer command");
        tfitsToViewer.addActionListener(this);
        tfitsViewerType = new JComboBox(viewerTypes);
        fitsToViewer = new JTextField(12);
        fitsToViewer.setToolTipText("Path to viewer command");
        fitsToViewer.addActionListener(this);
        fitsViewerType = new JComboBox(viewerTypes);
        cubeToViewer = new JTextField(12);
        cubeToViewer.setToolTipText("Path to viewer command");
        cubeToViewer.addActionListener(this);
        cubeViewerType = new JComboBox(viewerTypes);
        // JButtons
        insertRow1 = new JButton("Insert Row");
        insertRow1.setToolTipText(
            "Left-click for insert below, right-click for insert above.");
        insertRow1.addMouseListener(new MouseAdapter() {
            public void mousePressed(MouseEvent e) {
                if ((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) {
                    insertBlankRow(fsDispKeys, false);
                }
                if ((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) {
                    insertBlankRow(fsDispKeys, true);
                }
            }

        });
        deleteRow1 = new JButton("Delete Row");
        deleteRow1.setToolTipText("Delete selected row.");
        deleteRow1.addActionListener(this);

        fontChooser = new JButton("Choose new font");
        fontChooser.addActionListener(DisplayPanel.this);

        // JRadioButtons
        shortName = new JRadioButton("truncate filenames", false);
        radecConversion = new JRadioButton("convert RA/DEC", false);
        mjdSorting = new JRadioButton("sort by MJD", false);
        showExtensions = new JRadioButton("show fits extensions", false);
        showExtensions.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent arg0) {
                if (showExtensions.isSelected()) {
                    showWarning(
                        "Display of FITS extension headers is only\n"
                            + "possible when viewing non-compressed FITS files.\n"
                            + "FITS file extension headers for compressed files\n"
                            + "will not be displayed.");
                }
            }
        });

        fontInUse = new JLabel("Font Test");
        fontInUse.setFont(selectedFont);
        
        // code commented for DFS06603
        // --
        // this border was creating problems with the GUI: selecting a big font
        // size (e.g.: 18) caused all controls in the right pane to become
        // bigger and it made the "cube viewer type" combobox disappear !
        // --
        //fontInUse.setBorder(new EmptyBorder(3, 10, 3, 3));
        
        // Table area
        fsDispKeys = new JTable();
        //fsDispKeys.setCellSelectionEnabled(true);
        fsDispKeys.setRowSelectionAllowed(true);
        fsDispKeys.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        fsDispKeys.setPreferredScrollableViewportSize(new Dimension(240, 140));

        /*
         * Set up the instrument grouping table and model
         */
        JPanel instGroupingPanel = setupInstrumentGroupingPanel(gp);

        fsDispKeysModel = new KeywordListAdaptor(gp.fsDisplayColumns, fsHeaderLabel, true);

        fsDispKeys.setModel(fsDispKeysModel);
        fsDispKeys.setDefaultEditor(
            fsDispKeys.getColumnClass(0),
            new PropertyCellEditor());

        loadProperties(gp);

        // JPanels
        jpRadio = new JPanel(new GridLayout(4, 1));
        jpElse = new JPanel(new BorderLayout());
        jpLeft = new JPanel(new BorderLayout());
        jpLeft.setBorder(new TitledBorder("Display Columns and Formats"));

        jpr = new JPanel(new GridLayout(15, 1));
        jpl = new JPanel(new GridLayout(15, 1));
        jpTable1 = new JPanel(new BorderLayout());

        JPanel jpSouth = new JPanel(new BorderLayout());
        JScrollPane scrollPane1 = new JScrollPane(fsDispKeys);
        scrollPane1.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);

        jpOptions = new JPanel(new BorderLayout());
        jpOptions.setBorder(new EmptyBorder(3, 3, 3, 3));

        jpTable1.setBorder(new EmptyBorder(5, 5, 5, 5));
        jpElse.setBorder(new EmptyBorder(5, 5, 5, 5));

        jplower = new JPanel();
        jplower.add(insertRow1);
        jplower.add(deleteRow1);
        jpTable1.add(scrollPane1, BorderLayout.CENTER);
        jpTable1.add(jplower, BorderLayout.SOUTH);

        jpLeft.add(jpTable1, "North");

        // Radio Buttons
        jpRadio.add(shortName);
        jpRadio.add(radecConversion);
        jpRadio.add(mjdSorting);
        jpRadio.add(showExtensions);
        jpRadio.setBorder(new EmptyBorder(5, 5, 5, 5));
        // Left Side
        jpl.add(new JLabel("Printer Settings"));  
        jpl.add(new JLabel("Printer Name:"));
        jpl.add(new JLabel("Printer orientation"));
        jpl.add(new JLabel("Printer Font:"));
        jpl.add(new JLabel(""));
        jpl.add(new JLabel(""));
        jpl.add(new JLabel("External Viewers"));  
        jpl.add(new JLabel("Tfits Viewer:"));
        jpl.add(new JLabel("Tfits Viewer Type:"));
        jpl.add(new JLabel(""));
        jpl.add(new JLabel("Fits Viewer:"));
        jpl.add(new JLabel("Fits Viewer Type:"));
        jpl.add(new JLabel(""));
        jpl.add(new JLabel("Cube Viewer:"));
        jpl.add(new JLabel("Cube Viewer Type:"));
        // Right Side
        jpr.add(new JLabel(""));
        jpr.add(printJtf);
        jpr.add(printOrient);
        jpr.add(fontChooser);
        jpr.add(fontInUse);
        jpr.add(new JLabel(""));
        jpr.add(new JLabel(""));
        jpr.add(
            setupFileChooserPanel(
                tfitsToViewer,
                JFileChooser.FILES_ONLY));
        jpr.add(tfitsViewerType);
        jpr.add(new JLabel(""));
        jpr.add(
            setupFileChooserPanel(
                fitsToViewer,
                JFileChooser.FILES_ONLY));
        jpr.add(fitsViewerType);
        jpr.add(new JLabel(""));
        jpr.add(
            setupFileChooserPanel(
                cubeToViewer,
                JFileChooser.FILES_ONLY));
        jpr.add(cubeViewerType);

        jpOptions.add(jpl, "West");
        jpOptions.add(jpr, "East");

        jpElse.add(jpOptions, "North");
        jpElse.add(jpSouth, "Center");

        TableColumn mColumn = fsDispKeys.getColumn(fsHeaderLabel);
        mColumn.setResizable(false);

        setBorder(new EmptyBorder(3, 3, 3, 3));
        setLayout(new BorderLayout());

        jpLeft.add(jpRadio, "South");
        add(jpElse, BorderLayout.EAST);
        add(jpLeft, BorderLayout.WEST);
        add(instGroupingPanel, BorderLayout.SOUTH);
    }

    /*
     * a method that is used to break up the code in the constructor
     * a little.
     */
    private JPanel setupInstrumentGroupingPanel(GasProp gp) {
        JPanel instTableButtonPanel = new JPanel();
        instrTable = new JTable();
        instrTableModel = new InstGroupTableModel(gp);
        instrTableModel.setDebug(isDebug());
        instrTable.setModel(instrTableModel);
        instrTable.setDefaultEditor(
            instrTable.getColumnClass(1),
            new PropertyCellEditor());
        instrTable.setCellSelectionEnabled(true);
        instrTable.setPreferredScrollableViewportSize(new Dimension(240, 220));
        TableColumn mColumn = instrTable.getColumn("instrument");
        mColumn.setMinWidth(30);
        mColumn.setResizable(false);
        instTableInsertRow = new JButton("Insert Row");
        instTableInsertRow.addActionListener(this);
        instTableDeleteRow = new JButton("Delete Row");
        instTableDeleteRow.addActionListener(this);

        instTableButtonPanel.add(instTableInsertRow);
        instTableButtonPanel.add(instTableDeleteRow);
        String label;
        if (GasProp.isEnvironmentSafe()) {
            label = "Instrument Specific Grouping (filesystem view only)";
        } else {
            label = "Instrument Specific Grouping";
        }

        JPanel instGroupingPanel = new JPanel(new BorderLayout());

        instGroupingPanel.setBorder(new TitledBorder(label));
        JScrollPane scroller = new JScrollPane(instrTable);
        scroller.setPreferredSize(new Dimension(100, 100));
        instGroupingPanel.add(scroller, "Center");
        instGroupingPanel.add(instTableButtonPanel, "South");
        
        return instGroupingPanel;
    }

    public void loadProperties(GasProp gp) {
        gpi = gp;
        // load the file system display keys model.
        fsDispKeysModel.setData(gp.fsDisplayColumns);

        // load the instrument grouping table model.
        instrTable
            .getDefaultEditor(instrTable.getColumnClass(0))
            .cancelCellEditing();
        instrTableModel.loadProperties(gpi);

        // PRINTER AND FONTS
        // utility variables to determine the font
        String fontFull = gpi.getProperty("PRINT_FONT");
        String fontName = fontFull.substring(0, fontFull.indexOf(":", 0));
        String fontSize =
            fontFull.substring(fontFull.indexOf(":", 0) + 1, fontFull.length());
        selectedFont =
            new Font(fontName, Font.PLAIN, Integer.parseInt(fontSize));

        printJtf.setText(gpi.getProperty("PRINTER_NAME"));
        printJtf.addActionListener(this);

        if (gpi.getProperty("PRINT_ORIENTATION").equals("L")) {
            printOrient.setSelectedIndex(0);
        }
        if (gpi.getProperty("PRINT_ORIENTATION").equals("P")) {
            printOrient.setSelectedIndex(1);
        }

        // DISPLAY
        tfitsToViewer.setText(gpi.getProperty("TFITS_TO_VIEWER"));
        tfitsViewerType.setSelectedItem(gpi.getProperty("TFITS_VIEWER_TYPE"));
        fitsToViewer.setText(gpi.getProperty("FITS_TO_VIEWER"));
        fitsViewerType.setSelectedItem(gpi.getProperty("FITS_VIEWER_TYPE"));
        cubeToViewer.setText(gpi.getProperty("CUBE_TO_VIEWER"));
        cubeViewerType.setSelectedItem(gpi.getProperty("CUBE_VIEWER_TYPE"));

        if (gpi.getProperty("SHORT_FILENAME") != null
            && gpi.getProperty("SHORT_FILENAME").equals("true")) {
            shortName.setSelected(true);
        }
        if (gpi.getProperty("RADEC_CONVERSION") != null
            && gpi.getProperty("RADEC_CONVERSION").equals("true")) {
            radecConversion.setSelected(true);
        }
        if (gpi.getProperty("MJD_SORTING") != null
            && gpi.getProperty("MJD_SORTING").equals("true")) {
            mjdSorting.setSelected(true);
        }
        if (gpi.getProperty(GasProp.SHOW_EXTENSIONS_KEY) != null
            && gpi.getProperty(GasProp.SHOW_EXTENSIONS_KEY).equals("true")) {
            showExtensions.setSelected(true);
        }
    }

    /*
     * Save the properties from the GUI to the properties database.
     * 
     * @return String null on success, non-null on error - the returned
     * String is the error message.
     */
    public String saveProperties() {
        String rc = null;
        GUIUtils.stopEditing(fsDispKeys);
        for (int i = 0; i < gpi.fsDisplayColumns.size(); i++) {
            if (gpi.fsDisplayColumns.elementAt(i).toString().equals("")
                || gpi.fsDisplayColumns.elementAt(i).toString().equals(
                    "*INVALID*")
                || gpi.fsDisplayColumns.elementAt(i) == null) {
                gpi.fsDisplayColumns.removeElementAt(i);
            }
        }

        if (shortName.isSelected()) {
            gpi.put("SHORT_FILENAME", "true");
        } else {
            gpi.put("SHORT_FILENAME", "false");
        }
        if (radecConversion.isSelected()) {
            gpi.put("RADEC_CONVERSION", "true");
        } else {
            gpi.put("RADEC_CONVERSION", "false");
        }

        if (mjdSorting.isSelected()) {
            gpi.put("MJD_SORTING", "true");
        } else {
            gpi.put("MJD_SORTING", "false");
        }

        if (showExtensions.isSelected()) {
            gpi.put("SHOW_EXTENSIONS", "true");
        } else {
            gpi.put("SHOW_EXTENSIONS", "false");
        }
        String viewer;
        gpi.put("TFITS_TO_VIEWER", tfitsToViewer.getText());
        viewer = (String) tfitsViewerType.getSelectedItem();
        if (viewer == null) {
            viewer = "";
        }
        gpi.put("TFITS_VIEWER_TYPE", viewer);
        gpi.put("FITS_TO_VIEWER", fitsToViewer.getText());
        viewer = (String) fitsViewerType.getSelectedItem();
        if (viewer == null) {
            viewer = "";
        }
        gpi.put("FITS_VIEWER_TYPE", viewer);
        gpi.put("CUBE_TO_VIEWER", cubeToViewer.getText());
        viewer = (String) cubeViewerType.getSelectedItem();
        if (viewer == null) {
            viewer = "";
        }
        gpi.put("CUBE_VIEWER_TYPE", viewer);
        gpi.put("PRINTER_NAME", printJtf.getText());
        gpi.put(
            "PRINT_ORIENTATION",
            (printOrient.getSelectedIndex() == 0) ? "L" : "P");
        selectedFont = null;
        if (newFontProperty != null) {
            gpi.put("PRINT_FONT", newFontProperty);
            newFontProperty = null;
        }
        return rc;
    }

    /**
     * this method is called before flushing to check
     * whether the instrument grouping has changed<p>
     * if it has, the user will be prompted to accept
     * immediate termination of the application<p>
     * @return: false no action needed, true no action needed.
     */
    public int restartRequired() {
        instrTable
            .getDefaultEditor(instrTable.getColumnClass(0))
            .stopCellEditing();
        for (int i = 0; i < gpi.instruments.size(); i++) {
            if (((String) gpi.instruments.elementAt(i)).equals("")) {
                gpi.instruments.removeElementAt(i);
                gpi.instrumentGrouping.removeElementAt(i);
                ((PropertyTableModel) instrTable.getModel())
                    .fireTableDataChanged();
            }
        }

        if (gpi.instrumentGrouping.size()
            != GasProp.getProperties().instrumentGrouping.size()) {
            return RELOAD_ON_SAVE;
        }
        for (int i = 0; i < gpi.instrumentGrouping.size(); i++) {
            if (!gpi
                .instrumentGrouping
                .elementAt(i)
                .equals(
                    GasProp.getProperties().instrumentGrouping.elementAt(i))) {
                return RELOAD_ON_SAVE;
            }
        }

        if ((mjdSorting.isSelected()
            && GasProp.getProperties().getProperty("MJD_SORTING").equals("false"))
            || (!mjdSorting.isSelected()
                && GasProp.getProperties().getProperty("MJD_SORTING").equals(
                    "true"))) {
            return RELOAD_ON_SAVE;
        }
        return NOACTION_ON_SAVE;
    }

    void fireFontChooser() {
        FontChooser fc = new FontChooser(selectedFont);
        if (JOptionPane
            .showConfirmDialog(
                this,
                fc,
                "Edit fonts",
                JOptionPane.OK_CANCEL_OPTION,
                JOptionPane.PLAIN_MESSAGE)
            == JOptionPane.OK_OPTION) {
            if (isDebug()) {
                System.out.println(
                    "selected Font is:"
                        + fc.getNewFont().getName()
                        + ":"
                        + fc.getNewFont().getSize());
            }
            newFontProperty =
                fc.getNewFont().getName() + ":" + fc.getNewFont().getSize();
            fontInUse.setFont(fc.getNewFont());
            selectedFont = fc.getNewFont();
        }
    }

    /**
     * Insert a row into the table. Assumes that the table model is
     * either a ListAdaptor or PropertyTableModel. If the table model
     * does not match either of these, then nothing happens.
     * TODO: same method appears in MenuPanel - needs refactoring.
     * @param table
     * @param after
     */
    private void insertBlankRow(final JTable table, boolean after) {
        GUIUtils.stopEditing(table);
        int selRow;
        if (table.getSelectedRow() < 0) {
            // insert at beginning or append to end as no specific row was selected.
            selRow = after ? table.getRowCount() - 1 : 0;
        } else {
            selRow = table.getSelectedRow();
        }
        if (isDebug()) {
            System.out.println("Insert selected row: " + selRow+" after: "+after);
        }
        if (after && table.getRowCount() != 0) {
            TableModel mdl = table.getModel();
            if (mdl instanceof ListAdaptor) {
                ((ListAdaptor)mdl).insertRow(selRow+1, "");
            } else if (mdl instanceof PropertyTableModel) {
                ((PropertyTableModel)mdl).insertRow(selRow+1);
            }
            final int selRowFinal = selRow;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    table.clearSelection();
                    table.setRowSelectionInterval(selRowFinal + 1, selRowFinal + 1);
                }
            });
            
        } else {
            TableModel mdl = table.getModel();
            if (mdl.getRowCount() == 0) {
                selRow = 0;
            }
            if (mdl instanceof ListAdaptor) {
                ((ListAdaptor)mdl).insertRow(selRow, "");
            } else if (mdl instanceof PropertyTableModel) {
                ((PropertyTableModel)mdl).insertRow(selRow);
            }
            final int selRowFinal = selRow;
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    table.clearSelection();
                    table.setRowSelectionInterval(selRowFinal, selRowFinal);
                }
            });

        }
    }
    
    /**
     * Delete a row into the table. Assumes that the table model is
     * either a ListAdaptor or PropertyTableModel. If the table model
     * does not match either of these, then nothing happens.
     * TODO: same method appears in MenuPanel - needs refactoring.
     */
    private void deleteSelectedRow(final JTable table) {
        GUIUtils.stopEditing(table);
        final int selRow = table.getSelectedRow();
        if (selRow < 0) {
            return;
        }
        if (isDebug()) {
            System.out.println("Delete selected row: " + selRow);
        }
        TableModel mdl = table.getModel();
        if (mdl instanceof ListAdaptor) {
            ((ListAdaptor)mdl).removeRow(selRow);
        } else if (mdl instanceof PropertyTableModel) {
            ((PropertyTableModel)mdl).removeRow(selRow);
        }
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                table.clearSelection();
                int row;
                if (table.getRowCount() <= selRow) {
                    row = table.getRowCount()-1;
                } else {
                    row = selRow;
                }
                if (row >= 0) {
                    table.setRowSelectionInterval(row, row);
                }
            }
        });
    }

    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == deleteRow1) {
            deleteSelectedRow(fsDispKeys);
        } else if (e.getSource() == printJtf) {
            gpi.put("PRINTER_NAME", printJtf.getText());
        } else if (e.getSource() == fontChooser) {
            fireFontChooser();
        } else if (e.getSource() == instTableDeleteRow) {
            deleteSelectedRow(instrTable);
        } else if (e.getSource() == instTableInsertRow) {
            insertBlankRow(instrTable, true);
        }
    }

    private void showWarning(String warning) {
        JOptionPane.showMessageDialog(
            this,
            warning,
            "Warning",
            JOptionPane.OK_OPTION);
    }
}

