/**
 * @author N. Kornweibel
 * @version 1.0
 */

package org.eso.gasgano.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.Enumeration;
import java.util.Vector;

import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ListCellRenderer;
import javax.swing.border.EmptyBorder;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

public class FileListPane
    extends JTabbedPane
    implements ListSelectionListener {

    private class MyCellRenderer
        extends JTextField
        implements ListCellRenderer {
        Font font = null;
        Color backColor = null;
        public MyCellRenderer(Color back) {
            setBorder(new EmptyBorder(0, 0, 0, 0));
            backColor = back;
        }

        public Component getListCellRendererComponent(
            JList list,
            Object value,
        // value to display
        int index, // cell index
        boolean isSelected, // is the cell selected
        boolean cellHasFocus) // the list and the cell have the focus
        {
            String s = value.toString();
            font = getFont();
            setFont(new Font(font.getName(), Font.PLAIN, font.getSize()));
            setBackground(Color.white);

            if (chosenFew[fileLists.indexOf(list)].contains(value)) {
                setFont(new Font(font.getName(), Font.BOLD, font.getSize()));
            }
            if (isSelected) {
                setBackground(backColor);
            }
            setText(s);
            return this;
        }
    }

    private Vector selectionListeners = null;
    private Vector fileLists = null;
    private boolean clearSelection = false;
    private Vector[] chosenFew = null;

    public FileListPane() {
        selectionListeners = new Vector();
        fileLists = new Vector();
        chosenFew = new Vector[4];
        for (int i = 0; i < 4; i++) {
            chosenFew[i] = new Vector();
        }
    }

    public void addList(FileListModel m) {
        JList newList = new JList(m);
        newList.addListSelectionListener(this);
        newList.setCellRenderer(
            new MyCellRenderer(newList.getSelectionBackground()));
        fileLists.addElement(newList);
        add(new JScrollPane(newList), m.getLabel());
    }

    public void removeList(FileListModel m) {
        for (int i = fileLists.size() - 1; i >= 0; i--) {
            if (((JList) fileLists.elementAt(i)).getModel() == m) {
                fileLists.removeElementAt(i);
                removeTabAt(i);
            }
        }
    }

    public void emptyLists() {
        Enumeration e = fileLists.elements();
        while (e.hasMoreElements()) {
            JList jl = (JList) e.nextElement();
            ((FileListModel) jl.getModel()).emptyList();
        }
    }

    public FileListModel getListModel(int listIndex) {
        JList jl = (JList) fileLists.elementAt(listIndex);
        return (FileListModel) jl.getModel();
    }

    public void setListModel(FileListModel model, int index) {
        JList jl = (JList) fileLists.elementAt(index);
        jl.setModel(model);
    }

    /**
     * @return Vector of JList components.
     */
    public Vector getLists() {
        return fileLists;
    }

    // implement the ListSelectionListener interface.
    public void valueChanged(ListSelectionEvent evt) {
        if (clearSelection) {
            return;
        }
        // clear the selection from the other lists.
        Object src = evt.getSource();
        boolean fileSelected = false;
        Enumeration e = fileLists.elements();
        clearSelection = true;
        while (e.hasMoreElements()) {
            JList jl = (JList) e.nextElement();
            if (src != jl) {
                jl.clearSelection();
            } else {
                // if the list is empty, it means the "empty message" was
                // selected - see org.eso.gasgano.FileListModel.
                fileSelected = !(((FileListModel) jl.getModel()).isEmpty());
            }
        }
        clearSelection = false;
        // pass the event onto the ListSelectionListeners if the list
        // selected was not empty.
        if (fileSelected) {
            e = selectionListeners.elements();
            while (e.hasMoreElements()) {
                ListSelectionListener lsl =
                    (ListSelectionListener) e.nextElement();
                lsl.valueChanged(evt);
            }
        }
    }

    public void addListSelectionListener(ListSelectionListener l) {
        if (!selectionListeners.contains(l)) {
            selectionListeners.addElement(l);
        }
    }

    public void removeListSelectionListener(ListSelectionListener l) {
        selectionListeners.removeElement(l);
    }

    public void setActiveElements(Vector[] chosenFew) {
        for (int i = 0; i < 4; i++) {
            this.chosenFew[i] = chosenFew[i];
        }
    }

} // end of class FileListPane.
