package org.eso.gasgano.gui;

import java.util.Vector;

import javax.swing.tree.TreePath;

/**
 * @author Nick Kornweibel
 * 
 * Class for iterating a Tree Model.
 * It is used when searching the model for a specific node and performs
 * a depth first traversal of the tree model.
 */
public class TreeModelIterator {
    
    /**
     * The model to iterate
     */
    private TreeTableModel tm = null;
    /**
     * The current path
     */
    private Vector currentPath = null;
    
    /**
     * has nextPath() been called.
     */
    private boolean hasIterated = false;

    /**
     * Create the iterator, setting the model and the start path.
     * If start is null, the root node is taken as the start path.
     * @param t
     * @param start
     */
    public TreeModelIterator(TreeTableModel t, TreePath start) {
        tm = t;
        currentPath = new Vector();
        if (start == null || start.getPathCount() <= 0) {
            currentPath.addElement(tm.getRoot());
            
        } else {
            Object[] startPath = start.getPath();
            for (int i = 0; i < startPath.length; i++) {
                currentPath.addElement(startPath[i]);
            }
        }
    }

    /**
     * Get the current path. This method can be called after each
     * call to iteratePath(), to get the next path in the model iteration.
     * @return Vector the current path, in a Vector of nodes.
     */
    public Vector getCurrentPath() {
        return currentPath;
    }

    /**
     * Get the next tree path, iterating in a depth first manner.
     */
    public void iteratePath() {
        int pathLength = currentPath.size();
        Object me = currentPath.elementAt(pathLength - 1);

        if (pathLength > 1) {
            Object myParent = currentPath.elementAt(pathLength - 2);
            int myIndex = tm.getIndexOfChild(myParent, me);

            if (myIndex + 1 < tm.getChildCount(myParent)) {
                // I have siblings...replace myself with next in line

                Object parent = tm.getChild(myParent, myIndex + 1);
                currentPath.setElementAt(parent, pathLength - 1);
                while (tm.getChildCount(parent) > 0) {
                    parent = tm.getChild(parent, 0);
                    currentPath.addElement(parent);
                }
            } else {
                // go up to my parents
                currentPath.removeElementAt(pathLength - 1);
            }
        } else {
            // I am the root node, go to my children:
            Object parent = me;
            while (tm.getChildCount(parent) > 0) {
                parent = tm.getChild(parent, 0);
                currentPath.addElement(parent);
            }
        }
    }
}
