package org.eso.gasgano.properties;

/**
 * @author nkornwei
 *
 * Singleton model class for providing proerties to across the Gasgano applciation.
 */
public class PropertyDB {

    private static PropertyDB instance;
    
    private PropertySource source;
    
    /**
     * current debug/verbose level
     */
    private int debugLevel = 0;
    
    // debug levels
    public static final int debugDataModelChange = 1;
    public static final int debugDataModelEvent = 2;
    public static final int debugExternalCommands = 3;
    public static final int debugBaseName = 4;
    public static final int debugTiming = 5;
    public static final int debugEvent = 6;
    public static final int debugExpressionAndRules = 7;
    public static final int debugDatabase = 8;
    public static final int debugAll = 9;
    
    /**
     * Constructor for PropertyDB.
     */
    private PropertyDB() {
        super();
    }
    
    /**
     * Get the singleton instance
     * @return PropertyDB the instance of PropertyDB.
     */
    public static PropertyDB getInstance() {
        if (instance == null) {
            instance = new PropertyDB();
        }
        return instance;
    }
    
    /**
     * Set the source for all property data.
     * @param s
     */
    public void setPropertySource(PropertySource s) {
        source = s;
    }
    
    /**
     * Get a property from the source, or null if not found.
     * @param name of property ot fetch from source.
     * @return String value of property
     */
    public String getProperty(String name) {
        String rc = null;
        if (source != null) {
            rc = source.getProperty(name);
        }
        return rc;
    }
    
    /**
     * Set a property to the source, does nothing if source is null.
     * @param name of property to set (non-null)
     * @param val value of property to set (non-null)
     */
    public void setProperty(String name, String val) {
        if (source != null) {
            source.setProperty(name, val);
        }
    }
    
    public boolean isDebug(int specDebug) {
        boolean rc = false;
        if (getDebugLevel() == specDebug || getDebugLevel() == debugAll) {
            rc = true;
        }
        return rc;
    }
    
    public int getDebugLevel() {
        return debugLevel;
    }
    
    public void setDebugLevel(int l) {
        debugLevel = l;
    }
}
