/**
 * @author N. Kornweibel
 * @version 1.0
 * A hashtable which allows the elementAt() call as per a Vector - the
 * element returned is taken for a sorted list of the elements in the
 * Hashtable.
 * The list can be given a name (in the constructor) if required.
 */

package org.eso.gasgano.tools;

import java.util.Dictionary;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;

import org.eso.gasgano.datamodel.gui.KeywordContainerOBComponent;
import org.eso.gasgano.properties.PropertyDB;

public class SortedHashtable extends Dictionary {
    
    private Hashtable hTable = null;
    private Vector sortedList = null;
    private String title = null;
    private boolean mjdsort;

    public SortedHashtable() {
        hTable = new Hashtable();
    }
  
    public SortedHashtable(boolean mjdsort) {
        hTable = new Hashtable();
	this.mjdsort = mjdsort;
    }
  
    public SortedHashtable(String label) {
        hTable = new Hashtable();
        title = label;
    }

    public synchronized boolean contains(Object value) {
        return hTable.contains(value);
    }

    public synchronized boolean containsKey(Object key) {
        return hTable.containsKey(key);
    }
  
    public synchronized Object get(Object key) {
        return hTable.get(key);
    }
    
    public synchronized Object put(Object key, Object value) {
        destroySortedList();
        return hTable.put(key, value);
    }
  
    public synchronized Object remove(Object key) {
        destroySortedList();
        return hTable.remove(key);
    }

    public synchronized Enumeration elements() {
        return hTable.elements();
    }

    public synchronized Enumeration keys() {
        return hTable.keys();
    }

    public synchronized void clear() {
        destroySortedList();
        hTable.clear();
    }

    public int size() {
        return hTable.size();
    }

    public boolean isEmpty() {
        return hTable.isEmpty();
    }

    public synchronized String toString() {
        String rc = title;
        if (rc == null) {
            rc = hTable.toString();
        }
        return rc;
    }

    public synchronized Object elementAt(int index) {
        this.buildSortedList();
        return sortedList.elementAt(index);
    }

    public int indexOf(Object elem) {
        buildSortedList();
        return sortedList.indexOf(elem);
    }

    private void buildSortedList() {
        if (sortedList == null) {
            sortedList = new Vector(hTable.size());
            int numInserted = 0;
            int i;
            Enumeration e = hTable.elements();
            while (e.hasMoreElements()) {
                Object insertee = e.nextElement();
                String insertVal = insertee.toString();
                for (i = 0; i < numInserted; i++) {
	      if ( mjdsort && PropertyDB.getInstance().getProperty("MJD_SORTING").equals("true") )  { 
		  if ( !( insertee instanceof  KeywordContainerOBComponent)) continue;
		  if ( insertee instanceof KeywordContainerOBComponent 
		       && sortedList.elementAt(i) instanceof KeywordContainerOBComponent) {
		      if ( ((KeywordContainerOBComponent)insertee).precedes(  ((KeywordContainerOBComponent)sortedList.elementAt( i ))  ) )  {
			  break;
		      }
		  }
	      } else {                    
		  if (insertVal.compareTo(sortedList.elementAt(i).toString()) <= 0) {
		        break;
		  }
	      }
                }
                sortedList.insertElementAt(insertee, i);
                numInserted++;
            }
        }
    }

    private void destroySortedList() {
        sortedList = null;
    }
}
// end of file SortedHashtable.java.
