/**
 * @author N. Kornweibel
 * @version 1.0
 * An interface for report generation.
 *
 * Column Width information can be requested from the implementing class
 * once the report is generated.
 *
 * The report is generated with a generateReport() call, where a
 * PrintWriter object is expected.
 *
 * All other errors than IOExceptions are reported in the report body.
 * If all goes well, true is returned, else false.
 *
 * As the Report is run as a seperate thread, the interrupt() method
 * allows ReportGeneration to be interrupted, with resources being
 * deallocated and so forth.
 */
package org.eso.gasgano.tools;

import java.io.PrintWriter;

public class TestReport extends Report {

    public int[] colWidths = null;
    private int rows = 250;
    private int cols = 15;
    PrintWriter printer = null;

    public TestReport() {
        colWidths = new int[cols];
        for (int i = 0; i < cols; i++) {
            colWidths[i] = i+1;
        }
    }
    public int[] getColumnWidths() {
        return colWidths;
    }

    public void setPrintWriter(PrintWriter p) {
        printer = p;
    }

    public boolean generateReport() {
        printer.println("\nTest Report\n");
        for (int i = 0; i < rows; i++) {
            for (int x = 0; x < cols; x++) {
                int rep = x+1;
                if (x > 0) {
                    printer.print("\t");
                }
                while (rep > 0) {
                    printer.print(--rep);
                }
            }
            printer.println("");
        }
        return true;
    }
}  // end of interface Report.
// end of file Report.java.
