package org.eso.cpl;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Represents the level that a {@link Frame} has.
 * <p>
 * This corresponds to the <tt>cpl_frame_level</tt> enum in the
 * <tt>cpl_frame.h</tt> header file.  I don't know anywhere its sematics
 * are documented.
 * <p>
 * This class exemplifies the typesafe enum pattern.
 *
 * @author   Mark Taylor
 */
public class FrameLevel {

    private final String name_;
    private final String idName_;

    /**
     * Constructs a new FrameLevel object.
     * This private constructor is only used to create static final members
     * of this class.
     *
     * @param  name  the public name of this object
     * @param  idName  the name of the <tt>cpl_frame_level</tt> enum member
     *         to which it corresponds (may be used by native code)
     */
    private FrameLevel(String name, String idName) {
        name_ = name;
        idName_ = idName;
    }

    public String getName() {
        return name_;
    }

    public String toString() {
        return getName();
    }

    public static final FrameLevel TEMPORARY = 
        new FrameLevel("TEMPORARY", "CPL_FRAME_LEVEL_TEMPORARY");
    public static final FrameLevel INTERMEDIATE =
        new FrameLevel("INTERMEDIATE", "CPL_FRAME_LEVEL_INTERMEDIATE");
    public static final FrameLevel FINAL = 
        new FrameLevel("FINAL", "CPL_FRAME_LEVEL_FINAL");

    /** Unmodifiable list containing all the instances of this class.  */
    public static final List ALL_LEVELS =
        Collections.unmodifiableList(Arrays.asList(new FrameLevel[] {
            TEMPORARY, INTERMEDIATE, FINAL,
        }));

}
