package org.eso.cpl;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;

/**
 * Represents the type of a {@link Frame}.
 * <p>
 * This corresponds to the <tt>cpl_frame_type</tt> enum in the
 * <tt>cpl_frame.h</tt> header file.  I don't know anywhere its semantics
 * are documented.
 * <p>
 * This class exemplifies the typesafe enum pattern.
 * 
 * @author   Mark Taylor
 */
public class FrameType {

    private final String name_;
    private final String idName_;

    /**
     * Constructs a new FrameType object.
     * This private constructor is only used to create static final members
     * of this class.
     *
     * @param  name  the public name of this object
     * @param   idName  the name of the <tt>cpl_frame_type</tt> enum member
     *          to which it corresponds (may be used by native code)
     */
    private FrameType(String name, String idName) {
        name_ = name;
        idName_ = idName;
    }

    public String getName() {
        return name_;
    }

    public String toString() {
        return getName();
    }

    public static final FrameType IMAGE = 
        new FrameType("IMAGE", "CPL_FRAME_TYPE_IMAGE");
    public static final FrameType MATRIX = 
        new FrameType("MATRIX", "CPL_FRAME_TYPE_MATRIX");
    public static final FrameType TABLE =
        new FrameType("TABLE", "CPL_FRAME_TYPE_TABLE");

    /** Unmodifiable list containing all the instances of this class.  */
    public static final List ALL_TYPES =
        Collections.unmodifiableList(Arrays.asList(new FrameType[] {
            IMAGE, MATRIX, TABLE,
        }));
}
