package org.eso.cpl;

/**
 * Interface for restrictions on the permissible values of a parameter. 
 *
 * @author   Mark Taylor
 */
public interface ParameterConstraint {

    /**
     * Checks that a value is allowed.  If it is, no action is taken;
     * if it is not, an exception is thrown.
     *
     * @param  val  the value to check
     * @throws  ParameterValueException  if <tt>val</tt> violates this
     *          constraint
     */
    void checkValue(Object val) throws ParameterValueException;

    /**
     * Returns a description of this constraint.  It should be ideally
     * be a short summary suitable for presentation to a user.
     *
     * @return  short textual description of this constraint
     */
    String getDescription();
}
