package org.eso.cpl;

/**
 * Defines the services which an implementation must provide to
 * a {@link Parameter} object.
 * The bridge pattern is used here to clarify the role played by the
 * implementation.  As well as providing metadata about the parameter
 * (name, help information, constraints) and its default value, 
 * class handles transmitting a new validated value to the parameter list.
 * It does not deal with parameter validation or maintaining the current
 * value however.
 *
 * @author   Mark Taylor
 */
public interface ParameterImp {
    String getContext();
    String getHelp();
    String getName();
    String getTag();
    ParameterType getType();
    ParameterConstraint getConstraint();

    /**
     * Returns the initial (default) value for this parameter.
     * The returned value must meet the requirements defined by
     * the {@link #getType} and {@link #getConstraint}.
     *
     * @return  default value
     */
    Object getInitialValue();

    /**
     * Sets a new value for this parameter from a given object.
     * This method may message the associated recipe's parameter list
     * so that it is aware of the parameter's new value.
     * For implementations with no native peer, this method may not
     * need to do anything.
     * This method does not need to concern itself with validation;
     * callers must ensure that <tt>val</tt> meets the requirements
     * defined by {@link #getType} and {@link #getConstraint}.
     *
     * @param  val  the new value for this parameter
     */
    void setValue(Object val);
}
