package org.eso.cpl.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

/**
 * Utility class for displaying an error dialog to the user.
 *
 * @author   Mark Taylor
 */
public class ErrorDialog {

    /**
     * Pops up a modal dialog to the user displaying a <tt>Throwable</tt> 
     * object.  As well as a short message, the throwable's stack trace
     * is displayed in a scrollable window.  The user has to click on 
     * a button or something to dismiss it.
     *
     * @param  th  the Throwable object to display
     * @param  message  a short text message to display in the dialog
     *         (may be <tt>null</tt>). The throwable message is
     *         appeneded to this message to create the full error message.
     * @param  parent  a parent window, used for positioning the dialog
     *         (may be null)
     */
    public static void showError(final Throwable th, String message,
                                 final Component parent) {

        final String fullMessage;
        if (message == null) {
            fullMessage = th.getMessage();
        } else {
            fullMessage = message+": "+th.getMessage();
        }
        // String title = th.getClass().getName();
        final String title = "Error";
        JButton detailsButton = new JButton(new AbstractAction("Details...") {
            public void actionPerformed(ActionEvent e) {

                /* Get the stack trace as a string. */
                StringWriter traceWriter = new StringWriter();
                th.printStackTrace(new PrintWriter(traceWriter));
                String trace = traceWriter.toString();
 
                /* Add the full stack trace in a scrollable window. */
                JTextArea ta = new JTextArea();
                ta.setLineWrap(false);
                ta.setEditable(false);
                ta.append(trace);
                ta.setCaretPosition(0);
                JScrollPane sp = new JScrollPane();
                sp.setMaximumSize(new Dimension(500, 300));
                sp.setPreferredSize(new Dimension(500, 200));
                sp.setViewportView(ta);

                /* Construct the stack of objects which will form the body of 
                 * the window. */
                Object[] content = new Object[] {fullMessage, sp};

                /* Present the whole lot to the user in a JOptionPane. */
                JOptionPane.showMessageDialog(parent, content, title, 
                                              JOptionPane.ERROR_MESSAGE);
            }
        });

        /* Construct the stack of objects which will form the body of 
         * the window.
         */
        Object[] content = new Object[] {fullMessage, detailsButton};

        /* Present the whole lot to the user in a JOptionPane. 
         * The message with a "details" button for showing the stack trace.
         */
        JOptionPane.showMessageDialog(parent, content, title, 
                                      JOptionPane.ERROR_MESSAGE);
    }
}
