package org.eso.cpl.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.dnd.DropTarget;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.border.Border;

/**
 * Utility class containing miscellaneous static methods for GUI operations.
 *
 * @author   Mark Taylor, Nick Kornweibel
 */
class GUIUtils {

    /** 
     * Prepare table for use.
     *  
     * @param  table table to configure 
     */                                       
    static void configureTable(JTable table) {
        table.setRowSelectionAllowed(false);
        table.setColumnSelectionAllowed(false);
        table.setCellSelectionEnabled(false);
    }
    
    /**
     * Set up the table to handle FileList drag/drop operations. This
     * will only work for JTables with FrameTableModel models.
     * 
     * @param table
     */
    static void enableFileListDragDrop(JTable table) {
        table.setDragEnabled(true);
        table.setTransferHandler(new FrameTableModelTransferHandler());
    }
     
    /**
     * Set up the table to handle FileList drag/drop operations. This
     * will only work for JTables with FrameTableModel models.
     * 
     * @param table
     */
    static void enableFileListDragDrop(JScrollPane scroller) {
        scroller.setTransferHandler(new FrameTableModelTransferHandler());
    }
     
    /**
     * Wraps a component in a JScrollPane in a uniform way and performs
     * some cosmetic configuration on it.
     * 
     * @param  comp  the component to wrap
     * @param  title  the text to appear in the border title
     * @return  the wrapped component
     */ 
    static JScrollPane wrapInScroller(JComponent comp) {
        JScrollPane scroller = new JScrollPane(comp);
        Color bg = scroller.getViewport().getView().getBackground();
        scroller.getViewport().setBackground(bg);
        scroller.setPreferredSize(new Dimension(700, 140));
        return scroller;
    }

    /**
     * Sets the border of a component to include a title.
     * The component is returned as a convenience. 
     * 
     * @param  comp  the component whose border is to be set
     * @param  title  the text to appear in the border title
     * @return  the re-bordered component
     */
    static JComponent wrapInBorder(JComponent comp, String title) {
        Border baseBorder =
            BorderFactory.createLineBorder(Color.black); 
        Border titleBorder =
            BorderFactory.createTitledBorder(baseBorder, title);
        Border gapBorder = 
            BorderFactory.createEmptyBorder(0, 1, 0, 0);
        Border addedBorder =
            BorderFactory.createCompoundBorder(gapBorder, titleBorder);
        Border oldBorder = comp.getBorder(); 
        Border newBorder =
            BorderFactory.createCompoundBorder(addedBorder, oldBorder);
        comp.setBorder(newBorder);
        return comp;
    }
}
