package org.eso.cpl.gui;

import java.awt.Component;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JComponent;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 * A stack of check boxes, one for each item which can be saved/loaded
 * when serializing/deserializing a Request object.
 *
 * The Recipe selection is commented out and a CPLFrame should not change the
 * recipe it is associated with after loading.
 * 
 * The reason for that is given in the CPLFrame header comment.
 * 
 * @author   Mark Taylor
 */
class RequestItemSelector extends JComponent implements ChangeListener {

    /** Checkbox indicating whether the Recipe is selected. */
    //final JCheckBox recipeBox_ = new JCheckBox("Recipe");

    /** Checkbox indicating whether the Parameter list is selected. */
    final JCheckBox parametersBox_ = new JCheckBox("Parameter values");

    /** Checkbox indicating whether the Calibration frames are selected. */
    final JCheckBox calibFramesBox_ = new JCheckBox("Calibration frames");

    /** Checkbox indicating whether the Raw frames are selected. */
    final JCheckBox rawFramesBox_ = new JCheckBox("Raw frames");

    /** Checkbox indicating whether the product root is selected. */
    final JCheckBox productRootBox_ = new JCheckBox("Product root");

    /** Checkbox indicating whether the product naming scheme is selected. */
    final JCheckBox namingSchemeBox_ = new JCheckBox("Naming scheme");

    /**
     * Constructs a new default selector component.
     */
    public RequestItemSelector() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));

        // Set default states for the buttons.
        //recipeBox_.setSelected(true);
        parametersBox_.setSelected(true);
        calibFramesBox_.setSelected(true);
        rawFramesBox_.setSelected(true);
        productRootBox_.setSelected(true);
        namingSchemeBox_.setSelected(true);

        // Install this object to listen for state changes.
        //recipeBox_.addChangeListener(this);
        
        // Add the check boxes to this component.
        //add(recipeBox_);
        add(parametersBox_);
        add(calibFramesBox_);
        add(rawFramesBox_);
        add(productRootBox_);
        add(namingSchemeBox_);
    }

    /**
     * Reacts when some of the check boxes are been checked/unchecked,
     * to ensure that the states and enablednesses are consistent.
     */
    public void stateChanged(ChangeEvent evt) {
        /*
        Object source = evt.getSource();
        if (source == recipeBox_) {
            if (recipeBox_.isSelected()) {
                parametersBox_.setSelected(true);
                parametersBox_.setEnabled(true);
            }
            else {
                parametersBox_.setSelected(false);
                parametersBox_.setEnabled(false);
            }
        }
        */
    }

    /**
     * Pops up a dialog window containing this selector and allows the
     * user to make selections from it.
     *
     * @param  parent    the component which controls the dialog
     * @param  title     the title of the dialog window
     * @param  textLines lines of explanatory text to be presented in the dialog
     * @return  JOptionPane.OK_OPTION or JOptionPane.CANCEL_OPTION
     */
    public int showDialog(Component parent, String title, String[] textLines) {
        JComponent box = new Box(BoxLayout.Y_AXIS);
        if (textLines != null && textLines.length > 0) {
            for (int i = 0; i < textLines.length; i++) {
                box.add(new JLabel(textLines[i]));
            }
            box.add(Box.createVerticalStrut(10));
        }
        box.add(this);
        return JOptionPane.showConfirmDialog(parent, box, title,
                                             JOptionPane.OK_CANCEL_OPTION,
                                             JOptionPane.QUESTION_MESSAGE);
    }
}
