package org.eso.cpl.gui;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.DefaultListModel;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.ListCellRenderer;
import javax.swing.ListModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.eso.cpl.Request;

/**
 * Provides a visual display for a {@link RequestPool} object.
 * The RequestPool is the model for the view provided by this object.
 * 
 * @author   Mark Taylor
 */
public class RequestPoolDisplay extends JList {

    private RequestPool pool;
    /**
     * Construct a new display from a given request queue.
     *
     * @param  queue the request queue to display
     */
    public RequestPoolDisplay(RequestPool p) {
        super(p.items_);
        pool = p;
        String deleteLabel = "DELETE_CURRENT_REQUEST";
        getActionMap().put(deleteLabel, new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                removeSelectedRequest();
            }
        });
        getInputMap().put(
            KeyStroke.getKeyStroke(KeyEvent.VK_DELETE, 0),
            deleteLabel);
            

        addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                int[] selection = getSelectedIndices();
                pool.setRequestSelection(selection);                
            }});
            
        setCellRenderer(new RequestCellRenderer());
    }

    /**
     * Returns a scrollpane which wraps this object.
     */
    public JScrollPane wrapInScroller() {
        Box hbox = new Box(BoxLayout.X_AXIS);
        hbox.add(this);
        hbox.add(Box.createHorizontalGlue());
        Box vbox = new Box(BoxLayout.Y_AXIS);
        vbox.add(Box.createVerticalGlue());
        vbox.add(hbox);
        JScrollPane scroller = new JScrollPane(vbox) {
            public Dimension getPreferredSize() {
                Dimension d = super.getPreferredSize();
                d.width = 128;
                return d;
            }
            public Dimension getMaximumSize() {
                return new Dimension(
                    getPreferredSize().width,
                    super.getMaximumSize().height);
            }
        };
        scroller.setVerticalScrollBarPolicy(
            JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scroller.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        scroller.getViewport().setBackground(getBackground());
        return scroller;
    }

    /**
     * Remove the request that is selected in the queue
     */
    public void removeSelectedRequest() {
        removeRequest(getSelectedIndex());
    }

    /**
     * Remove the specified request from the queue.
     * Does nothing if the index is out of range.
     * @param row
     */
    public void removeRequest(int row) {
        ListModel mdl = getModel();
        if (row >= 0 && row < mdl.getSize()) {
            if (mdl instanceof DefaultListModel) {
                // can only remove from a DefaultListModel
                 ((DefaultListModel) mdl).remove(row);
            }
        }
    }

    static class RequestCellRenderer
        extends JLabel
        implements ListCellRenderer {
        final static Icon okIcon = Icons.EXECOK;
        final static Icon errIcon = Icons.EXECERR;
        final static Icon emptyIcon = Icons.EXECBLANK;

        public Component getListCellRendererComponent(
            JList list, Object value, int index, boolean isSelected,
            boolean cellHasFocus) {

            String text = value.toString(); 
            Icon image = emptyIcon;
            if (value instanceof Request) {
                Request r = (Request)value;
                if (r.getState() == Request.SUCCESS) {
                    image = okIcon;
                } else if (r.getState() == Request.FAILURE) {
                    image = errIcon;
                }
            }
            setText(text);
            setIcon(image);
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            setEnabled(list.isEnabled());
            setFont(list.getFont());
            setOpaque(true);
            return this;
        }
    }

}