package org.eso.cpl.gui;

import javax.swing.DefaultListModel;
import org.eso.cpl.Request;

/**
 * Keeps track of a queue-like (FIFO) structure containing a number of 
 * {@link Request} objects.
 * 
 *
 * @author   Mark Taylor
 */
public class RequestQueue {

    final DefaultListModel items_ = new DefaultListModel();

    /**
     * Constructs a new request queue.
     */
    public RequestQueue() {
    }

    /**
     * Indicates whether any objects remain in this queue.
     *
     * @return  true iff the <tt>next</tt> method will return a request
     */
    public boolean hasNext() {
        return items_.size() > 0;
    }

    /**
     * Removes the object at the head of the queue and returns it.
     * If the queue is empty, returns <tt>null</tt>.
     *
     * @return   the next request in the queue
     */
    public Request next() {
        try {
            return (Request) items_.remove(items_.size() - 1);
        }
        catch (ArrayIndexOutOfBoundsException e) {
            return null;
        }
    }

    /**
     * Returns the number of items in this queue.
     *
     * @return   number of requests
     */
    public int size() {
        return items_.size();
    }

    /**
     * Adds a new request to the end of the queue.
     *
     * @param req  request to add
     */
    public void add(Request req) {
        items_.add(0, req);
    }

    /**
     * Removes all items from the queue.
     */
    public void clear() {
        items_.removeAllElements();
    }
}
