package org.eso.cpl.gui;

import java.awt.Component;
import java.awt.Point;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

/**
 * Listener which forwards events to any cell contents in a JTable 
 * which are themselves Component objects (note, not that the 
 * renderers are).
 * <p>
 * This code mostly pinched from
 * <a href="http://www.fawcette.com/Archives/premier/mgznarch/javapro/1999/jp_jan_99/ap0199/ap0199.asp">a JavaPro article</a>.
 *
 * @author   Daniel Saverese
 * @author   Mark Taylor
 */
class TableComponentMouseListener implements MouseListener {

    public void mouseClicked(MouseEvent evt) {
        forwardEvent(evt);
    }
    public void mousePressed(MouseEvent evt) {
        forwardEvent(evt);
    }
    public void mouseReleased(MouseEvent evt) {
        forwardEvent(evt);
    }

    public void mouseEntered(MouseEvent evt) {
        forwardEvent(evt);
    }
    public void mouseExited(MouseEvent evt) {
        forwardEvent(evt);
    }

    private void forwardEvent(MouseEvent evt) {
        Object src = evt.getSource();
        if (! (src instanceof JTable)) {
            return;
        }
        JTable table = (JTable) src;
        Point point = evt.getPoint();
        int icol = table.columnAtPoint(point);
        int irow = table.rowAtPoint(point);
        if (icol >= 0 && icol < table.getColumnCount() &&
            irow >= 0 && irow < table.getRowCount()) {
            Object value = table.getValueAt(irow, icol);
            if (value instanceof Component) {
                Component comp = (Component) value;
                MouseEvent cEvt = SwingUtilities
                                 .convertMouseEvent(table, evt, comp);
                comp.dispatchEvent(cEvt);
            }
        }
        // table.repaint();
    }
}
