package org.eso.cpl.test;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eso.cpl.EnumConstraint;
import org.eso.cpl.ParameterConstraint;
import org.eso.cpl.ParameterType;
import org.eso.cpl.ParameterValueException;
import org.eso.cpl.RangeConstraint;

/**
 * Tests ParameterConstraint implementations.
 * <p>
 * $Id: ConstraintTest.java,v 1.2 2007/02/20 13:37:10 dsforna Exp $
 * @author   Mark Taylor
 */
public class ConstraintTest extends TestCase {

    public ConstraintTest(String name) {
        super(name);
    }

    public void testEnumConstraint() throws ParameterValueException {
        String[] options = new String[] {"cow", "pig", "mouse"};
        List optList = Arrays.asList(options);
        EnumConstraint cons1 = new EnumConstraint(options);
        EnumConstraint cons2 = new EnumConstraint(optList);
        assertEquals(new ArrayList(optList), 
                     new ArrayList(cons1.getOptions()));
        assertNotNull(cons1.getDescription());

        assertEquals(new ArrayList(cons1.getOptions()),
                     new ArrayList(cons2.getOptions()));
        assertEquals(cons1.getDescription(), cons2.getDescription());

        goodValue(cons1, "pig");
        goodValue(cons1, "cow");
        goodValue(cons1, "mouse");
        badValue(cons1, "sheep");
    }

    public void testRangeConstraint() throws ParameterValueException {
        int imin = 23;
        int imax = 801;
        Object min = new Integer(imin);
        Object max = new Integer(imax);
        RangeConstraint cons = new RangeConstraint(ParameterType.INT, min, max);
        assertEquals(min, cons.getMinimum());
        assertEquals(max, cons.getMaximum());
        badValue(cons, new Integer(imin - 1));
        goodValue(cons, new Integer(imin));
        goodValue(cons, new Integer(imin + 1));
        goodValue(cons, new Integer(imax - 1));
        goodValue(cons, new Integer(imax));
        badValue(cons, new Integer(imax + 1));
    }

    private void goodValue(ParameterConstraint cons, Object value)
            throws ParameterValueException {
        cons.checkValue(value);
    }

    private void badValue(ParameterConstraint cons, Object value) {
        try {
            cons.checkValue(value);
            fail();
        }
        catch (ParameterValueException e) {
        }
    }


    public static TestSuite suite() {
        return new TestSuite(ConstraintTest.class);
    }

    public static void main(String[] args) {
        TestRunner.run(suite());
    }
}
