package org.eso.cpl.test;

import java.io.File;
import java.util.Iterator;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eso.cpl.Frame;
import org.eso.cpl.FrameGroup;
import org.eso.cpl.FrameLevel;
import org.eso.cpl.FrameList;
import org.eso.cpl.FrameType;

/**
 * Tests the FrameList class.
 * <p>
 * $Id: FrameListTest.java,v 1.3 2007/02/20 13:37:10 dsforna Exp $
 * @author   Mark Taylor
 */
public class FrameListTest extends TestCase {

    private Frame f1;
    private Frame f2;
    private Frame f3;
    private FrameList flist;

    public FrameListTest(String name) {
        super(name);
    }

    public void setUp() {
        f1 = new Frame(new File("f1.fits"));
        f2 = new Frame(new File("f2.fits"));
        f3 = new Frame(new File("f3.fits"));
        flist = new FrameList(new Frame[] {f1, f2, f3});
    }

    public void testEquality() {
        FrameList flb = new FrameList();
        assertTrue(!flist.equals(flb));
        flb.add(f2);
        flb.add(0, f1);
        flb.add(new Frame(f3.getFile()));
        assertEquals(flist, flb);

        f2.setLevel(FrameLevel.FINAL);
        assertEquals(flist, flb);
        f3.setGroup(FrameGroup.PRODUCT);
        assertTrue(!flist.equals(flb));
    }

    public void testSelections() {
        assertEquals(0, flist.selectByGroup(FrameGroup.RAW).size());
        assertEquals(0, flist.selectByLevel(FrameLevel.FINAL).size());
        assertEquals(0, flist.selectByType(FrameType.IMAGE).size());
        assertEquals(0, flist.selectByTag("GutenTag").size());
        assertEquals(0, flist.selectByTag("").size());

        assertEquals(flist, flist.selectByGroup(null));
        assertEquals(flist, flist.selectByLevel(null));
        assertEquals(flist, flist.selectByType(null));
        assertEquals(flist, flist.selectByTag(null));
  
        FrameList some = new FrameList();
        exerciseSelections(flist, some);
        some.add(f1);
        exerciseSelections(flist, some);
        some.add(f2);
        exerciseSelections(flist, some);
        some.add(f3);
    }

    private void exerciseSelections(FrameList all, FrameList some) {
        for (Iterator it = some.iterator(); it.hasNext();) {
            Frame frm = (Frame) it.next();
            frm.setGroup(FrameGroup.RAW);
            frm.setLevel(FrameLevel.FINAL);
            frm.setType(FrameType.IMAGE);
            frm.setTag("GutenTag");
        }
        assertEquals(some, all.selectByGroup(FrameGroup.RAW));
        assertEquals(some, all.selectByLevel(FrameLevel.FINAL));
        assertEquals(some, all.selectByType(FrameType.IMAGE));
        assertEquals(some, all.selectByTag("GutenTag"));
    }

    public static TestSuite suite() {
        return new TestSuite(FrameListTest.class);
    }

    public static void main(String[] args) {
        TestRunner.run(suite());
    }
}
