package org.eso.cpl.test;

import java.io.File;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eso.cpl.Frame;
import org.eso.cpl.FrameGroup;
import org.eso.cpl.FrameLevel;
import org.eso.cpl.FrameType;

/**
 * Test the Frame class.
 * <p>
 * $Id: FrameTest.java,v 1.4 2007/02/20 13:37:10 dsforna Exp $
 * @author   Mark Taylor
 */
public class FrameTest extends TestCase {

    public FrameTest(String name) {
        super(name);
    }

    public void testFrame() {
        Frame f1 = new Frame(new File("f1.fits"));
        Frame f2 = new Frame(new File("f2.fits"));
        different(f1, f2);

        f1.setFile(new File("same.fits"));
        f2.setFile(new File("./same.fits"));
        same(f1, f2);

        f1.setTag("testing");
        different(f1, f2);
        f2.setTag("testing");
        same(f1, f2);

        f1.setLevel(FrameLevel.INTERMEDIATE);
        different(f1, f2);
        f2.setLevel(FrameLevel.TEMPORARY);
        different(f1, f2);
        f2.setLevel(FrameLevel.INTERMEDIATE);
        same(f1, f1);

        f1.setGroup(FrameGroup.RAW);
        different(f1, f2);
        f2.setGroup(FrameGroup.PRODUCT);
        different(f1, f2);
        f2.setGroup(FrameGroup.RAW);
        same(f1, f2);

        f1.setType(FrameType.MATRIX);
        different(f1, f2);
        f2.setType(FrameType.TABLE);
        different(f1, f2);
        f2.setType(FrameType.MATRIX);
        same(f1, f2);
    }

    public void testFrameGroup() {
        assertEquals("CALIB", FrameGroup.CALIB.getName());
        assertEquals("PRODUCT", FrameGroup.PRODUCT.getName());
        assertEquals("RAW", FrameGroup.RAW.getName());
    }

    public void testFrameLevel() {
        assertEquals("FINAL", FrameLevel.FINAL.getName());
        assertEquals("INTERMEDIATE", FrameLevel.INTERMEDIATE.getName());
        assertEquals("TEMPORARY", FrameLevel.TEMPORARY.getName());
    }

    public void testFrameType() {
        assertEquals("IMAGE", FrameType.IMAGE.getName());
        assertEquals("MATRIX", FrameType.MATRIX.getName());
        assertEquals("TABLE", FrameType.TABLE.getName());
    }

    private static void same(Frame f1, Frame f2) {
        assertTrue(f1.equals(f2));
        assertTrue(f1.hashCode() == f2.hashCode());
        checkClone(f1);
    }

    private static void different(Frame f1, Frame f2) {
        assertTrue(! f1.equals(f2));
        assertTrue("hashcodes match - not necessarily wrong, but probably is",
                   (! (f1.hashCode() == f2.hashCode()))); 
        checkClone(f1);
        checkClone(f2);
    }

    private static void checkClone(Frame f) {
        assertEquals(f, new Frame(f));
    }

    public static TestSuite suite() {
        return new TestSuite(FrameTest.class);
    }

    public static void main(String[] args) {
        TestRunner.run(suite());
    }
}
