package org.eso.cpl.test;

import junit.framework.TestCase;
import junit.framework.TestSuite;
import junit.textui.TestRunner;

import org.eso.cpl.CPLException;
import org.eso.cpl.Recipe;
import org.eso.cpl.jni.JNIRecipe;
import org.eso.cpl.jni.LTDLException;
import org.eso.cpl.jni.LibraryLoader;
import org.eso.cpl.jni.OtherPlugin;
import org.eso.cpl.jni.PluginLibrary;

/**
 * Tests the PluginLibrary class based on a dummy plugin library.
 * <p>
 * $Id: PluginLibraryTest.java,v 1.7 2008/10/01 15:49:11 mpadovan Exp $
 * @author   Mark Taylor
 */
public class PluginLibraryTest extends TestCase {

    public static final String DLL_NAME = "libtestplugins.so";

    private PluginLibrary pluglib;
    private Recipe[] recipes;
    private OtherPlugin[] others;
 
    public PluginLibraryTest(String name) {
        super(name);
    }

    public void setUp() throws LTDLException, CPLException {
        pluglib = getTestLibrary();
        recipes = pluglib.getRecipes();
        others = (OtherPlugin[]) pluglib.getOtherItems();
    }

    public static PluginLibrary getTestLibrary() throws LTDLException, CPLException {
        LibraryLoader loader = new LibraryLoader();
        loader.getPath().add(".");
        return loader.getLibrary(DLL_NAME);
    }

    public void testLibrary() {
        assertEquals(JNIRecipe.class, recipes.getClass().getComponentType());
    }


    public void testPlugins() {
        assertEquals("Fails without a patched cpl_parameterlist.c", 
                     2, recipes.length);
        assertEquals(4, others.length);
        assertTrue(others[0].getClass().getName().endsWith("NonRecipePlugin"));
        assertTrue(others[1].getClass().getName().endsWith("NonRecipePlugin"));
        assertTrue(others[2].getClass().getName().endsWith("BadAPIPlugin"));
        assertTrue(others[3].getClass().getName().endsWith("BadAPIPlugin"));
    }

    public static TestSuite suite() {
        return new TestSuite(PluginLibraryTest.class);
    }

    public static void main(String[] args) {
        TestRunner.run(suite());
    }
}
